@@ -20,11 +20,17 @@
 import sys
 import os
 import xml.dom.minidom
-from copy import deepcopy
 
 
-# SET CONFING FOLDER
-# ------------------
+# SET VERSION
+# -----------
+
+version = '3.11.0'
+nightbuild = ''
+
+
+# SET CONFIG FOLDER
+# -----------------
 
 if sys.platform == 'darwin':
     confdir = 'configs'
@@ -53,49 +59,27 @@ if not os.path.exists(confdir):
     raise IOError, "Configuration folder cannot be found!"
 
 
-# LOAD mspy MODULE
-# ----------------
-
-import blocks
-
-if not os.path.exists(os.path.join(confdir,'elements.xml')):
-    blocks.saveElements(os.path.join(confdir,'elements.xml'))
-if not os.path.exists(os.path.join(confdir,'aminoacids.xml')):
-    blocks.saveAminoacids(os.path.join(confdir,'aminoacids.xml'))
-if not os.path.exists(os.path.join(confdir,'enzymes.xml')):
-    blocks.saveEnzymes(os.path.join(confdir,'enzymes.xml'))
-if not os.path.exists(os.path.join(confdir,'fragments.xml')):
-    blocks.saveFragments(os.path.join(confdir,'fragments.xml'))
-if not os.path.exists(os.path.join(confdir,'modifications.xml')):
-    blocks.saveModifications(os.path.join(confdir,'modifications.xml'))
-
-import mspy
-
-
-# DEFAULT CONFIG VALUES
-# ---------------------
-
-import libs
-
-version = '3.10.0'
-testing = False
+# INIT DEFAULT VALUES
+# -------------------
 
 internal={
     'canvasXrange': None,
 }
 
 main={
-    'appWidth': 1000,
-    'appHeight': 550,
+    'appWidth': 1043,
+    'appHeight': 620,
     'appMaximized': 0,
     'unlockGUI': 0,
     'mzDigits': 4,
     'intDigits': 0,
     'ppmDigits': 1,
     'lastDir': '',
+    'lastSeqDir': '',
     'errorUnits': 'Da',
     'printQuality': 5,
     'macListCtrlGeneric': 1,
+    'peaklistColumns': ['mz', 'int', 'rel', 'sn', 'z', 'fwhm', 'resol'],
     'donate': '',
     'updatesEnabled': 1,
     'updatesChecked': '',
@@ -103,7 +87,7 @@ main={
     'updatesAvailable': version,
     'compassMode': 'Profile',
     'compassFormat': 'mzData',
-    'compassDeleteFile': 0,
+    'compassDeleteFile': 1,
 }
 
 recent=[]
@@ -155,12 +139,11 @@ spectrum={
     'yLabel': 'a.i.',
     'showGrid': 1,
     'showLegend': 1,
-    'showPosBar': 1,
-    'showIntBar': 1,
+    'showPosBars': 1,
     'showGel': 1,
     'showGelLegend': 1,
     'showTracker': 1,
-    'showNotationMarks': 1,
+    'showNotations': 1,
     'showLabels': 1,
     'showAllLabels': 1,
     'showTicks': 1,
@@ -172,8 +155,6 @@ spectrum={
     'normalize': 0,
     'overlapLabels': 0,
     'checkLimits': 1,
-    'mzDigits': 4,
-    'intDigits': 1,
     'labelAngle': 90,
     'labelCharge': 1,
     'labelBgr': 1,
@@ -182,6 +163,10 @@ spectrum={
     'tickColour': [255,75,75],
     'tmpSpectrumColour': [255,0,0],
     'notationMarksColour': [0,255,0],
+    'notationMaxLength': 40,
+    'notationMarks': 1,
+    'notationLabels': 0,
+    'notationMZ': 0,
 }
 
 match={
@@ -206,9 +191,8 @@ processing={
         'highMass': 5000,
     },
     'baseline':{
-        'segments': 15,
-        'offset': 0,
-        'smooth': 1,
+        'precision': 15,
+        'offset': 0.25,
     },
     'smoothing':{
         'method': 'SG',
@@ -216,11 +200,11 @@ processing={
         'cycles': 2,
     },
     'peakpicking':{
-        'snThreshold': 3,
+        'snThreshold': 3.0,
         'absIntThreshold': 0,
-        'relIntThreshold': 0,
-        'adaptiveNoise': 0,
+        'relIntThreshold': 0.0,
         'pickingHeight': 0.75,
+        'baseline': 1,
         'smoothing': 1,
         'deisotoping': 1,
         'monoisotopic': 0,
@@ -230,8 +214,11 @@ processing={
         'maxCharge': 1,
         'massTolerance': 0.1,
         'intTolerance': 0.5,
+        'isotopeShift': 0.0,
         'removeIsotopes': 1,
         'removeUnknown': 1,
+        'labelEnvelope': 'monoisotopic',
+        'setAsMonoisotopic': 0,
     },
 }
 
@@ -263,7 +250,7 @@ sequence={
         'fragments': ['a','b','y','-NH3','-H2O'],
         'filterFragments': 1,
         'listFormat': 'b.S.a [m]',
-        'matchFormat': '[f] [r] b.S.a [m]',
+        'matchFormat': 'f [r] [m]',
     },
     'search':{
         'mass': 0,
@@ -271,6 +258,7 @@ sequence={
         'charge': 1,
         'massType': 0,
         'enzyme': 'Trypsin',
+        'semiSpecific': True,
         'tolerance': 0.2,
         'units': 'Da',
         'retainPos': 0,
@@ -312,6 +300,14 @@ compare={
     'compare': 'peaklists',
 }
 
+generator={
+    'fwhm': 0.1,
+    'points': 10,
+    'noise': 0,
+    'showPeaks': 1,
+    'showOverlay': 0,
+}
+
 mascot={
     'common':{
         'title':'',
@@ -464,6 +460,7 @@ prospector={
 links={
     'mMassHomepage': 'http://www.mmass.org/',
     'mMassForum': 'http://forum.mmass.org/',
+    'mMassTwitter':  'http://www.twitter.com/mmassorg/',
     'mMassCite': 'http://www.mmass.org/home/donate.php',
     'mMassDonate': 'http://www.mmass.org/home/donate.php',
     'mMassDownload': 'http://www.mmass.org/download/',
@@ -487,25 +484,49 @@ links={
 }
 
 replacements={
-    'PubChemC':{
-        'pattern': 'CID([0-9]{1,10})',
-        'url': 'http://pubchem.ncbi.nlm.nih.gov/summary/summary.cgi?cid=%s',
+    'sequences':{
+        'general':{
+            'pattern': '^([A-Z0-9_]+[\.0-9]*)$',
+            'url': 'http://www.ncbi.nlm.nih.gov/protein/%s',
+        },
+        'gi':{
+            'pattern': '^gi\|?([0-9]+[\.0-9]*)$',
+            'url': 'http://www.ncbi.nlm.nih.gov/protein/%s',
+        },
+        'gb':{
+            'pattern': '^gb\|?([A-Z]{3}[0-9]{5}[\.0-9]*)$',
+            'url': 'http://www.ncbi.nlm.nih.gov/protein/%s',
+        },
+        'sp':{
+            'pattern': '^sp\|?([A-Z][A-Z0-9]+)$',
+            'url': 'http://www.uniprot.org/uniprot/%s',
+        },
+        'ref':{
+            'pattern': '^ref\|?([A-Z]{2}_[0-9]+[\.0-9]*)$',
+            'url': 'http://www.ncbi.nlm.nih.gov/protein/%s',
+        },
     },
-    'LipidMaps':{
-        'pattern': '(LM[A-Z]{2}[0-9]{4}[0-9A-Z]{2}[0-9]{2})',
-        'url': 'http://www.lipidmaps.org/data/LMSDRecord.php?LMID=%s',
-    },
-    'NORINE':{
-        'pattern': '(NOR[0-9]{5})',
-        'url': 'http://bioinfo.lifl.fr/norine/result.jsp?ID=%s',
+    'compounds':{
+        'PubChemC':{
+            'pattern': 'CID([0-9]{1,10})',
+            'url': 'http://pubchem.ncbi.nlm.nih.gov/summary/summary.cgi?cid=%s',
+        },
+        'LipidMaps':{
+            'pattern': '(LM[A-Z]{2}[0-9]{4}[0-9A-Z]{2}[0-9]{2})',
+            'url': 'http://www.lipidmaps.org/data/LMSDRecord.php?LMID=%s',
+        },
+        'NORINE':{
+            'pattern': '(NOR[0-9]{5})',
+            'url': 'http://bioinfo.lifl.fr/norine/result.jsp?ID=%s',
+        },
     },
 }
 
 
-# LOAD FUNCTIONS
-# --------------
+# LOAD AND SAVE CONFIG FILE
+# -------------------------
 
-def loadConfig(path=os.path.join(confdir,'config.xml')):
+def loadConfig(path=os.path.join(confdir, 'config.xml')):
     """Parse config XML and get data."""
     
     # parse XML
@@ -515,6 +536,9 @@ def loadConfig(path=os.path.join(confdir,'config.xml')):
     mainTags = document.getElementsByTagName('main')
     if mainTags:
         _getParams(mainTags[0], main)
+        
+        if type(main['peaklistColumns']) != list:
+            main['peaklistColumns'] = main['peaklistColumns'].split(';')
     
     # recent files
     recentTags = document.getElementsByTagName('recent')
@@ -613,12 +637,12 @@ def loadConfig(path=os.path.join(confdir,'config.xml')):
         if type(sequence['fragment']['fragments']) != list:
             sequence['fragment']['fragments'] = sequence['fragment']['fragments'].split(';')
     
-    # masscalc
+    # mass calculator
     masscalcTags = document.getElementsByTagName('masscalc')
     if masscalcTags:
         _getParams(masscalcTags[0], masscalc)
     
-    # compounds
+    # compounds search
     compoundsTags = document.getElementsByTagName('compounds')
     if compoundsTags:
         _getParams(compoundsTags[0], compounds)
@@ -626,16 +650,21 @@ def loadConfig(path=os.path.join(confdir,'config.xml')):
         if type(compounds['adducts']) != list:
             compounds['adducts'] = compounds['adducts'].split(';')
     
-    # differences
+    # peak differences
     differencesTags = document.getElementsByTagName('differences')
     if differencesTags:
         _getParams(differencesTags[0], differences)
     
-    # compare
+    # compare peaklists
     compareTags = document.getElementsByTagName('compare')
     if compareTags:
         _getParams(compareTags[0], compare)
     
+    # spectrum generator
+    generatorTags = document.getElementsByTagName('generator')
+    if generatorTags:
+        _getParams(generatorTags[0], generator)
+    
     # mascot
     mascotTags = document.getElementsByTagName('mascot')
     if mascotTags:
@@ -701,213 +730,12 @@ def loadConfig(path=os.path.join(confdir,'config.xml')):
         for linkTag in linkTags:
             name = linkTag.getAttribute('name')
             value = linkTag.getAttribute('value')
-            if name not in ('mMassHomepage', 'mMassCite', 'mMassDonate', 'mMassDownload'):
+            if name not in ('mMassHomepage', 'mMassForum', 'mMassTwitter', 'mMassCite', 'mMassDonate', 'mMassDownload'):
                 links[name] = value
 # ----
 
 
-def loadPresets(path=os.path.join(confdir,'presets.xml')):
-    """Parse processing presets XML and get data."""
-    
-    # parse XML
-    document = xml.dom.minidom.parse(path)
-    
-    # get operator presets
-    operatorTags = document.getElementsByTagName('operator')
-    if operatorTags:
-        presetsTags = operatorTags[0].getElementsByTagName('presets')
-        if presetsTags:
-            libs.presets['operator'].clear()
-            
-            for presetsTag in presetsTags:
-                name = presetsTag.getAttribute('name')
-                libs.presets['operator'][name] = {'operator':'', 'contact':'', 'institution':'', 'instrument':''}
-                _getParams(presetsTag, libs.presets['operator'][name])
-    
-    # get processing presets
-    processingTags = document.getElementsByTagName('processing')
-    if processingTags:
-        presetsTags = processingTags[0].getElementsByTagName('presets')
-        if presetsTags:
-            libs.presets['processing'].clear()
-            
-            for presetsTag in presetsTags:
-                name = presetsTag.getAttribute('name')
-                libs.presets['processing'][name] = deepcopy(processing)
-                
-                cropTags = presetsTag.getElementsByTagName('crop')
-                if cropTags:
-                    _getParams(cropTags[0], libs.presets['processing'][name]['crop'])
-                
-                baselineTags = presetsTag.getElementsByTagName('baseline')
-                if baselineTags:
-                    _getParams(baselineTags[0], libs.presets['processing'][name]['baseline'])
-                
-                smoothingTags = presetsTag.getElementsByTagName('smoothing')
-                if smoothingTags:
-                    _getParams(smoothingTags[0], libs.presets['processing'][name]['smoothing'])
-                
-                peakpickingTags = presetsTag.getElementsByTagName('peakpicking')
-                if peakpickingTags:
-                    _getParams(peakpickingTags[0], libs.presets['processing'][name]['peakpicking'])
-                
-                deisotopingTags = presetsTag.getElementsByTagName('deisotoping')
-                if deisotopingTags:
-                    _getParams(deisotopingTags[0], libs.presets['processing'][name]['deisotoping'])
-    
-    # get modifications presets
-    modificationsTags = document.getElementsByTagName('modifications')
-    if modificationsTags:
-        presetsTags = modificationsTags[0].getElementsByTagName('presets')
-        if presetsTags:
-            libs.presets['modifications'].clear()
-            
-            for presetsTag in presetsTags:
-                name = presetsTag.getAttribute('name')
-                
-                modifications = []
-                modificationTags = presetsTag.getElementsByTagName('modification')
-                for modificationTag in modificationTags:
-                    modName = modificationTag.getAttribute('name')
-                    modPosition = modificationTag.getAttribute('position')
-                    modType = modificationTag.getAttribute('type')
-                    modifications.append([modName, modPosition, modType])
-                
-                libs.presets['modifications'][name] = modifications
-    
-    # get fragments presets
-    fragmentsTags = document.getElementsByTagName('fragments')
-    if fragmentsTags:
-        presetsTags = fragmentsTags[0].getElementsByTagName('presets')
-        if presetsTags:
-            libs.presets['fragments'].clear()
-            
-            for presetsTag in presetsTags:
-                name = presetsTag.getAttribute('name')
-                
-                fragments = []
-                fragmentTags = presetsTag.getElementsByTagName('fragment')
-                for fragmentTag in fragmentTags:
-                    fragName = fragmentTag.getAttribute('name')
-                    fragments.append(fragName)
-                
-                libs.presets['fragments'][name] = fragments
-# ----
-
-
-def loadReferences(path=os.path.join(confdir,'references.xml')):
-    """Parse calibration references XML and get data."""
-    
-    # parse XML
-    document = xml.dom.minidom.parse(path)
-    
-    # get references
-    groupTags = document.getElementsByTagName('group')
-    if groupTags:
-        libs.references.clear()
-        for groupTag in groupTags:
-            groupName = groupTag.getAttribute('name')
-            libs.references[groupName] = []
-            
-            referenceTags = groupTag.getElementsByTagName('reference')
-            if referenceTags:
-                for referenceTag in referenceTags:
-                    name = referenceTag.getAttribute('name')
-                    mass = referenceTag.getAttribute('mass')
-                    libs.references[groupName].append((name, float(mass)))
-# ----
-
-
-def loadCompounds(path=os.path.join(confdir,'compounds.xml')):
-    """Parse compounds XML and get data."""
-    
-    # parse XML
-    document = xml.dom.minidom.parse(path)
-    
-    # get references
-    groupTags = document.getElementsByTagName('group')
-    if groupTags:
-        libs.compounds.clear()
-        for groupTag in groupTags:
-            groupName = groupTag.getAttribute('name')
-            libs.compounds[groupName] = {}
-            
-            compoundTags = groupTag.getElementsByTagName('compound')
-            if compoundTags:
-                for compoundTag in compoundTags:
-                    try:
-                        name = compoundTag.getAttribute('name')
-                        compound = mspy.compound(compoundTag.getAttribute('formula'))
-                        compound.description = _getNodeText(compoundTag)
-                        libs.compounds[groupName][name] = compound
-                    except:
-                        pass
-# ----
-
-
-def loadMascot(path=os.path.join(confdir,'mascot.xml')):
-    """Parse mascot servers XML and get data."""
-    
-    # parse XML
-    document = xml.dom.minidom.parse(path)
-    
-    # get references
-    serverTags = document.getElementsByTagName('server')
-    if serverTags:
-        libs.mascot.clear()
-        for serverTag in serverTags:
-            name = serverTag.getAttribute('name')
-            libs.mascot[name] = {}
-            
-            libs.mascot[name] = {
-                'host': '',
-                'path': '/',
-                'search': 'cgi/nph-mascot.exe',
-                'results': 'cgi/master_results.pl',
-                'export': 'cgi/export_dat_2.pl',
-                'params': 'cgi/get_params.pl',
-            }
-            _getParams(serverTag, libs.mascot[name])
-# ----
-
-
-def _getParams(sectionTag, section):
-    """Get params from nodes."""
-    
-    if sectionTag:
-        paramTags = sectionTag.getElementsByTagName('param')
-        if paramTags:
-            if paramTags:
-                for paramTag in paramTags:
-                    name = paramTag.getAttribute('name')
-                    value = paramTag.getAttribute('value')
-                    valueType = paramTag.getAttribute('type')
-                    if name in section:
-                        if valueType in ('unicode', 'str', 'float', 'int'):
-                            try:
-                                section[name] = eval(valueType+'(value)')
-                            except:
-                                pass
-# ----
-
-
-def _getNodeText(node):
-    """Get text from node list."""
-    
-    buff = ''
-    for node in node.childNodes:
-        if node.nodeType == node.TEXT_NODE:
-            buff += node.data
-    
-    return buff
-# ----
-
-
-
-# SAVE FUNCTIONS
-# --------------
-
-def saveConfig(path=os.path.join(confdir,'config.xml')):
+def saveConfig(path=os.path.join(confdir, 'config.xml')):
     """Make and save config XML."""
     
     buff = '<?xml version="1.0" encoding="utf-8" ?>\n'
@@ -920,10 +748,12 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '    <param name="appMaximized" value="%d" type="int" />\n' % (bool(main['appMaximized']))
     buff += '    <param name="unlockGUI" value="%d" type="int" />\n' % (bool(main['unlockGUI']))
     buff += '    <param name="macListCtrlGeneric" value="%d" type="int" />\n' % (bool(main['macListCtrlGeneric']))
+    buff += '    <param name="peaklistColumns" value="%s" type="str" />\n' % (';'.join(main['peaklistColumns']))
     buff += '    <param name="mzDigits" value="%d" type="int" />\n' % (main['mzDigits'])
     buff += '    <param name="intDigits" value="%d" type="int" />\n' % (main['intDigits'])
     buff += '    <param name="ppmDigits" value="%d" type="int" />\n' % (main['ppmDigits'])
     buff += '    <param name="lastDir" value="%s" type="unicode" />\n' % (_escape(main['lastDir']))
+    buff += '    <param name="lastSeqDir" value="%s" type="unicode" />\n' % (_escape(main['lastSeqDir']))
     buff += '    <param name="errorUnits" value="%s" type="str" />\n' % (main['errorUnits'])
     buff += '    <param name="printQuality" value="%d" type="int" />\n' % (main['printQuality'])
     buff += '    <param name="donate" value="%s" type="str" />\n' % (main['donate'])
@@ -969,12 +799,11 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '    <param name="yLabel" value="%s" type="unicode" />\n' % (_escape(spectrum['yLabel']))
     buff += '    <param name="showGrid" value="%d" type="int" />\n' % (bool(spectrum['showGrid']))
     buff += '    <param name="showLegend" value="%d" type="int" />\n' % (bool(spectrum['showLegend']))
-    buff += '    <param name="showPosBar" value="%d" type="int" />\n' % (bool(spectrum['showPosBar']))
-    buff += '    <param name="showIntBar" value="%d" type="int" />\n' % (bool(spectrum['showIntBar']))
+    buff += '    <param name="showPosBars" value="%d" type="int" />\n' % (bool(spectrum['showPosBars']))
     buff += '    <param name="showGel" value="%d" type="int" />\n' % (bool(spectrum['showGel']))
     buff += '    <param name="showGelLegend" value="%d" type="int" />\n' % (bool(spectrum['showGelLegend']))
     buff += '    <param name="showTracker" value="%d" type="int" />\n' % (bool(spectrum['showTracker']))
-    buff += '    <param name="showNotationMarks" value="%d" type="int" />\n' % (bool(spectrum['showNotationMarks']))
+    buff += '    <param name="showNotations" value="%d" type="int" />\n' % (bool(spectrum['showNotations']))
     buff += '    <param name="showDataPoints" value="%d" type="int" />\n' % (bool(spectrum['showDataPoints']))
     buff += '    <param name="showLabels" value="%d" type="int" />\n' % (bool(spectrum['showLabels']))
     buff += '    <param name="showAllLabels" value="%d" type="int" />\n' % (bool(spectrum['showAllLabels']))
@@ -993,6 +822,10 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '    <param name="tickColour" value="%02x%02x%02x" type="str" />\n' % tuple(spectrum['tickColour'])
     buff += '    <param name="tmpSpectrumColour" value="%02x%02x%02x" type="str" />\n' % tuple(spectrum['tmpSpectrumColour'])
     buff += '    <param name="notationMarksColour" value="%02x%02x%02x" type="str" />\n' % tuple(spectrum['notationMarksColour'])
+    buff += '    <param name="notationMaxLength" value="%d" type="int" />\n' % (spectrum['notationMaxLength'])
+    buff += '    <param name="notationMarks" value="%d" type="int" />\n' % (bool(spectrum['notationMarks']))
+    buff += '    <param name="notationLabels" value="%d" type="int" />\n' % (bool(spectrum['notationLabels']))
+    buff += '    <param name="notationMZ" value="%d" type="int" />\n' % (bool(spectrum['notationMZ']))
     buff += '  </spectrum>\n\n'
     
     # match
@@ -1014,9 +847,8 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '      <param name="highMass" value="%d" type="int" />\n' % (processing['crop']['highMass'])
     buff += '    </crop>\n'
     buff += '    <baseline>\n'
-    buff += '      <param name="segments" value="%d" type="int" />\n' % (processing['baseline']['segments'])
+    buff += '      <param name="precision" value="%d" type="int" />\n' % (processing['baseline']['precision'])
     buff += '      <param name="offset" value="%f" type="float" />\n' % (processing['baseline']['offset'])
-    buff += '      <param name="smooth" value="%d" type="int" />\n' % (bool(processing['baseline']['smooth']))
     buff += '    </baseline>\n'
     buff += '    <smoothing>\n'
     buff += '      <param name="method" value="%s" type="str" />\n' % (processing['smoothing']['method'])
@@ -1027,12 +859,11 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '      <param name="snThreshold" value="%f" type="float" />\n' % (processing['peakpicking']['snThreshold'])
     buff += '      <param name="absIntThreshold" value="%f" type="float" />\n' % (processing['peakpicking']['absIntThreshold'])
     buff += '      <param name="relIntThreshold" value="%f" type="float" />\n' % (processing['peakpicking']['relIntThreshold'])
-    buff += '      <param name="adaptiveNoise" value="%d" type="int" />\n' % (bool(processing['peakpicking']['adaptiveNoise']))
     buff += '      <param name="pickingHeight" value="%f" type="float" />\n' % (processing['peakpicking']['pickingHeight'])
+    buff += '      <param name="baseline" value="%d" type="int" />\n' % (bool(processing['peakpicking']['baseline']))
     buff += '      <param name="smoothing" value="%d" type="int" />\n' % (bool(processing['peakpicking']['smoothing']))
     buff += '      <param name="deisotoping" value="%d" type="int" />\n' % (bool(processing['peakpicking']['deisotoping']))
     buff += '      <param name="removeShoulders" value="%d" type="int" />\n' % (bool(processing['peakpicking']['removeShoulders']))
-    buff += '      <param name="monoisotopic" value="%d" type="int" />\n' % (bool(processing['peakpicking']['monoisotopic']))
     buff += '    </peakpicking>\n'
     buff += '    <deisotoping>\n'
     buff += '      <param name="maxCharge" value="%d" type="int" />\n' % (processing['deisotoping']['maxCharge'])
@@ -1040,6 +871,8 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '      <param name="intTolerance" value="%f" type="float" />\n' % (processing['deisotoping']['intTolerance'])
     buff += '      <param name="removeIsotopes" value="%d" type="int" />\n' % (bool(processing['deisotoping']['removeIsotopes']))
     buff += '      <param name="removeUnknown" value="%d" type="int" />\n' % (bool(processing['deisotoping']['removeUnknown']))
+    buff += '      <param name="labelEnvelope" value="%s" type="str" />\n' % (processing['deisotoping']['labelEnvelope'])
+    buff += '      <param name="setAsMonoisotopic" value="%d" type="int" />\n' % (bool(processing['deisotoping']['setAsMonoisotopic']))
     buff += '    </deisotoping>\n'
     buff += '  </processing>\n\n'
     
@@ -1080,6 +913,7 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '      <param name="charge" value="%d" type="int" />\n' % (sequence['search']['charge'])
     buff += '      <param name="massType" value="%d" type="int" />\n' % (sequence['search']['massType'])
     buff += '      <param name="enzyme" value="%s" type="unicode" />\n' % (_escape(sequence['search']['enzyme']))
+    buff += '      <param name="semiSpecific" value="%d" type="int" />\n' % (bool(sequence['search']['semiSpecific']))
     buff += '      <param name="tolerance" value="%f" type="float" />\n' % (sequence['search']['tolerance'])
     buff += '      <param name="units" value="%s" type="str" />\n' % (sequence['search']['units'])
     buff += '      <param name="retainPos" value="%d" type="int" />\n' % (bool(sequence['search']['retainPos']))
@@ -1087,7 +921,7 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '    </search>\n'
     buff += '  </sequence>\n\n'
     
-    # masscalc
+    # mass calculator
     buff += '  <masscalc>\n'
     buff += '    <param name="ionseriesAgent" value="%s" type="str" />\n' % (masscalc['ionseriesAgent'])
     buff += '    <param name="ionseriesAgentCharge" value="%d" type="int" />\n' % (masscalc['ionseriesAgentCharge'])
@@ -1096,7 +930,7 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '    <param name="patternThreshold" value="%f" type="float" />\n' % (masscalc['patternThreshold'])
     buff += '  </masscalc>\n\n'
     
-    # compounds
+    # compounds search
     buff += '  <compounds>\n'
     buff += '    <param name="massType" value="%d" type="int" />\n' % (compounds['massType'])
     buff += '    <param name="maxCharge" value="%d" type="int" />\n' % (compounds['maxCharge'])
@@ -1104,7 +938,7 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '    <param name="adducts" value="%s" type="str" />\n' % (';'.join(compounds['adducts']))
     buff += '  </compounds>\n\n'
     
-    # differences
+    # peak differences
     buff += '  <differences>\n'
     buff += '    <param name="aminoacids" value="%d" type="int" />\n' % (bool(differences['aminoacids']))
     buff += '    <param name="dipeptides" value="%d" type="int" />\n' % (bool(differences['dipeptides']))
@@ -1112,13 +946,22 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     buff += '    <param name="massType" value="%d" type="int" />\n' % (differences['massType'])
     buff += '  </differences>\n\n'
     
-    # compare
+    # compare peaklists
     buff += '  <compare>\n'
     buff += '    <param name="tolerance" value="%f" type="float" />\n' % (compare['tolerance'])
     buff += '    <param name="units" value="%s" type="str" />\n' % (compare['units'])
     buff += '    <param name="ignoreCharge" value="%d" type="int" />\n' % (bool(compare['ignoreCharge']))
     buff += '  </compare>\n\n'
     
+    # spectrum generator
+    buff += '  <generator>\n'
+    buff += '    <param name="fwhm" value="%f" type="float" />\n' % (generator['fwhm'])
+    buff += '    <param name="points" value="%d" type="int" />\n' % (generator['points'])
+    buff += '    <param name="noise" value="%f" type="float" />\n' % (generator['noise'])
+    buff += '    <param name="showPeaks" value="%d" type="int" />\n' % (bool(generator['showPeaks']))
+    buff += '    <param name="showOverlay" value="%d" type="int" />\n' % (bool(generator['showOverlay']))
+    buff += '  </generator>\n\n'
+    
     # mascot
     buff += '  <mascot>\n'
     buff += '    <common>\n'
@@ -1269,7 +1112,7 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
     # links
     buff += '  <links>\n'
     for name in links:
-        if name not in ('mMassHomepage', 'mMassCite', 'mMassDonate', 'mMassDownload'):
+        if name not in ('mMassHomepage', 'mMassForum', 'mMassTwitter', 'mMassCite', 'mMassDonate', 'mMassDownload'):
             buff += '    <link name="%s" value="%s" />\n' % (_escape(name), _escape(links[name]))
     buff += '  </links>\n\n'
     
@@ -1286,171 +1129,23 @@ def saveConfig(path=os.path.join(confdir,'config.xml')):
 # ----
 
 
-def savePresets(path=os.path.join(confdir,'presets.xml')):
-    """Make and save presets XML."""
+def _getParams(sectionTag, section):
+    """Get params from nodes."""
     
-    buff = '<?xml version="1.0" encoding="utf-8" ?>\n'
-    buff += '<mMassPresets version="1.0">\n\n'
-    
-    # operator presets
-    buff += '  <operator>\n\n'
-    for name in sorted(libs.presets['operator'].keys()):
-        item = libs.presets['operator'][name]
-        buff += '    <presets name="%s">\n' % (_escape(name))
-        buff += '      <param name="operator" value="%s" type="unicode" />\n' % (_escape(item['operator']))
-        buff += '      <param name="contact" value="%s" type="unicode" />\n' % (_escape(item['contact']))
-        buff += '      <param name="institution" value="%s" type="unicode" />\n' % (_escape(item['institution']))
-        buff += '      <param name="instrument" value="%s" type="unicode" />\n' % (_escape(item['instrument']))
-        buff += '    </presets>\n\n'
-    buff += '  </operator>\n\n'
-    
-    # processing presets
-    buff += '  <processing>\n\n'
-    for name in sorted(libs.presets['processing'].keys()):
-        item = libs.presets['processing'][name]
-        buff += '    <presets name="%s">\n' % (_escape(name))
-        buff += '      <crop>\n'
-        buff += '        <param name="lowMass" value="%d" type="int" />\n' % (item['crop']['lowMass'])
-        buff += '        <param name="highMass" value="%d" type="int" />\n' % (item['crop']['highMass'])
-        buff += '      </crop>\n'
-        buff += '      <baseline>\n'
-        buff += '        <param name="segments" value="%d" type="int" />\n' % (item['baseline']['segments'])
-        buff += '        <param name="offset" value="%f" type="float" />\n' % (item['baseline']['offset'])
-        buff += '        <param name="smooth" value="%d" type="int" />\n' % (bool(item['baseline']['smooth']))
-        buff += '      </baseline>\n'
-        buff += '      <smoothing>\n'
-        buff += '        <param name="method" value="%s" type="str" />\n' % (item['smoothing']['method'])
-        buff += '        <param name="windowSize" value="%f" type="float" />\n' % (item['smoothing']['windowSize'])
-        buff += '        <param name="cycles" value="%d" type="int" />\n' % (item['smoothing']['cycles'])
-        buff += '      </smoothing>\n'
-        buff += '      <peakpicking>\n'
-        buff += '        <param name="snThreshold" value="%f" type="float" />\n' % (item['peakpicking']['snThreshold'])
-        buff += '        <param name="absIntThreshold" value="%f" type="float" />\n' % (item['peakpicking']['absIntThreshold'])
-        buff += '        <param name="relIntThreshold" value="%f" type="float" />\n' % (item['peakpicking']['relIntThreshold'])
-        buff += '        <param name="adaptiveNoise" value="%d" type="int" />\n' % (bool(item['peakpicking']['adaptiveNoise']))
-        buff += '        <param name="pickingHeight" value="%f" type="float" />\n' % (item['peakpicking']['pickingHeight'])
-        buff += '        <param name="smoothing" value="%d" type="int" />\n' % (bool(item['peakpicking']['smoothing']))
-        buff += '        <param name="deisotoping" value="%d" type="int" />\n' % (bool(item['peakpicking']['deisotoping']))
-        buff += '        <param name="removeShoulders" value="%d" type="int" />\n' % (bool(item['peakpicking']['removeShoulders']))
-        buff += '        <param name="monoisotopic" value="%d" type="int" />\n' % (bool(item['peakpicking']['monoisotopic']))
-        buff += '      </peakpicking>\n'
-        buff += '      <deisotoping>\n'
-        buff += '        <param name="maxCharge" value="%d" type="int" />\n' % (item['deisotoping']['maxCharge'])
-        buff += '        <param name="massTolerance" value="%f" type="float" />\n' % (item['deisotoping']['massTolerance'])
-        buff += '        <param name="intTolerance" value="%f" type="float" />\n' % (item['deisotoping']['intTolerance'])
-        buff += '        <param name="removeIsotopes" value="%d" type="int" />\n' % (bool(item['deisotoping']['removeIsotopes']))
-        buff += '        <param name="removeUnknown" value="%d" type="int" />\n' % (bool(item['deisotoping']['removeUnknown']))
-        buff += '      </deisotoping>\n'
-        buff += '    </presets>\n\n'
-    buff += '  </processing>\n\n'
-    
-    # modifications presets
-    buff += '  <modifications>\n\n'
-    for name in sorted(libs.presets['modifications'].keys()):
-        buff += '    <presets name="%s">\n' % (_escape(name))
-        for mod in libs.presets['modifications'][name]:
-            buff += '      <modification name="%s" position="%s" type="%s" />\n' % (mod[0], mod[1], mod[2])
-        buff += '    </presets>\n\n'
-    buff += '  </modifications>\n\n'
-    
-    # fragments presets
-    buff += '  <fragments>\n\n'
-    for name in sorted(libs.presets['fragments'].keys()):
-        buff += '    <presets name="%s">\n' % (_escape(name))
-        for fragment in libs.presets['fragments'][name]:
-            buff += '      <fragment name="%s" />\n' % (fragment)
-        buff += '    </presets>\n\n'
-    buff += '  </fragments>\n\n'
-    
-    buff += '</mMassPresets>'
-    
-    # save config file
-    try:
-        save = file(path, 'w')
-        save.write(buff.encode("utf-8"))
-        save.close()
-        return True
-    except:
-        return False
-# ----
-
-
-def saveReferences(path=os.path.join(confdir,'references.xml')):
-    """Make and save calibration references XML."""
-    
-    buff = '<?xml version="1.0" encoding="utf-8" ?>\n'
-    buff += '<mMassReferenceMasses version="1.0">\n\n'
-    
-    for group in sorted(libs.references.keys()):
-        buff += '  <group name="%s">\n' % (_escape(group))
-        for ref in libs.references[group]:
-            buff += '    <reference name="%s" mass="%f" />\n' % (_escape(ref[0]), ref[1])
-        buff += '  </group>\n\n'
-    
-    buff += '</mMassReferenceMasses>'
-    
-    # save config file
-    try:
-        save = file(path, 'w')
-        save.write(buff.encode("utf-8"))
-        save.close()
-        return True
-    except:
-        return False
-# ----
-
-
-def saveCompounds(path=os.path.join(confdir,'compounds.xml')):
-    """Make and save compounds XML."""
-    
-    buff = '<?xml version="1.0" encoding="utf-8" ?>\n'
-    buff += '<mMassCompounds version="1.0">\n\n'
-    
-    for group in sorted(libs.compounds.keys()):
-        buff += '  <group name="%s">\n' % (_escape(group))
-        for name, compound in sorted(libs.compounds[group].items()):
-            buff += '    <compound name="%s" formula="%s">%s</compound>\n' % (_escape(name), compound.rawFormula, _escape(compound.description))
-        buff += '  </group>\n\n'
-    
-    buff += '</mMassCompounds>'
-    
-    # save config file
-    try:
-        save = file(path, 'w')
-        save.write(buff.encode("utf-8"))
-        save.close()
-        return True
-    except:
-        return False
-# ----
-
-
-def saveMascot(path=os.path.join(confdir,'mascot.xml')):
-    """Make and save mascot servers XML."""
-    
-    buff = '<?xml version="1.0" encoding="utf-8" ?>\n'
-    buff += '<mMassMascot version="1.0">\n\n'
-    
-    for name in sorted(libs.mascot.keys()):
-        buff += '   <server name="%s">\n' % (_escape(name))
-        buff += '     <param name="host" value="%s" type="unicode" />\n' % (_escape(libs.mascot[name]['host']))
-        buff += '     <param name="path" value="%s" type="unicode" />\n' % (_escape(libs.mascot[name]['path']))
-        buff += '     <param name="search" value="%s" type="unicode" />\n' % (_escape(libs.mascot[name]['search']))
-        buff += '     <param name="results" value="%s" type="unicode" />\n' % (_escape(libs.mascot[name]['results']))
-        buff += '     <param name="export" value="%s" type="unicode" />\n' % (_escape(libs.mascot[name]['export']))
-        buff += '     <param name="params" value="%s" type="unicode" />\n' % (_escape(libs.mascot[name]['params']))
-        buff += '   </server>\n\n'
-    
-    buff += '</mMassMascot>'
-    
-    # save config file
-    try:
-        save = file(path, 'w')
-        save.write(buff.encode("utf-8"))
-        save.close()
-        return True
-    except:
-        return False
+    if sectionTag:
+        paramTags = sectionTag.getElementsByTagName('param')
+        if paramTags:
+            if paramTags:
+                for paramTag in paramTags:
+                    name = paramTag.getAttribute('name')
+                    value = paramTag.getAttribute('value')
+                    valueType = paramTag.getAttribute('type')
+                    if name in section:
+                        if valueType in ('unicode', 'str', 'float', 'int'):
+                            try:
+                                section[name] = eval(valueType+'(value)')
+                            except:
+                                pass
 # ----
 
 
@@ -1468,16 +1163,5 @@ def _escape(text):
 
 
 
-# LOAD CONFIGS
-# ------------
-
 try: loadConfig()
 except: saveConfig()
-try: loadPresets()
-except: savePresets()
-try: loadReferences()
-except: saveReferences()
-try: loadCompounds()
-except: saveCompounds()
-try: loadMascot()
-except: saveMascot()
