@@ -22,8 +22,9 @@ import xml.dom.minidom
 
 # load modules
 from ids import *
-import config
 import mwx
+import config
+import libs
 import mspy
 
 
@@ -190,7 +191,7 @@ class dlgCompoundsEditor(wx.Dialog):
         
         # get selected group
         group = self.groupName_combo.GetValue()
-        if group in config.libs.compounds:
+        if group in libs.compounds:
             self.group = group
         else:
             self.group = None
@@ -206,7 +207,7 @@ class dlgCompoundsEditor(wx.Dialog):
         
         # get selected item
         name = evt.GetText()
-        compound = config.libs.compounds[self.group][name]
+        compound = libs.compounds[self.group][name]
         
         # update item editor
         self.itemName_value.SetValue(name)
@@ -256,8 +257,8 @@ class dlgCompoundsEditor(wx.Dialog):
         selectAfter = 'Compounds lists'
         replaceAll = False
         for item in selected:
-            if replaceAll or not item in config.libs.compounds:
-                config.libs.compounds[item] = importedItems[item]
+            if replaceAll or not item in libs.compounds:
+                libs.compounds[item] = importedItems[item]
                 selectAfter = item
             else:
                 title = 'Group entitled "%s"\nis already in you library. Do you want to replace it?' % item
@@ -269,10 +270,10 @@ class dlgCompoundsEditor(wx.Dialog):
                     continue
                 elif ID == ID_dlgReplaceAll:
                     replaceAll = True
-                    config.libs.compounds[item] = importedItems[item]
+                    libs.compounds[item] = importedItems[item]
                     selectAfter = item
                 elif ID == ID_dlgReplace:
-                    config.libs.compounds[item] = importedItems[item]
+                    libs.compounds[item] = importedItems[item]
                     selectAfter = item
         
         # update gui
@@ -295,7 +296,7 @@ class dlgCompoundsEditor(wx.Dialog):
             return
         
         # check group name
-        if name in config.libs.compounds:
+        if name in libs.compounds:
             wx.Bell()
             dlg = mwx.dlgMessage(self, title='Group with the same name already exists.', message='Type a different name.')
             dlg.ShowModal()
@@ -303,7 +304,7 @@ class dlgCompoundsEditor(wx.Dialog):
             return
         
         # add group
-        config.libs.compounds[name] = {}
+        libs.compounds[name] = {}
         
         # update gui
         self.updateGroups()
@@ -332,7 +333,7 @@ class dlgCompoundsEditor(wx.Dialog):
         # check group name
         if name == self.group:
             return
-        if name in config.libs.compounds:
+        if name in libs.compounds:
             wx.Bell()
             dlg = mwx.dlgMessage(self, title='Group with the same name already exists.', message='Type a different name.')
             dlg.ShowModal()
@@ -340,9 +341,9 @@ class dlgCompoundsEditor(wx.Dialog):
             return
         
         # rename group
-        data = copy.deepcopy(config.libs.compounds[self.group])
-        config.libs.compounds[name] = data
-        del config.libs.compounds[self.group]
+        data = copy.deepcopy(libs.compounds[self.group])
+        libs.compounds[name] = data
+        del libs.compounds[self.group]
         
         # update gui
         self.updateGroups()
@@ -371,7 +372,7 @@ class dlgCompoundsEditor(wx.Dialog):
             dlg.Destroy()
         
         # remove group
-        del config.libs.compounds[self.group]
+        del libs.compounds[self.group]
         
         # update gui
         self.updateGroups()
@@ -394,7 +395,7 @@ class dlgCompoundsEditor(wx.Dialog):
             return
         
         # check name
-        if itemData.name in config.libs.compounds[self.group]:
+        if itemData.name in libs.compounds[self.group]:
             wx.Bell()
             title = 'Compound with the same name already exists.\nDo you want to replace it?'
             message = 'Old compound definition will be lost.'
@@ -407,7 +408,7 @@ class dlgCompoundsEditor(wx.Dialog):
                 dlg.Destroy()
         
         # add/replace item
-        config.libs.compounds[self.group][itemData.name] = itemData
+        libs.compounds[self.group][itemData.name] = itemData
         
         # update gui
         self.updateItemsList()
@@ -438,7 +439,7 @@ class dlgCompoundsEditor(wx.Dialog):
         for i in self.itemsList.getSelected():
             index = self.itemsList.GetItemData(i)
             name = self.itemsMap[index][0]
-            del config.libs.compounds[self.group][name]
+            del libs.compounds[self.group][name]
         
         # update gui
         self.updateItemsList()
@@ -460,7 +461,7 @@ class dlgCompoundsEditor(wx.Dialog):
         self.groupName_combo.Clear()
         
         # get names
-        choices = sorted(config.libs.compounds.keys())
+        choices = sorted(libs.compounds.keys())
         choices.insert(0, 'Compounds lists')
         
         # add names
@@ -479,7 +480,7 @@ class dlgCompoundsEditor(wx.Dialog):
             return
         
         # make map
-        for name, compound in sorted(config.libs.compounds[self.group].items()):
+        for name, compound in sorted(libs.compounds[self.group].items()):
             mass = compound.mass()
             self.itemsMap.append((
                 name,
