@@ -22,8 +22,9 @@ import xml.dom.minidom
 
 # load modules
 from ids import *
-import config
 import mwx
+import config
+import libs
 import mspy
 
 
@@ -117,7 +118,7 @@ class dlgReferencesEditor(wx.Dialog):
         
         # make columns
         self.itemsList.InsertColumn(0, "name", wx.LIST_FORMAT_LEFT)
-        self.itemsList.InsertColumn(1, "reference mass", wx.LIST_FORMAT_RIGHT)
+        self.itemsList.InsertColumn(1, "reference m/z", wx.LIST_FORMAT_RIGHT)
         
         # set column widths
         for col, width in enumerate((370,210)):
@@ -134,7 +135,7 @@ class dlgReferencesEditor(wx.Dialog):
         itemDescription_label = wx.StaticText(self, -1, "Description:")
         self.itemDescription_value = wx.TextCtrl(self, -1, "", size=(280, -1))
         
-        itemMass_label = wx.StaticText(self, -1, "Mass:")
+        itemMass_label = wx.StaticText(self, -1, "m/z:")
         self.itemMass_value = wx.TextCtrl(self, -1, "", size=(280, -1))
         
         # buttons
@@ -166,7 +167,7 @@ class dlgReferencesEditor(wx.Dialog):
         
         # get selected group
         group = self.groupName_combo.GetValue()
-        if group in config.libs.references:
+        if group in libs.references:
             self.group = group
         else:
             self.group = None
@@ -232,8 +233,8 @@ class dlgReferencesEditor(wx.Dialog):
         selectAfter = 'Reference lists'
         replaceAll = False
         for item in selected:
-            if replaceAll or not item in config.libs.references:
-                config.libs.references[item] = importedItems[item]
+            if replaceAll or not item in libs.references:
+                libs.references[item] = importedItems[item]
                 selectAfter = item
             else:
                 title = 'Group entitled "%s"\nis already in you library. Do you want to replace it?' % item
@@ -245,10 +246,10 @@ class dlgReferencesEditor(wx.Dialog):
                     continue
                 elif ID == ID_dlgReplaceAll:
                     replaceAll = True
-                    config.libs.references[item] = importedItems[item]
+                    libs.references[item] = importedItems[item]
                     selectAfter = item
                 elif ID == ID_dlgReplace:
-                    config.libs.references[item] = importedItems[item]
+                    libs.references[item] = importedItems[item]
                     selectAfter = item
         
         # update gui
@@ -271,7 +272,7 @@ class dlgReferencesEditor(wx.Dialog):
             return
         
         # check group name
-        if name in config.libs.compounds:
+        if name in libs.compounds:
             wx.Bell()
             dlg = mwx.dlgMessage(self, title='Group with the same name already exists.', message='Type a different name.')
             dlg.ShowModal()
@@ -279,7 +280,7 @@ class dlgReferencesEditor(wx.Dialog):
             return
         
         # add group
-        config.libs.references[name] = []
+        libs.references[name] = []
         
         # update gui
         self.updateGroups()
@@ -308,7 +309,7 @@ class dlgReferencesEditor(wx.Dialog):
         # check group name
         if name == self.group:
             return
-        if name in config.libs.compounds:
+        if name in libs.compounds:
             wx.Bell()
             dlg = mwx.dlgMessage(self, title='Group with the same name already exists.', message='Type a different name.')
             dlg.ShowModal()
@@ -316,9 +317,9 @@ class dlgReferencesEditor(wx.Dialog):
             return
         
         # rename group
-        data = copy.deepcopy(config.libs.references[self.group])
-        config.libs.references[name] = data
-        del config.libs.references[self.group]
+        data = copy.deepcopy(libs.references[self.group])
+        libs.references[name] = data
+        del libs.references[self.group]
         
         # update gui
         self.updateGroups()
@@ -347,7 +348,7 @@ class dlgReferencesEditor(wx.Dialog):
             dlg.Destroy()
         
         # remove group
-        del config.libs.references[self.group]
+        del libs.references[self.group]
         
         # update gui
         self.updateGroups()
@@ -370,7 +371,7 @@ class dlgReferencesEditor(wx.Dialog):
             return
         
         # add item
-        config.libs.references[self.group].append(itemData)
+        libs.references[self.group].append(itemData)
         
         # update gui
         self.updateItemsList()
@@ -407,7 +408,7 @@ class dlgReferencesEditor(wx.Dialog):
         indexes.sort()
         indexes.reverse()
         for i in indexes:
-            del config.libs.references[self.group][i]
+            del libs.references[self.group][i]
         
         # update gui
         self.updateItemsList()
@@ -422,7 +423,7 @@ class dlgReferencesEditor(wx.Dialog):
         self.groupName_combo.Clear()
         
         # get names
-        choices = sorted(config.libs.references.keys())
+        choices = sorted(libs.references.keys())
         choices.insert(0,'Reference lists')
         
         # add names
@@ -441,7 +442,7 @@ class dlgReferencesEditor(wx.Dialog):
             return
         
         # make map
-        self.itemsMap = config.libs.references[self.group]
+        self.itemsMap = libs.references[self.group]
     # ----
     
     
