@@ -28,6 +28,7 @@ import re
 import numpy
 
 # load modules
+import images
 import config
 import mspy
 
@@ -50,6 +51,7 @@ class document():
         self.institution = ''
         self.instrument = ''
         self.notes = ''
+        self.precision = 4
         
         self.spectrum = mspy.scan()
         self.annotations = []
@@ -63,9 +65,9 @@ class document():
         
         # undo buffers
         self.undo = None
-        self.spectrumBuff = None
-        self.annotationsBuff = None
-        self.sequencesBuff = None
+        self._spectrumBuff = None
+        self._annotationsBuff = None
+        self._sequencesBuff = None
     # ----
     
     
@@ -75,23 +77,23 @@ class document():
         self.undo = items
         
         # delete old
-        self.spectrumBuff = None
-        self.annotationsBuff = None
-        self.sequencesBuff = None
+        self._spectrumBuff = None
+        self._annotationsBuff = None
+        self._sequencesBuff = None
         
         if not items:
             return
         
         # store data
         if 'spectrum' in items:
-            self.spectrumBuff = copy.deepcopy(self.spectrum)
+            self._spectrumBuff = copy.deepcopy(self.spectrum)
         if 'annotations' in items:
-            self.annotationsBuff = copy.deepcopy(self.annotations)
+            self._annotationsBuff = copy.deepcopy(self.annotations)
         if 'sequences' in items:
-            self.sequencesBuff = copy.deepcopy(self.sequences)
+            self._sequencesBuff = copy.deepcopy(self.sequences)
         if 'notations' in items:
-            self.annotationsBuff = copy.deepcopy(self.annotations)
-            self.sequencesBuff = copy.deepcopy(self.sequences)
+            self._annotationsBuff = copy.deepcopy(self.annotations)
+            self._sequencesBuff = copy.deepcopy(self.sequences)
     # ----
     
     
@@ -105,21 +107,21 @@ class document():
         # revert data
         items = self.undo
         if 'spectrum' in items:
-            self.spectrum = self.spectrumBuff
+            self.spectrum = self._spectrumBuff
         if 'annotations' in items:
-            self.annotations[:] = self.annotationsBuff[:]
+            self.annotations[:] = self._annotationsBuff[:]
         if 'sequences' in items:
-            self.sequences[:] = self.sequencesBuff[:]
+            self.sequences[:] = self._sequencesBuff[:]
         if 'notations' in items:
-            self.annotations[:] = self.annotationsBuff[:]
+            self.annotations[:] = self._annotationsBuff[:]
             for x in range(len(self.sequences)):
-                self.sequences[x].matches[:] = self.sequencesBuff[x].matches[:]
+                self.sequences[x].matches[:] = self._sequencesBuff[x].matches[:]
         
         # clear buffers
         self.undo = None
-        self.spectrumBuff = None
-        self.annotationsBuff = None
-        self.sequencesBuff = None
+        self._spectrumBuff = None
+        self._annotationsBuff = None
+        self._sequencesBuff = None
         
         return items
     # ----
@@ -147,7 +149,23 @@ class document():
     # ----
     
     
-    def sortMatches(self):
+    def sortSequences(self):
+        """Sort sequences by titles."""
+        
+        # get sequences
+        sequences = []
+        for sequence in self.sequences:
+            sequences.append((sequence.title, sequence))
+        sequences.sort()
+        
+        # update document
+        del self.sequences[:]
+        for title, sequence in sequences:
+            self.sequences.append(sequence)
+    # ----
+    
+    
+    def sortSequenceMatches(self):
         """Sort sequence matches by m/z."""
         
         for sequence in self.sequences:
@@ -207,7 +225,7 @@ class document():
         # format peaklist
         buff += '  <peaklist>\n'
         for peak in self.spectrum.peaklist:
-            attributes = 'mz="%.6f" intensity="%.6f" baseline="%.6f"' % (peak.mz, peak.intensity, peak.baseline)
+            attributes = 'mz="%.6f" intensity="%.6f" baseline="%.6f"' % (peak.mz, peak.ai, peak.base)
             if peak.sn != None:
                 attributes += ' sn="%.3f"' % peak.sn
             if peak.charge != None:
@@ -222,7 +240,7 @@ class document():
         # format annotations
         buff += '  <annotations>\n'
         for annot in self.annotations:
-            attributes = 'peakMZ="%.6f" peakIntensity="%.6f" peakBaseline="%.6f"' % (annot.mz, annot.intensity, annot.baseline)
+            attributes = 'peakMZ="%.6f" peakIntensity="%.6f" peakBaseline="%.6f"' % (annot.mz, annot.ai, annot.base)
             if annot.charge !=None:
                 attributes += ' charge="%d"' % annot.charge
             if annot.radical:
@@ -240,6 +258,7 @@ class document():
             attributes = 'index="%d"' % index
             buff += '    <sequence %s>\n' % attributes
             buff += '      <title>%s</title>\n' % self._escape(sequence.title)
+            buff += '      <accession>%s</accession>\n' % self._escape(sequence.accession)
             buff += '      <seq>%s</seq>\n' % sequence.format('S')
             
             # format modifications
@@ -256,7 +275,7 @@ class document():
             # format matches
             buff += '      <matches>\n'
             for match in sequence.matches:
-                attributes = 'peakMZ="%.6f" peakIntensity="%.6f" peakBaseline="%.6f"' % (match.mz, match.intensity, match.baseline)
+                attributes = 'peakMZ="%.6f" peakIntensity="%.6f" peakBaseline="%.6f"' % (match.mz, match.ai, match.base)
                 if match.charge !=None:
                     attributes += ' charge="%d"' % match.charge
                 if match.radical:
@@ -301,9 +320,9 @@ class document():
         points = len(self.spectrum.points)
         peaks = len(self.spectrum.peaklist)
         
-        basePeak = self.spectrum.peaklist.basePeak
+        basePeak = self.spectrum.peaklist.basepeak()
         if basePeak:
-            basePeak = basePeak.intensity - basePeak.baseline
+            basePeak = basePeak.intensity
         
         if self.spectrum.scanNumber != None:
             scanNumber = self.spectrum.scanNumber
@@ -366,10 +385,10 @@ class document():
                 deltaDa = ''
                 deltaPpm = ''
                 formula = ''
-                label = self._replaceAnnotationIDs(annot.label)
+                label = self._replaceLabelIDs('compounds', annot.label)
                 
                 if basePeak:
-                    relIntensity = '%0.2f' % (((annot.intensity-annot.baseline)/basePeak)*100)
+                    relIntensity = '%0.2f' % (((annot.ai-annot.base)/basePeak)*100)
                 if annot.theoretical:
                     theoretical = mzFormat % annot.theoretical
                     deltaDa = mzFormat % annot.delta('Da')
@@ -384,20 +403,26 @@ class document():
         # sequences
         if self.sequences:
             for x, sequence in enumerate(self.sequences):
-                moMass = mzFormat % sequence.mass(0)
-                avMass = mzFormat % sequence.mass(1)
+                accession = self._replaceLabelIDs('sequences', sequence.accession)
+                mass = sequence.mass()
+                moMass = mzFormat % mass[0]
+                avMass = mzFormat % mass[1]
                 chain = self._formatSequence(sequence)
                 coverage = self._getSequenceCoverage(sequence)
                 tableID = 'tableSequenceMatches%d' % x
                 
-                buff += '  <h2>Sequence - <span>%s</span></h2>\n' % sequence.title
+                if accession:
+                    buff += '  <h2>Sequence - <span>%s</span> - [%s]</h2>\n' % (sequence.title, accession)
+                else:
+                    buff += '  <h2>Sequence - <span>%s</span></h2>\n' % sequence.title
+                
                 buff += '  <table id="tableSequenceInfo">\n'
                 buff += '    <thead>\n'
-                buff += '      <tr><th>Aminoacids</th><th>Mo. Mass</th><th>Av. Mass</th><th>Coverage</th></tr>\n'
+                buff += '      <tr><th>Accession</th><th>Aminoacids</th><th>Mo. Mass</th><th>Av. Mass</th><th>Coverage</th></tr>\n'
                 buff += '    </thead>\n'
                 buff += '    <tbody>\n'
-                buff += '      <tr><td class="right">%s</td><td class="right">%s</td><td class="right">%s</td><td class="right">%s</td></tr>\n' % (len(sequence), moMass, avMass, coverage)
-                buff += '      <tr><td colspan="4" class="sequence">%s</td></tr>\n' % chain
+                buff += '      <tr><td class="right">%s</td><td class="right">%s</td><td class="right">%s</td><td class="right">%s</td><td class="right">%s</td></tr>\n' % (accession, len(sequence), moMass, avMass, coverage)
+                buff += '      <tr><td colspan="5" class="sequence">%s</td></tr>\n' % chain
                 buff += '    </tbody>\n'
                 buff += '  </table>\n'
                 
@@ -435,7 +460,7 @@ class document():
                         formula = ''
                         
                         if basePeak:
-                            relIntensity = '%0.2f' % (((m.intensity-m.baseline)/basePeak)*100)
+                            relIntensity = '%0.2f' % (((m.ai-m.base)/basePeak)*100)
                         if m.theoretical:
                             theoretical = mzFormat % m.theoretical
                             deltaDa = mzFormat % m.delta('Da')
@@ -506,7 +531,7 @@ class document():
         for x, amino in enumerate(sequence):
             attributes = ''
             
-            if sequence.isModified(x, True):
+            if sequence.ismodified(x, True):
                 attributes += 'modified '
             if coverage[x]:
                 attributes += 'matched '
@@ -544,8 +569,9 @@ class document():
                 modtype = 'variable'
             
             # format masses
-            massMo = format % mspy.modifications[name].mass[0]
-            massAv = format % mspy.modifications[name].mass[1]
+            mass = mspy.modifications[name].mass
+            massMo = format % mass[0]
+            massAv = format % mass[1]
             
             # format formula
             formula = mspy.modifications[name].gainFormula
@@ -577,22 +603,23 @@ class document():
     # ----
     
     
-    def _replaceAnnotationIDs(self, label):
+    def _replaceLabelIDs(self, section, label):
         """Replace IDs with links in annotations."""
         
         # replace IDs
-        for name in config.replacements:
-            self._currentReplacement = name
-            label = re.sub(config.replacements[name]['pattern'], self._replaceIDs, label)
+        for name in config.replacements[section]:
+            self._currentReplacement = (section, name)
+            label = re.sub(config.replacements[section][name]['pattern'], self._replaceIDs, label)
         
         return label
     # ----
     
     
     def _replaceIDs(self, matchobj):
-        """Replace IDs to links in annotations."""
+        """Replace IDs to links."""
         
-        url = config.replacements[self._currentReplacement]['url'] % matchobj.group(1)
+        section, name = self._currentReplacement
+        url = config.replacements[section][name]['url'] % matchobj.group(1)
         return '<a href="%s" title="More information...">%s</a>' % (url, matchobj.group(0))
     # ----
     
@@ -605,12 +632,12 @@ class document():
 class annotation():
     """Annotation object definition."""
     
-    def __init__(self, label, mz, intensity, baseline=0., charge=None, radical=None, theoretical=None, formula=None):
+    def __init__(self, label, mz, ai, base=0., charge=None, radical=None, theoretical=None, formula=None):
         
         self.label = label
         self.mz = mz
-        self.intensity = intensity
-        self.baseline = baseline
+        self.ai = ai
+        self.base = base
         self.charge = charge
         self.radical = radical
         self.theoretical = theoretical
@@ -636,12 +663,12 @@ class annotation():
 class match():
     """Match object definition."""
     
-    def __init__(self, label, mz, intensity, baseline=0., charge=None, radical=None, theoretical=None, formula=None):
+    def __init__(self, label, mz, ai, base=0., charge=None, radical=None, theoretical=None, formula=None):
         
         self.label = label
         self.mz = mz
-        self.intensity = intensity
-        self.baseline = baseline
+        self.ai = ai
+        self.base = base
         self.charge = charge
         self.radical = radical
         self.theoretical = theoretical
@@ -673,9 +700,9 @@ class parseMSD():
     
     def __init__(self, path):
         
-        self.version = None
         self.path = path
-        self.parsedData = None
+        self._version = None
+        self._parsedData = None
         
         # init new document
         self.document = document()
@@ -688,16 +715,15 @@ class parseMSD():
         """Get document."""
         
         # parse data
-        if not self.parsedData:
-            try: self.parsedData = xml.dom.minidom.parse(self.path)
-            except: return False
-        
-        # get version
-        if not self.version:
-            self._getVersion()
+        if not self._parsedData:
+            try:
+                self._parsedData = xml.dom.minidom.parse(self.path)
+                self._version = self._getVersion()
+            except:
+                return False
         
         # get data
-        if self.version == '1.0':
+        if self._version == '1.0':
             self.handleDescription()
             self.handleSpectrum()
             self.handlePeaklist_10()
@@ -719,23 +745,22 @@ class parseMSD():
         """Get list of available sequences."""
         
         # parse data
-        if not self.parsedData:
-            try: self.parsedData = xml.dom.minidom.parse(self.path)
-            except: return False
-        
-        # get version
-        if not self.version:
-            self._getVersion()
+        if not self._parsedData:
+            try:
+                self._parsedData = xml.dom.minidom.parse(self.path)
+                self._version = self._getVersion()
+            except:
+                return False
         
         # set handler
-        if self.version == '1.0':
+        if self._version == '1.0':
             handler = self.handleSequence_10
         else:
             handler = self.handleSequence
         
         # get sequence
         data = []
-        sequenceTags = self.parsedData.getElementsByTagName('sequence')
+        sequenceTags = self._parsedData.getElementsByTagName('sequence')
         if sequenceTags:
             for sequenceTag in sequenceTags:
                 sequence = handler(sequenceTag)
@@ -753,7 +778,7 @@ class parseMSD():
         """Get document info."""
         
         # get description
-        descriptionTags = self.parsedData.getElementsByTagName('description')
+        descriptionTags = self._parsedData.getElementsByTagName('description')
         if descriptionTags:
             
             titleTags = descriptionTags[0].getElementsByTagName('title')
@@ -790,7 +815,7 @@ class parseMSD():
         """Get spectrum data."""
         
         # get spectrum
-        spectrumTags = self.parsedData.getElementsByTagName('spectrum')
+        spectrumTags = self._parsedData.getElementsByTagName('spectrum')
         if spectrumTags:
             
             # get metadata
@@ -853,11 +878,17 @@ class parseMSD():
                 return False
             
             # format data
-            points = map(list, zip(mzData, intData))
-            points = numpy.array(points)
+            mzData = numpy.array(mzData)
+            mzData.shape = (-1,1)
+            
+            intData = numpy.array(intData)
+            intData.shape = (-1,1)
+            
+            points = numpy.concatenate((mzData,intData), axis=1)
+            points = points.copy()
             
             # add to spectrum
-            self.document.spectrum.points = points
+            self.document.spectrum.setPoints(points)
     # ----
     
     
@@ -867,7 +898,7 @@ class parseMSD():
         peaklist = []
         
         # get peaklist
-        peaklistTags = self.parsedData.getElementsByTagName('peaklist')
+        peaklistTags = self._parsedData.getElementsByTagName('peaklist')
         if peaklistTags:
             
             # get peaks
@@ -877,16 +908,16 @@ class parseMSD():
                 # get data
                 try:
                     mz = float(peakTag.getAttribute('mz'))
-                    intensity = float(peakTag.getAttribute('intensity'))
+                    ai = float(peakTag.getAttribute('intensity'))
                     
-                    baseline = 0
+                    base = 0.0
                     sn = None
                     charge = None
                     isotope = None
                     fwhm = None
                     
                     if peakTag.hasAttribute('baseline'):
-                        baseline = float(peakTag.getAttribute('baseline'))
+                        base = float(peakTag.getAttribute('baseline'))
                     if peakTag.hasAttribute('sn'):
                         sn = float(peakTag.getAttribute('sn'))
                     if peakTag.hasAttribute('charge'):
@@ -900,12 +931,12 @@ class parseMSD():
                     continue
                 
                 # make peak
-                peak = mspy.peak(mz=mz, intensity=intensity, baseline=baseline, sn=sn, charge=charge, isotope=isotope, fwhm=fwhm)
+                peak = mspy.peak(mz=mz, ai=ai, base=base, sn=sn, charge=charge, isotope=isotope, fwhm=fwhm)
                 peaklist.append(peak)
         
         # add peaklist to document
         peaklist = mspy.peaklist(peaklist)
-        self.document.spectrum.peaklist = peaklist
+        self.document.spectrum.setPeaklist(peaklist)
     # ----
     
     
@@ -913,7 +944,7 @@ class parseMSD():
         """Get annotations."""
         
         # get annotations
-        annotationsTags = self.parsedData.getElementsByTagName('annotations')
+        annotationsTags = self._parsedData.getElementsByTagName('annotations')
         if annotationsTags:
             
             # get annotation
@@ -924,17 +955,17 @@ class parseMSD():
                 try:
                     label = self._getNodeText(annotationTag)
                     mz = float(annotationTag.getAttribute('peakMZ'))
-                    intensity = 0.
-                    baseline = 0.
+                    ai = 0.
+                    base = 0.
                     charge = None
                     radical = None
                     theoretical = None
                     formula = None
                     
                     if annotationTag.hasAttribute('peakIntensity'):
-                        intensity = float(annotationTag.getAttribute('peakIntensity'))
+                        ai = float(annotationTag.getAttribute('peakIntensity'))
                     if annotationTag.hasAttribute('peakBaseline'):
-                        baseline = float(annotationTag.getAttribute('peakBaseline'))
+                        base = float(annotationTag.getAttribute('peakBaseline'))
                     if annotationTag.hasAttribute('charge'):
                         charge = int(annotationTag.getAttribute('charge'))
                     if annotationTag.hasAttribute('radical'):
@@ -944,7 +975,7 @@ class parseMSD():
                     if annotationTag.hasAttribute('formula'):
                         formula = annotationTag.getAttribute('formula')
                     
-                    annot = annotation(label=label, mz=mz, intensity=intensity, baseline=baseline, charge=charge, radical=radical, theoretical=theoretical, formula=formula)
+                    annot = annotation(label=label, mz=mz, ai=ai, base=base, charge=charge, radical=radical, theoretical=theoretical, formula=formula)
                     
                 except ValueError:
                     continue
@@ -961,7 +992,7 @@ class parseMSD():
         """Get sequences."""
         
         # get sequences
-        sequencesTags = self.parsedData.getElementsByTagName('sequences')
+        sequencesTags = self._parsedData.getElementsByTagName('sequences')
         if sequencesTags:
             sequenceTags = sequencesTags[0].getElementsByTagName('sequence')
             for sequenceTag in sequenceTags:
@@ -975,10 +1006,16 @@ class parseMSD():
         """Get sequence."""
         
         # get title
-        seqTitle = ''
+        title = ''
         titleTags = sequenceTag.getElementsByTagName('title')
         if titleTags:
-            seqTitle = self._getNodeText(titleTags[0])
+            title = self._getNodeText(titleTags[0])
+        
+        # get accession
+        accession = ''
+        accessionTags = sequenceTag.getElementsByTagName('accession')
+        if accessionTags:
+            accession = self._getNodeText(accessionTags[0])
         
         # get sequence
         chain = ''
@@ -988,7 +1025,7 @@ class parseMSD():
         
         # make sequence
         try:
-            sequence = mspy.sequence(chain, seqTitle)
+            sequence = mspy.sequence(chain, title=title, accession=accession)
             sequence.matches = []
         except:
             return False
@@ -1023,8 +1060,8 @@ class parseMSD():
                 label = self._getNodeText(matchTag)
                 mz = float(matchTag.getAttribute('peakMZ'))
                 
-                intensity = 0.
-                baseline = 0.
+                ai = 0.
+                base = 0.
                 charge = None
                 radical = None
                 theoretical = None
@@ -1034,9 +1071,9 @@ class parseMSD():
                 fragmentIndex = None
                 
                 if matchTag.hasAttribute('peakIntensity'):
-                    intensity = float(matchTag.getAttribute('peakIntensity'))
+                    ai = float(matchTag.getAttribute('peakIntensity'))
                 if matchTag.hasAttribute('peakBaseline'):
-                    baseline = float(matchTag.getAttribute('peakBaseline'))
+                    base = float(matchTag.getAttribute('peakBaseline'))
                 if matchTag.hasAttribute('charge'):
                     charge = int(matchTag.getAttribute('charge'))
                 if matchTag.hasAttribute('radical'):
@@ -1052,7 +1089,7 @@ class parseMSD():
                 if matchTag.hasAttribute('fragmentIndex'):
                     fragmentIndex = int(matchTag.getAttribute('fragmentIndex'))
                 
-                m = match(label=label, mz=mz, intensity=intensity, baseline=baseline, charge=charge, radical=radical, theoretical=theoretical, formula=formula)
+                m = match(label=label, mz=mz, ai=ai, base=base, charge=charge, radical=radical, theoretical=theoretical, formula=formula)
                 m.sequenceRange = sequenceRange
                 m.fragmentSerie = fragmentSerie
                 m.fragmentIndex = fragmentIndex
@@ -1075,7 +1112,7 @@ class parseMSD():
         peaklist = []
         
         # get peaklist
-        peaklistTags = self.parsedData.getElementsByTagName('peaklist')
+        peaklistTags = self._parsedData.getElementsByTagName('peaklist')
         if peaklistTags:
             
             # get peaks
@@ -1085,22 +1122,22 @@ class parseMSD():
                 # get data
                 try:
                     mz = float(peakTag.getAttribute('mass'))
-                    intensity = float(peakTag.getAttribute('intens'))
+                    ai = float(peakTag.getAttribute('intens'))
                     annot = peakTag.getAttribute('annots')
                 except ValueError:
                     continue
                 
                 # make peak
-                peak = mspy.peak(mz=mz, intensity=intensity)
+                peak = mspy.peak(mz=mz, ai=ai)
                 peaklist.append(peak)
                 
                 # make annotation
                 if annot:
-                    self.document.annotations.append(annotation(label=annot, mz=mz, intensity=intensity))
+                    self.document.annotations.append(annotation(label=annot, mz=mz, ai=ai))
         
         # add peaklist to document
         peaklist = mspy.peaklist(peaklist)
-        self.document.spectrum.peaklist = peaklist
+        self.document.spectrum.setPeaklist(peaklist)
     # ----
     
     
@@ -1108,7 +1145,7 @@ class parseMSD():
         """Get sequences from mSD version 1.0."""
         
         # get sequences
-        sequencesTags = self.parsedData.getElementsByTagName('sequences')
+        sequencesTags = self._parsedData.getElementsByTagName('sequences')
         if sequencesTags:
             sequenceTags = sequencesTags[0].getElementsByTagName('sequence')
             for sequenceTag in sequenceTags:
@@ -1122,20 +1159,20 @@ class parseMSD():
         """Get sequence from mSD version 1.0."""
         
         # get title
-        seqTitle = ''
+        title = ''
         titleTags = sequenceTag.getElementsByTagName('title')
         if titleTags:
-            seqTitle = self._getNodeText(titleTags[0])
+            title = self._getNodeText(titleTags[0])
         
         # get sequence
-        seq = ''
+        chain = ''
         seqTags = sequenceTag.getElementsByTagName('seq')
         if seqTags:
-            seq = self._getNodeText(seqTags[0])
+            chain = self._getNodeText(seqTags[0])
         
         # make sequence
         try:
-            sequence = mspy.sequence(seq, seqTitle)
+            sequence = mspy.sequence(chain, title=title)
             sequence.matches = []
         except:
             return False
@@ -1195,16 +1232,14 @@ class parseMSD():
         """Get mSD format version."""
         
         # mSD document
-        mSDTags = self.parsedData.getElementsByTagName('mSD')
+        mSDTags = self._parsedData.getElementsByTagName('mSD')
         if mSDTags:
-            self.version = mSDTags[0].getAttribute('version')
-            return
+            return mSDTags[0].getAttribute('version')
         
         # mMassDoc document
-        mMassDocTags = self.parsedData.getElementsByTagName('mMassDoc')
+        mMassDocTags = self._parsedData.getElementsByTagName('mMassDoc')
         if mMassDocTags:
-            self.version = mMassDocTags[0].getAttribute('version')
-            return
+            return mMassDocTags[0].getAttribute('version')
     # ----
     
     
