@@ -28,32 +28,6 @@ HK_preferences = ''
 if wx.Platform == '__WXMAC__':
     HK_preferences = '\tCtrl+,'
 
-# tools
-ID_toolsBatch = wx.NewId()
-ID_toolsProcessing = wx.NewId()
-ID_toolsCalibration = wx.NewId()
-ID_toolsSequence = wx.NewId()
-ID_toolsPeriodicTable = wx.NewId()
-ID_toolsMasscalc = wx.NewId()
-ID_toolsMassfilter = wx.NewId()
-ID_toolsCompounds = wx.NewId()
-ID_toolsDifferences = wx.NewId()
-ID_toolsCompare = wx.NewId()
-ID_toolsMascot = wx.NewId()
-ID_toolsProfound = wx.NewId()
-ID_toolsProspector = wx.NewId()
-ID_toolsInfo = wx.NewId()
-ID_toolsReport = wx.NewId()
-ID_toolsExport = wx.NewId()
-
-HK_toolsCalibration = '\tCtrl+R'
-HK_toolsPeriodicTable = '\tShift+Ctrl+T'
-HK_toolsMasscalc = '\tShift+Ctrl+M'
-HK_toolsMassfilter = '\tShift+Ctrl+F'
-HK_toolsCompounds = '\tShift+Ctrl+U'
-HK_toolsDifferences = '\tShift+Ctrl+D'
-HK_toolsCompare = '\tShift+Ctrl+C'
-
 # file
 ID_documentNew = wx.NewId()
 ID_documentDuplicate = wx.NewId()
@@ -72,14 +46,12 @@ ID_documentOffset = wx.NewId()
 ID_documentClearOffset = wx.NewId()
 ID_documentClearOffsets = wx.NewId()
 ID_documentColour = wx.NewId()
-ID_documentCalibrateByAnnotations = wx.NewId()
 ID_documentAnnotationEdit = wx.NewId()
+ID_documentAnnotationDelete = wx.NewId()
 ID_documentAnnotationPattern = wx.NewId()
-ID_documentDeleteAnnotations = wx.NewId()
-ID_documentDeleteAnnotation = wx.NewId()
-ID_documentDeleteNotations = wx.NewId()
-
-HK_documentFlip = '\tAlt+Ctrl+F'
+ID_documentAnnotationsDelete = wx.NewId()
+ID_documentAnnotationsCalibrateBy = wx.NewId()
+ID_documentNotationsDelete = wx.NewId()
 
 ID_documentRecent0 = wx.NewId()
 ID_documentRecent1 = wx.NewId()
@@ -103,18 +75,54 @@ HK_documentExport = '\tCtrl+E'
 HK_documentInfo = '\tCtrl+I'
 HK_documentPrintSpectrum = '\tCtrl+P'
 HK_documentReport = '\tShift+Ctrl+R'
+HK_documentFlip = '\tAlt+Ctrl+F'
 
-# spectrum
-ID_spectrumLabelPeak = wx.NewId()
-ID_spectrumLabelPoint = wx.NewId()
-ID_spectrumDeleteLabel = wx.NewId()
-ID_spectrumMeasure = wx.NewId()
-ID_spectrumOffset = wx.NewId()
+# view
+ID_viewGrid = wx.NewId()
+ID_viewLegend = wx.NewId()
+ID_viewPosBars = wx.NewId()
+ID_viewGel = wx.NewId()
+ID_viewGelLegend = wx.NewId()
+ID_viewTracker = wx.NewId()
+ID_viewDataPoints = wx.NewId()
+ID_viewLabels = wx.NewId()
+ID_viewTicks = wx.NewId()
+ID_viewLabelCharge = wx.NewId()
+ID_viewLabelAngle = wx.NewId()
+ID_viewAllLabels = wx.NewId()
+ID_viewLabelBgr = wx.NewId()
+ID_viewOverlapLabels = wx.NewId()
+ID_viewNotations = wx.NewId()
+ID_viewNotationMarks = wx.NewId()
+ID_viewNotationLabels = wx.NewId()
+ID_viewNotationMz = wx.NewId()
+ID_viewAutoscale = wx.NewId()
+ID_viewNormalize = wx.NewId()
+ID_viewRange = wx.NewId()
+ID_viewCanvasProperties = wx.NewId()
 
-HK_spectrumLabelPeak = '\tShift+Ctrl+P'
-HK_spectrumLabelPoint = '\tShift+Ctrl+I'
-HK_spectrumDeleteLabel = '\tShift+Ctrl+X'
-HK_spectrumMeasure = '\tShift+Ctrl+H'
+ID_viewPeaklistColumnMz = wx.NewId()
+ID_viewPeaklistColumnAi = wx.NewId()
+ID_viewPeaklistColumnBase = wx.NewId()
+ID_viewPeaklistColumnInt = wx.NewId()
+ID_viewPeaklistColumnRel = wx.NewId()
+ID_viewPeaklistColumnSn = wx.NewId()
+ID_viewPeaklistColumnZ = wx.NewId()
+ID_viewPeaklistColumnMass = wx.NewId()
+ID_viewPeaklistColumnFwhm = wx.NewId()
+ID_viewPeaklistColumnResol = wx.NewId()
+
+HK_viewPosBars = '\tAlt+Ctrl+P'
+HK_viewGel = '\tAlt+Ctrl+G'
+HK_viewLabels = '\tAlt+Ctrl+L'
+HK_viewTicks = '\tAlt+Ctrl+T'
+HK_viewLabelAngle = '\tAlt+Ctrl+H'
+HK_viewAllLabels = '\tAlt+Ctrl+Shift+L'
+HK_viewOverlapLabels = '\tAlt+Ctrl+O'
+HK_viewAutoscale = '\tAlt+Ctrl+A'
+HK_viewNormalize = '\tAlt+Ctrl+N'
+HK_viewRange = '\tAlt+Ctrl+R'
+HK_viewCanvasProperties = '\tCtrl+J'
 
 # processing
 ID_processingUndo = wx.NewId()
@@ -132,10 +140,6 @@ HK_processingDeisotoping = '\tCtrl+D'
 HK_processingSmoothing = '\tCtrl+G'
 HK_processingBaseline = '\tCtrl+B'
 
-# calibration
-ID_calibrationReferences = wx.NewId()
-ID_calibrationErrors = wx.NewId()
-
 # sequence
 ID_sequenceNew = wx.NewId()
 ID_sequenceImport = wx.NewId()
@@ -144,93 +148,53 @@ ID_sequenceModifications = wx.NewId()
 ID_sequenceDigest = wx.NewId()
 ID_sequenceFragment = wx.NewId()
 ID_sequenceSearch = wx.NewId()
-ID_sequenceCalibrateByMatches = wx.NewId()
-ID_sequenceMatchEdit = wx.NewId()
-ID_sequenceMatchPattern = wx.NewId()
-ID_sequenceDeleteMatch = wx.NewId()
-ID_sequenceDeleteMatches = wx.NewId()
 ID_sequenceDelete = wx.NewId()
 ID_sequenceSort = wx.NewId()
+ID_sequenceMatchEdit = wx.NewId()
+ID_sequenceMatchDelete = wx.NewId()
+ID_sequenceMatchPattern = wx.NewId()
+ID_sequenceMatchesDelete = wx.NewId()
+ID_sequenceMatchesCalibrateBy = wx.NewId()
 
 HK_sequenceNew = '\tShift+Ctrl+N'
 
-# view
-ID_viewGrid = wx.NewId()
-ID_viewLegend = wx.NewId()
-ID_viewPosBar = wx.NewId()
-ID_viewIntBar = wx.NewId()
-ID_viewGel = wx.NewId()
-ID_viewGelLegend = wx.NewId()
-ID_viewTracker = wx.NewId()
-ID_viewDataPoints = wx.NewId()
-ID_viewNotationMarks = wx.NewId()
-ID_viewLabels = wx.NewId()
-ID_viewTicks = wx.NewId()
-ID_viewLabelCharge = wx.NewId()
-ID_viewLabelAngle = wx.NewId()
-ID_viewAllLabels = wx.NewId()
-ID_viewLabelBgr = wx.NewId()
-ID_viewOverlapLabels = wx.NewId()
-ID_viewAutoscale = wx.NewId()
-ID_viewNormalize = wx.NewId()
-ID_viewRange = wx.NewId()
-ID_viewCanvasProperties = wx.NewId()
+# tools
+ID_toolsProcessing = wx.NewId()
+ID_toolsCalibration = wx.NewId()
+ID_toolsSequence = wx.NewId()
+ID_toolsLabelPeak = wx.NewId()
+ID_toolsLabelPoint = wx.NewId()
+ID_toolsLabelEnvelope = wx.NewId()
+ID_toolsDeleteLabel = wx.NewId()
+ID_toolsMeasure = wx.NewId()
+ID_toolsOffset = wx.NewId()
+ID_toolsPeriodicTable = wx.NewId()
+ID_toolsMasscalc = wx.NewId()
+ID_toolsMassFilter = wx.NewId()
+ID_toolsCompoundsSearch = wx.NewId()
+ID_toolsPeakDifferences = wx.NewId()
+ID_toolsComparePeaklists = wx.NewId()
+ID_toolsSpectrumGenerator = wx.NewId()
+ID_toolsMascot = wx.NewId()
+ID_toolsProfound = wx.NewId()
+ID_toolsProspector = wx.NewId()
+ID_toolsDocumentInfo = wx.NewId()
+ID_toolsDocumentReport = wx.NewId()
+ID_toolsDocumentExport = wx.NewId()
 
-HK_viewPosBar = '\tAlt+Ctrl+P'
-HK_viewIntBar = '\tAlt+Ctrl+I'
-HK_viewGel = '\tAlt+Ctrl+G'
-HK_viewLabels = '\tAlt+Ctrl+L'
-HK_viewTicks = '\tAlt+Ctrl+T'
-HK_viewLabelAngle = '\tAlt+Ctrl+H'
-HK_viewAllLabels = '\tAlt+Ctrl+Shift+L'
-HK_viewOverlapLabels = '\tAlt+Ctrl+O'
-HK_viewAutoscale = '\tAlt+Ctrl+A'
-HK_viewNormalize = '\tAlt+Ctrl+N'
-HK_viewRange = '\tAlt+Ctrl+R'
-HK_viewCanvasProperties = '\tCtrl+J'
-
-# match
-ID_matchErrors = wx.NewId()
-ID_matchSummary = wx.NewId()
-
-# masscalc
-ID_masscalcSummary = wx.NewId()
-ID_masscalcIonSeries = wx.NewId()
-ID_masscalcPattern = wx.NewId()
-ID_masscalcCollapse = wx.NewId()
-
-# periodic table
-ID_ptableTable = wx.NewId()
-ID_ptablePattern = wx.NewId()
-
-# coumpounds search
-ID_compoundsCompounds = wx.NewId()
-ID_compoundsFormula = wx.NewId()
-
-# mascot
-ID_mascotPMF = wx.NewId()
-ID_mascotMIS = wx.NewId()
-ID_mascotSQ = wx.NewId()
-ID_mascotQuery = wx.NewId()
-
-# profound
-ID_profoundPMF = wx.NewId()
-ID_profoundQuery = wx.NewId()
-
-# prospector
-ID_prospectorMSFit = wx.NewId()
-ID_prospectorMSTag = wx.NewId()
-ID_prospectorQuery = wx.NewId()
-
-# info
-ID_infoSummary = wx.NewId()
-ID_infoSpectrum = wx.NewId()
-ID_infoNotes = wx.NewId()
-
-# export
-ID_exportImage = wx.NewId()
-ID_exportPeaklist = wx.NewId()
-ID_exportSpectrum = wx.NewId()
+HK_toolsCalibration = '\tCtrl+R'
+HK_toolsLabelPeak = '\tShift+Ctrl+P'
+HK_toolsLabelPoint = '\tShift+Ctrl+I'
+HK_toolsLabelEnvelope = '\tShift+Ctrl+E'
+HK_toolsDeleteLabel = '\tShift+Ctrl+X'
+HK_toolsMeasure = '\tShift+Ctrl+H'
+HK_toolsPeriodicTable = '\tShift+Ctrl+T'
+HK_toolsMasscalc = '\tShift+Ctrl+M'
+HK_toolsMassFilter = '\tShift+Ctrl+F'
+HK_toolsCompoundsSearch = '\tShift+Ctrl+U'
+HK_toolsPeakDifferences = '\tShift+Ctrl+D'
+HK_toolsComparePeaklists = '\tShift+Ctrl+C'
+HK_toolsSpectrumGenerator = '\tShift+Ctrl+G'
 
 # library
 ID_libraryModifications = wx.NewId()
@@ -262,11 +226,55 @@ ID_linksUniProt = wx.NewId()
 ID_helpAbout = wx.ID_ABOUT
 ID_helpHomepage = wx.NewId()
 ID_helpForum = wx.NewId()
-ID_helpUpdate = wx.NewId()
+ID_helpTwitter = wx.NewId()
 ID_helpCite = wx.NewId()
 ID_helpDonate = wx.NewId()
+ID_helpUpdate = wx.NewId()
 
-# dialogues
+# calibration panel
+ID_calibrationReferences = wx.NewId()
+ID_calibrationErrors = wx.NewId()
+
+# match panel
+ID_matchErrors = wx.NewId()
+ID_matchSummary = wx.NewId()
+
+# masscalc panel
+ID_massCalculatorSummary = wx.NewId()
+ID_massCalculatorIonSeries = wx.NewId()
+ID_massCalculatorPattern = wx.NewId()
+ID_massCalculatorCollapse = wx.NewId()
+
+# coumpounds search panel
+ID_compoundsSearchCompounds = wx.NewId()
+ID_compoundsSearchFormula = wx.NewId()
+
+# mascot panel
+ID_mascotPMF = wx.NewId()
+ID_mascotMIS = wx.NewId()
+ID_mascotSQ = wx.NewId()
+ID_mascotQuery = wx.NewId()
+
+# profound panel
+ID_profoundPMF = wx.NewId()
+ID_profoundQuery = wx.NewId()
+
+# prospector panel
+ID_prospectorMSFit = wx.NewId()
+ID_prospectorMSTag = wx.NewId()
+ID_prospectorQuery = wx.NewId()
+
+# info panel
+ID_documentInfoSummary = wx.NewId()
+ID_documentInfoSpectrum = wx.NewId()
+ID_documentInfoNotes = wx.NewId()
+
+# export panel
+ID_documentExportImage = wx.NewId()
+ID_documentExportPeaklist = wx.NewId()
+ID_documentExportSpectrum = wx.NewId()
+
+# dialog buttons
 ID_dlgDontSave = wx.NewId()
 ID_dlgSave = wx.NewId()
 ID_dlgCancel = wx.NewId()
@@ -276,7 +284,7 @@ ID_dlgReplace = wx.NewId()
 ID_dlgReplaceAll = wx.NewId()
 ID_dlgSkip = wx.NewId()
 
-# common
+# list pop-up menu
 ID_listViewAll = wx.NewId()
 ID_listViewMatched = wx.NewId()
 ID_listViewUnmatched = wx.NewId()
