@@ -19,11 +19,11 @@
 import time
 import copy
 import wx
-import config
 
 # load modules
-import images
 from ids import *
+import images
+import config
 
 
 # GUI CONSTANTS
@@ -95,7 +95,7 @@ if wx.Platform == '__WXMAC__':
     GRIDBAG_VSPACE = 10
     GRIDBAG_HSPACE = 5
     
-    GAUGE_HEIGHT = -1
+    GAUGE_HEIGHT = 11
     GAUGE_SPACE = 15
     
     MAIN_TOOLBAR_TOOLSIZE = (30,22)
@@ -230,15 +230,63 @@ class sortListCtrl(wx.ListCtrl):
     def __init__(self, parent, id=-1, pos=wx.DefaultPosition, size=wx.DefaultSize, style=wx.LC_REPORT):
         wx.ListCtrl.__init__(self, parent, id, pos, size, style)
         
-        self.Bind(wx.EVT_LIST_COL_CLICK, self._onColClick)
-        
         self._data = None
-        self._currentCol = 0
+        self._currentColumn = 0
         self._currentDirection = LISTCTRL_SORT
-        self._secondaryCol = None
+        self._secondarySortColumn = None
         
         self._defaultColour = self.GetBackgroundColour()
         self._altColour = self.GetBackgroundColour()
+        self._currentAttr = wx.ListItemAttr()
+        
+        self._getItemTextFce = None
+        self._getItemAttrFce = None
+        
+        # set events
+        self.Bind(wx.EVT_LIST_COL_CLICK, self._onColClick, self)
+    # ----
+    
+    
+    def OnGetItemText(self, row, col):
+        """Get text for selected cell."""
+        
+        if self._getItemTextFce != None:
+            return self._getItemTextFce(row, col)
+        else:
+            return unicode(self._data[row][col])
+    # ----
+    
+    
+    def OnGetItemAttr(self, row):
+        """Get attributes for selected cell."""
+        
+        # get user defined attr
+        attr = None
+        if self._getItemAttrFce != None:
+            attr = self._getItemAttrFce(row)
+        
+        # set background colour
+        if attr and attr.HasBackgroundColour():
+            self._currentAttr.SetBackgroundColour(attr.GetBackgroundColour())
+        elif row % 2:
+            self._currentAttr.SetBackgroundColour(self._defaultColour)
+        else:
+            self._currentAttr.SetBackgroundColour(self._altColour)
+        
+        # set text colour
+        if attr:
+            self._currentAttr.SetTextColour(attr.GetTextColour())
+        
+        # set font
+        if attr:
+            self._currentAttr.SetFont(attr.GetFont())
+        
+        return self._currentAttr
+    # ----
+    
+    
+    def OnGetItemImage(self, row):
+        return -1
     # ----
     
     
@@ -250,21 +298,55 @@ class sortListCtrl(wx.ListCtrl):
             return
         
         # get selected column
-        oldCol = self._currentCol
-        self._currentCol = evt.GetColumn()
+        oldCol = self._currentColumn
+        newCol = evt.GetColumn()
         
         # update direction flag
-        if oldCol == self._currentCol:
-            self._currentDirection *= -1
+        if oldCol == newCol:
+            direction = -1 * self._currentDirection
         else:
-            self._currentDirection = LISTCTRL_SORT
+            direction = LISTCTRL_SORT
+        
+        # sort
+        self._sort(newCol, direction)
+        evt.Skip()
+    # ----
+    
+    
+    def _sort(self, col, direction):
+        """Sort list."""
+        
+        # unselect all items
+        self.unselectAll()
+        
+        # set new flags
+        self._currentColumn = min(col, self.GetColumnCount()-1)
+        self._currentDirection = direction
         
         # sort data
-        self.SortItems(self._columnSorter)
-        evt.Skip()
+        if self.IsVirtual():
+            self._data.sort(self._sortItems)
+            self.Refresh()
+        else:
+            self.SortItems(self._sortData)
+            self.updateItemsBackground()
+    # ----
+    
+    
+    def _sortData(self, item1, item2):
+        """Sort data."""
+        return self._sortItems(self._data[item1], self._data[item2])
+    # ----
+    
+    
+    def _sortItems(self, item1, item2):
+        """Sort items."""
         
-        # set row colour
-        self.updateItemsBackground()
+        comp = cmp(item1[self._currentColumn], item2[self._currentColumn])
+        if comp == 0 and self._secondarySortColumn != None:
+            comp = cmp(item1[self._secondarySortColumn], item2[self._secondarySortColumn])
+        
+        return comp * self._currentDirection
     # ----
     
     
@@ -276,14 +358,14 @@ class sortListCtrl(wx.ListCtrl):
             return self._currentDirection
         
         # get values
-        item1 = self._data[key1][self._currentCol]
-        item2 = self._data[key2][self._currentCol]
+        item1 = self._data[key1][self._currentColumn]
+        item2 = self._data[key2][self._currentColumn]
         
         # compare values
         comp = cmp(item1, item2)
-        if comp == 0 and self._secondaryCol != None:
-            item1 = self._data[key1][self._secondaryCol]
-            item2 = self._data[key2][self._secondaryCol]
+        if comp == 0 and self._secondarySortColumn != None:
+            item1 = self._data[key1][self._secondarySortColumn]
+            item2 = self._data[key2][self._secondarySortColumn]
             comp = cmp(item1, item2)
         
         # set direction
@@ -293,9 +375,21 @@ class sortListCtrl(wx.ListCtrl):
     # ----
     
     
+    def setItemTextFce(self, fce):
+        """Set OnGetItemText callback."""
+        self._getItemTextFce = fce
+    # ----
+    
+    
+    def setItemAttrFce(self, fce):
+        """Set OnGetItemAttr callback."""
+        self._getItemAttrFce = fce
+    # ----
+    
+    
     def setSecondarySortColumn(self, col):
         """Set secondary column to sort by."""
-        self._secondaryCol = col
+        self._secondarySortColumn = col
     # ----
     
     
@@ -336,16 +430,37 @@ class sortListCtrl(wx.ListCtrl):
     def sort(self, col=None):
         """Sort by last or selected column."""
         
-        # get column
-        if col != None:
-            if self._currentCol != col:
-                self._currentDirection = LISTCTRL_SORT
+        # get column and direction
+        direction = self._currentDirection
+        if col == None:
+            col = self._currentColumn
+        else:
+            if self._currentColumn != col:
+                direction = LISTCTRL_SORT
         
-        # sort data
-        self.SortItems(self._columnSorter)
+        # sort
+        self._sort(col, direction)
+    # ----
+    
+    
+    def deleteColumns(self):
+        """Delete all columns."""
         
-        # set row colour
-        self.updateItemsBackground()
+        self._currentColumn = 0
+        while self.GetColumnCount():
+            self.DeleteColumn(0)
+    # ----
+    
+    
+    def unselectAll(self):
+        """Unselect all items."""
+        
+        i = -1
+        while True:
+            i = self.GetNextItem(i, wx.LIST_NEXT_ALL, wx.LIST_STATE_SELECTED)
+            self.SetItemState(i, 0, wx.LIST_STATE_SELECTED)
+            if i == -1:
+                break
     # ----
     
     
@@ -365,17 +480,28 @@ class sortListCtrl(wx.ListCtrl):
     # ----
     
     
-    def copyToClipboard(self):
+    def copyToClipboard(self, selected=False):
         """Copy current data to clipboard."""
         
-        # get data
         buff = ''
-        for row in range(self.GetItemCount()):
-            line = ''
-            for col in range(self.GetColumnCount()):
-                item = self.GetItem(row, col)
-                line += item.GetText() + '\t'
-            buff += '%s\n' % (line.rstrip())
+        
+        # get selected only
+        if selected:
+            for row in self.getSelected():
+                line = ''
+                for col in range(self.GetColumnCount()):
+                    item = self.GetItem(row, col)
+                    line += item.GetText() + '\t'
+                buff += '%s\n' % (line.rstrip())
+        
+        # get all
+        else:
+            for row in range(self.GetItemCount()):
+                line = ''
+                for col in range(self.GetColumnCount()):
+                    item = self.GetItem(row, col)
+                    line += item.GetText() + '\t'
+                buff += '%s\n' % (line.rstrip())
         
         # make text object for data
         obj = wx.TextDataObject()
@@ -412,7 +538,7 @@ class scrollTextCtrl(wx.TextCtrl):
         """Increase or decrease value while scrolling."""
         
         # set new value
-        self._setNewValue(evt.GetWheelRotation(), evt.m_altDown)
+        self._setNewValue(evt.GetWheelRotation(), evt.AltDown())
     # ----
     
     
@@ -432,7 +558,7 @@ class scrollTextCtrl(wx.TextCtrl):
             return
         
         # set new value
-        self._setNewValue(direction, evt.m_altDown)
+        self._setNewValue(direction, evt.AltDown())
     # ----
     
     
