@@ -23,17 +23,19 @@ import numpy
 
 # load modules
 from ids import *
-import config
 import mwx
 import images
+import config
+import libs
 import mspy
+import mspy.plot
 
 
-# FLOATING PANEL WITH CALIBRATION TOOLS
-# -------------------------------------
+# FLOATING PANEL WITH CALIBRATION TOOL
+# ------------------------------------
 
 class panelCalibration(wx.MiniFrame):
-    """Calibration tools."""
+    """Calibration tool."""
     
     def __init__(self, parent, tool='references'):
         wx.MiniFrame.__init__(self, parent, -1, 'Calibration', size=(400, 300), style=wx.DEFAULT_FRAME_STYLE & ~ (wx.RESIZE_BOX | wx.MAXIMIZE_BOX))
@@ -73,10 +75,12 @@ class panelCalibration(wx.MiniFrame):
         self.mainSizer.Add(errors, 1, wx.EXPAND, 0)
         self.mainSizer.Add(gauge, 0, wx.EXPAND, 0)
         
+        # hide panels
         self.mainSizer.Hide(1)
         self.mainSizer.Hide(2)
         self.mainSizer.Hide(3)
         
+        # fit layout
         self.mainSizer.Fit(self)
         self.SetSizer(self.mainSizer)
     # ----
@@ -170,7 +174,7 @@ class panelCalibration(wx.MiniFrame):
         # make controls
         references_label = wx.StaticText(ctrlPanel, -1, "References:")
         references_label.SetFont(wx.SMALL_FONT)
-        choices = config.libs.references.keys()
+        choices = libs.references.keys()
         choices.sort()
         choices.insert(0,'Reference lists')
         self.references_combo = wx.ComboBox(ctrlPanel, -1, choices=choices, size=(250, mwx.SMALL_COMBO_HEIGHT), style=wx.CB_READONLY)
@@ -259,12 +263,12 @@ class panelCalibration(wx.MiniFrame):
         self.errorCanvas.setProperties(showPosBar=True)
         self.errorCanvas.setProperties(posBarHeight=6)
         self.errorCanvas.setProperties(showGel=False)
-        self.errorCanvas.setProperties(showCurTracker=True)
         self.errorCanvas.setProperties(checkLimits=True)
         self.errorCanvas.setProperties(autoScaleY=False)
         self.errorCanvas.setProperties(xPosDigits=config.main['mzDigits'])
         self.errorCanvas.setProperties(yPosDigits=2)
         self.errorCanvas.setProperties(reverseDrawing=True)
+        self.errorCanvas.setMFunction('cross')
         
         axisFont = wx.Font(config.spectrum['axisFontSize'], wx.SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, 0)
         self.errorCanvas.setProperties(axisFont=axisFont)
@@ -285,6 +289,8 @@ class panelCalibration(wx.MiniFrame):
         
         # pack elements
         mainSizer = wx.BoxSizer(wx.VERTICAL)
+        if wx.Platform == '__WXMAC__':
+            mainSizer.Add(wx.StaticLine(panel), 0, wx.EXPAND|wx.TOP, -1)
         mainSizer.Add(self.gauge, 0, wx.EXPAND|wx.ALL, mwx.GAUGE_SPACE)
         
         # fit layout
@@ -364,12 +370,9 @@ class panelCalibration(wx.MiniFrame):
         
         # fit layout
         self.SetMinSize((-1,-1))
-        self.mainSizer.Fit(self)
         self.Layout()
-        size = self.GetSize()
-        self.SetSize((size[0]+1,size[1]))
-        self.SetSize(size)
-        self.SetMinSize(size)
+        self.mainSizer.Fit(self)
+        self.SetMinSize(self.GetSize())
     # ----
     
     
@@ -431,9 +434,9 @@ class panelCalibration(wx.MiniFrame):
         
         # get references
         group = self.references_combo.GetValue()
-        if group and group in config.libs.references:
+        if group and group in libs.references:
             self.currentReferences = []
-            for item in config.libs.references[group]:
+            for item in libs.references[group]:
                 self.currentReferences.append([item[0], item[1], None, None, None, None, True])
                 # 0 title, 1 theoretical, 2 measured, 3 calibrated, 4 error before, 5 error after, 6 use
         
@@ -707,7 +710,7 @@ class panelCalibration(wx.MiniFrame):
             container.append(obj)
         
         # set units
-        self.errorCanvas.setProperties(yLabel='error in %s' % (config.calibration['units']))
+        self.errorCanvas.setProperties(yLabel='error (%s)' % (config.calibration['units']))
         
         # draw container
         self.errorCanvas.draw(container)
@@ -878,7 +881,7 @@ class panelCalibration(wx.MiniFrame):
         self.currentDocument.backup(('spectrum', 'notations'))
         
         # recalibrate spectrum
-        self.currentDocument.spectrum.calibrate(fce, params)
+        self.currentDocument.spectrum.applyCalibration(fce, params)
         
         # recalibrate annotations
         for annotation in self.currentDocument.annotations:
@@ -923,11 +926,11 @@ class panelCalibration(wx.MiniFrame):
         
         # convert peaklist
         peaklist = []
-        basePeak = self.currentDocument.spectrum.peaklist.basePeak
-        f = abs(maxY - minY) / (basePeak.intensity - basePeak.baseline)
+        basePeak = self.currentDocument.spectrum.peaklist.basepeak()
+        f = abs(maxY - minY) / basePeak.intensity
         for peak in self.currentDocument.spectrum.peaklist:
-            intensity = ((peak.intensity - peak.baseline) * f) + minY
-            peaklist.append(mspy.peak(mz=peak.mz, intensity=intensity, baseline=minY))
+            intensity = (peak.intensity * f) + minY
+            peaklist.append(mspy.peak(mz=peak.mz, ai=intensity, base=minY))
         
         # convert to mspy.peaklist
         return mspy.peaklist(peaklist)
