@@ -20,9 +20,9 @@ import wx
 
 # load modules
 from ids import *
-import config
 import mwx
 import images
+import config
 import mspy
 import doc
 
@@ -113,7 +113,7 @@ class panelDocuments(wx.Panel):
         self.documentTree.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.onItemActivated)
         
         # set DnD
-        dropTarget = fileDropTarget(self.parent.onDocumentDropping)
+        dropTarget = fileDropTarget(self.parent.onDocumentDropped)
         self.documentTree.SetDropTarget(dropTarget)
     # ----
     
@@ -157,8 +157,13 @@ class panelDocuments(wx.Panel):
         # get item
         item, flags = self.documentTree.HitTest(evt.GetPosition())
         
-        # show/hide document
-        if (flags & wx.TREE_HITTEST_ONITEMICON) and self.documentTree.getItemIndent(item) == 1:
+        # document solo
+        if (evt.AltDown() or evt.ControlDown()) and self.documentTree.getItemIndent(item) == 1:
+            docIndex = self._getDocumentIndex(item)
+            self.parent.onDocumentSolo(docIndex)
+        
+        # enable / disable document
+        elif (flags & wx.TREE_HITTEST_ONITEMICON) and self.documentTree.getItemIndent(item) == 1:
             docIndex = self._getDocumentIndex(item)
             self.parent.onDocumentEnable(docIndex)
         
@@ -201,7 +206,7 @@ class panelDocuments(wx.Panel):
             menu.AppendSeparator()
             menu.Append(ID_documentInfo, "Notes and Information...")
             menu.Append(ID_documentColour, "Change Colour...")
-            menu.Append(ID_documentDeleteNotations, "Delete All Notations")
+            menu.Append(ID_documentNotationsDelete, "Delete All Notations")
             menu.AppendSeparator()
             menu.Append(ID_documentFlip, "Flip Spectrum")
             menu.Append(ID_documentOffset, "Offset Spectrum...")
@@ -217,22 +222,22 @@ class panelDocuments(wx.Panel):
                 menu.Enable(ID_documentClearOffset, False)
         
         elif itemType == 'annotations':
-            menu.Append(ID_documentCalibrateByAnnotations, "Calibrate by Annotations...")
+            menu.Append(ID_documentAnnotationsCalibrateBy, "Calibrate by Annotations...")
             menu.AppendSeparator()
-            menu.Append(ID_documentDeleteAnnotations, "Delete All Annotations")
+            menu.Append(ID_documentAnnotationsDelete, "Delete All Annotations")
             
             if not itemData:
-                menu.Enable(ID_documentDeleteAnnotations, False)
-                menu.Enable(ID_documentCalibrateByAnnotations, False)
+                menu.Enable(ID_documentAnnotationsDelete, False)
+                menu.Enable(ID_documentAnnotationsCalibrateBy, False)
         
         elif itemType == 'annotation':
             menu.Append(ID_documentAnnotationEdit, "Edit Annotation...")
             menu.Append(ID_documentAnnotationPattern, "Show Isotopic Pattern...")
             menu.AppendSeparator()
-            menu.Append(ID_documentCalibrateByAnnotations, "Calibrate by Annotations...")
+            menu.Append(ID_documentAnnotationsCalibrateBy, "Calibrate by Annotations...")
             menu.AppendSeparator()
-            menu.Append(ID_documentDeleteAnnotation, "Delete Annotation")
-            menu.Append(ID_documentDeleteAnnotations, "Delete All Annotations")
+            menu.Append(ID_documentAnnotationDelete, "Delete Annotation")
+            menu.Append(ID_documentAnnotationsDelete, "Delete All Annotations")
             
             if not itemData.formula:
                 menu.Enable(ID_documentAnnotationPattern, False)
@@ -245,42 +250,42 @@ class panelDocuments(wx.Panel):
             menu.Append(ID_sequenceFragment, "Fragment Peptide...")
             menu.Append(ID_sequenceSearch, "Mass Search...")
             menu.AppendSeparator()
-            menu.Append(ID_sequenceCalibrateByMatches, "Calibrate by Matches...")
+            menu.Append(ID_sequenceMatchesCalibrateBy, "Calibrate by Matches...")
             menu.AppendSeparator()
-            menu.Append(ID_sequenceDeleteMatches, "Delete All Matches")
+            menu.Append(ID_sequenceMatchesDelete, "Delete All Matches")
             menu.Append(ID_sequenceDelete, "Delete Sequence")
             
             if not itemData.matches:
-                menu.Enable(ID_sequenceCalibrateByMatches, False)
-                menu.Enable(ID_sequenceDeleteMatches, False)
+                menu.Enable(ID_sequenceMatchesCalibrateBy, False)
+                menu.Enable(ID_sequenceMatchesDelete, False)
         
         elif itemType == 'match':
             menu.Append(ID_sequenceMatchEdit, "Edit Match...")
             menu.Append(ID_sequenceMatchPattern, "Show Isotopic Pattern...")
             menu.AppendSeparator()
-            menu.Append(ID_sequenceCalibrateByMatches, "Calibrate by Matches...")
+            menu.Append(ID_sequenceMatchesCalibrateBy, "Calibrate by Matches...")
             menu.AppendSeparator()
-            menu.Append(ID_sequenceDeleteMatch, "Delete Sequence Match")
-            menu.Append(ID_sequenceDeleteMatches, "Delete All Matches")
+            menu.Append(ID_sequenceMatchDelete, "Delete Sequence Match")
+            menu.Append(ID_sequenceMatchesDelete, "Delete All Matches")
             
             if not itemData.formula:
                 menu.Enable(ID_documentAnnotationPattern, False)
         
         # bind events
-        self.Bind(wx.EVT_MENU, self.parent.onToolsInfo, id=ID_documentInfo)
+        self.Bind(wx.EVT_MENU, self.parent.onDocumentInfo, id=ID_documentInfo)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentColour, id=ID_documentColour)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentFlip, id=ID_documentFlip)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentOffset, id=ID_documentOffset)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentOffset, id=ID_documentClearOffset)
-        self.Bind(wx.EVT_MENU, self.parent.onDocumentDeleteNotations, id=ID_documentDeleteNotations)
+        self.Bind(wx.EVT_MENU, self.parent.onDocumentNotationsDelete, id=ID_documentNotationsDelete)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentDuplicate, id=ID_documentDuplicate)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentClose, id=ID_documentClose)
         
         self.Bind(wx.EVT_MENU, self.onNotationEdit, id=ID_documentAnnotationEdit)
         self.Bind(wx.EVT_MENU, self.onNotationPattern, id=ID_documentAnnotationPattern)
-        self.Bind(wx.EVT_MENU, self.parent.onDocumentCalibrateByAnnotations, id=ID_documentCalibrateByAnnotations)
-        self.Bind(wx.EVT_MENU, self.onNotationDelete, id=ID_documentDeleteAnnotation)
-        self.Bind(wx.EVT_MENU, self.parent.onDocumentDeleteAnnotations, id=ID_documentDeleteAnnotations)
+        self.Bind(wx.EVT_MENU, self.parent.onDocumentAnnotationsCalibrateBy, id=ID_documentAnnotationsCalibrateBy)
+        self.Bind(wx.EVT_MENU, self.onNotationDelete, id=ID_documentAnnotationDelete)
+        self.Bind(wx.EVT_MENU, self.parent.onDocumentAnnotationsDelete, id=ID_documentAnnotationsDelete)
         
         self.Bind(wx.EVT_MENU, self.parent.onSequenceNew, id=ID_sequenceNew)
         self.Bind(wx.EVT_MENU, self.parent.onToolsSequence, id=ID_sequenceEditor)
@@ -292,9 +297,9 @@ class panelDocuments(wx.Panel):
         
         self.Bind(wx.EVT_MENU, self.onNotationEdit, id=ID_sequenceMatchEdit)
         self.Bind(wx.EVT_MENU, self.onNotationPattern, id=ID_sequenceMatchPattern)
-        self.Bind(wx.EVT_MENU, self.parent.onSequenceCalibrateByMatches, id=ID_sequenceCalibrateByMatches)
-        self.Bind(wx.EVT_MENU, self.onNotationDelete, id=ID_sequenceDeleteMatch)
-        self.Bind(wx.EVT_MENU, self.parent.onSequenceDeleteMatches, id=ID_sequenceDeleteMatches)
+        self.Bind(wx.EVT_MENU, self.parent.onSequenceMatchesCalibrateBy, id=ID_sequenceMatchesCalibrateBy)
+        self.Bind(wx.EVT_MENU, self.onNotationDelete, id=ID_sequenceMatchDelete)
+        self.Bind(wx.EVT_MENU, self.parent.onSequenceMatchesDelete, id=ID_sequenceMatchesDelete)
         
         self.PopupMenu(menu)
         menu.Destroy()
@@ -370,7 +375,7 @@ class panelDocuments(wx.Panel):
         
         # document info
         if itemType == 'document':
-            self.parent.onToolsInfo()
+            self.parent.onDocumentInfo()
         
         # sequence editing
         elif itemType == 'sequence':
@@ -393,12 +398,15 @@ class panelDocuments(wx.Panel):
         menu = wx.Menu()
         menu.Append(ID_sequenceNew, "New Sequence...")
         menu.Append(ID_documentNew, "New Document")
+        menu.AppendSeparator()
+        menu.Append(ID_documentOpen, "Open Document...")
         
         menu.Enable(ID_sequenceNew, bool(indent))
         
         # set events
         self.Bind(wx.EVT_MENU, self.parent.onSequenceNew, id=ID_sequenceNew)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentNew, id=ID_documentNew)
+        self.Bind(wx.EVT_MENU, self.parent.onDocumentOpen, id=ID_documentOpen)
         
         self.PopupMenu(menu)
         menu.Destroy()
@@ -411,37 +419,37 @@ class panelDocuments(wx.Panel):
         
         # make menu
         menu = wx.Menu()
-        menu.Append(ID_documentDeleteAnnotation, "Delete Annotation")
-        menu.Append(ID_documentDeleteAnnotations, "Delete All Annotations")
+        menu.Append(ID_documentAnnotationDelete, "Delete Annotation")
+        menu.Append(ID_documentAnnotationsDelete, "Delete All Annotations")
         menu.AppendSeparator()
         menu.Append(ID_sequenceDelete, "Delete Sequence")
-        menu.Append(ID_sequenceDeleteMatch, "Delete Sequence Match")
-        menu.Append(ID_sequenceDeleteMatches, "Delete All Matches")
+        menu.Append(ID_sequenceMatchDelete, "Delete Sequence Match")
+        menu.Append(ID_sequenceMatchesDelete, "Delete All Matches")
         menu.AppendSeparator()
-        menu.Append(ID_documentDeleteNotations, "Delete All Notations")
+        menu.Append(ID_documentNotationsDelete, "Delete All Notations")
         menu.AppendSeparator()
         menu.Append(ID_documentClose, "Close Document")
         menu.Append(ID_documentCloseAll, "Close All Documents")
         
         # disable items
-        menu.Enable(ID_documentDeleteAnnotation, False)
-        menu.Enable(ID_documentDeleteAnnotations, False)
+        menu.Enable(ID_documentAnnotationDelete, False)
+        menu.Enable(ID_documentAnnotationsDelete, False)
         menu.Enable(ID_sequenceDelete, False)
-        menu.Enable(ID_sequenceDeleteMatch, False)
-        menu.Enable(ID_sequenceDeleteMatches, False)
-        menu.Enable(ID_documentDeleteNotations, False)
+        menu.Enable(ID_sequenceMatchDelete, False)
+        menu.Enable(ID_sequenceMatchesDelete, False)
+        menu.Enable(ID_documentNotationsDelete, False)
         menu.Enable(ID_documentClose, False)
         menu.Enable(ID_documentCloseAll, bool(self.documents))
         
         # bind events
-        self.Bind(wx.EVT_MENU, self.onNotationDelete, id=ID_documentDeleteAnnotation)
-        self.Bind(wx.EVT_MENU, self.parent.onDocumentDeleteAnnotations, id=ID_documentDeleteAnnotations)
+        self.Bind(wx.EVT_MENU, self.onNotationDelete, id=ID_documentAnnotationDelete)
+        self.Bind(wx.EVT_MENU, self.parent.onDocumentAnnotationsDelete, id=ID_documentAnnotationsDelete)
         
         self.Bind(wx.EVT_MENU, self.parent.onSequenceDelete, id=ID_sequenceDelete)
-        self.Bind(wx.EVT_MENU, self.onNotationDelete, id=ID_sequenceDeleteMatch)
-        self.Bind(wx.EVT_MENU, self.parent.onSequenceDeleteMatches, id=ID_sequenceDeleteMatches)
+        self.Bind(wx.EVT_MENU, self.onNotationDelete, id=ID_sequenceMatchDelete)
+        self.Bind(wx.EVT_MENU, self.parent.onSequenceMatchesDelete, id=ID_sequenceMatchesDelete)
         
-        self.Bind(wx.EVT_MENU, self.parent.onDocumentDeleteNotations, id=ID_documentDeleteNotations)
+        self.Bind(wx.EVT_MENU, self.parent.onDocumentNotationsDelete, id=ID_documentNotationsDelete)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentClose, id=ID_documentClose)
         self.Bind(wx.EVT_MENU, self.parent.onDocumentCloseAll, id=ID_documentCloseAll)
         
@@ -455,19 +463,19 @@ class panelDocuments(wx.Panel):
             seqIndex = self._getSequenceIndex(item)
             
             if self.documents[docIndex].annotations:
-                menu.Enable(ID_documentDeleteAnnotations, True)
+                menu.Enable(ID_documentAnnotationsDelete, True)
             if itemType == 'annotation':
-                menu.Enable(ID_documentDeleteAnnotation, True)
-                menu.Enable(ID_documentDeleteAnnotations, True)
+                menu.Enable(ID_documentAnnotationDelete, True)
+                menu.Enable(ID_documentAnnotationsDelete, True)
             if itemType == 'sequence':
                 menu.Enable(ID_sequenceDelete, True)
             if itemType == 'sequence' and self.documents[docIndex].sequences[seqIndex].matches:
-                menu.Enable(ID_sequenceDeleteMatches, True)
+                menu.Enable(ID_sequenceMatchesDelete, True)
             if itemType == 'match':
-                menu.Enable(ID_sequenceDeleteMatch, True)
-                menu.Enable(ID_sequenceDeleteMatches, True)
+                menu.Enable(ID_sequenceMatchDelete, True)
+                menu.Enable(ID_sequenceMatchesDelete, True)
             if itemType != None:
-                menu.Enable(ID_documentDeleteNotations, True)
+                menu.Enable(ID_documentNotationsDelete, True)
                 menu.Enable(ID_documentClose, True)
         
         self.PopupMenu(menu)
@@ -509,9 +517,9 @@ class panelDocuments(wx.Panel):
         # show profile
         if itemData.formula:
             if itemData.radical:
-                self.parent.onToolsMasscalc(formula=itemData.formula, charge=itemData.charge, agentFormula='e', agentCharge=-1)
+                self.parent.onToolsMassCalculator(formula=itemData.formula, charge=itemData.charge, agentFormula='e', agentCharge=-1)
             else:
-                self.parent.onToolsMasscalc(formula=itemData.formula, charge=itemData.charge, agentFormula='H', agentCharge=1)
+                self.parent.onToolsMassCalculator(formula=itemData.formula, charge=itemData.charge, agentFormula='H', agentCharge=1)
     # ----
     
     
@@ -526,13 +534,13 @@ class panelDocuments(wx.Panel):
         if itemType == 'annotation':
             annotIndex = self._getAnnotationIndex(item)
             if annotIndex != None:
-                self.parent.onDocumentDeleteAnnotations(annotIndex=annotIndex)
+                self.parent.onDocumentAnnotationsDelete(annotIndex=annotIndex)
         
         # delete sequence match
         elif itemType == 'match':
             matchIndex = self._getMatchIndex(item)
             if matchIndex != None:
-                self.parent.onSequenceDeleteMatches(matchIndex=matchIndex)
+                self.parent.onSequenceMatchesDelete(matchIndex=matchIndex)
     # ----
     
     
@@ -658,7 +666,7 @@ class panelDocuments(wx.Panel):
         
         # add new annotations
         for annotData in annotsData:
-            self.documentTree.appendMatch(annotsItem, annotData)
+            self.documentTree.appendNotation(annotsItem, annotData)
         
         # expand tree
         if expand:
@@ -767,7 +775,7 @@ class panelDocuments(wx.Panel):
         
         # add new matches
         for matchData in seqData.matches:
-            self.documentTree.appendMatch(seqItem, matchData)
+            self.documentTree.appendNotation(seqItem, matchData)
         
         # expand tree
         if expand:
@@ -883,7 +891,7 @@ class panelDocuments(wx.Panel):
 class documentsTree(wx.TreeCtrl):
     """Documents tree."""
     
-    def __init__(self, parent, id, sequence=None, size=(-1,-1), style=mwx.DOCTREE_STYLE):
+    def __init__(self, parent, id, size=(-1,-1), style=mwx.DOCTREE_STYLE):
         wx.TreeCtrl.__init__(self, parent, id, size=size, style=style)
         
         self.parent = parent
@@ -1032,20 +1040,21 @@ class documentsTree(wx.TreeCtrl):
             else:
                 self.SetItemImage(item, 1, wx.TreeItemIcon_Normal)
         
-        # set sequence bullet
+        # set annotations bullet
         elif itemType == 'annotations':
             if enable:
                 self.SetItemImage(item, 2, wx.TreeItemIcon_Normal)
             else:
                 self.SetItemImage(item, 3, wx.TreeItemIcon_Normal)
         
+        # set sequence bullet
         elif itemType == 'sequence':
             if enable:
                 self.SetItemImage(item, 4, wx.TreeItemIcon_Normal)
             else:
                 self.SetItemImage(item, 5, wx.TreeItemIcon_Normal)
         
-        # set match bullet
+        # set match / annotation bullet
         elif itemType == 'match' or  itemType == 'annotation':
             if enable:
                 self.SetItemImage(item, 6, wx.TreeItemIcon_Normal)
@@ -1075,7 +1084,7 @@ class documentsTree(wx.TreeCtrl):
         self.SetItemImage(annotsItem, 2, wx.TreeItemIcon_Normal)
         self.SetPyData(annotsItem, docData.annotations)
         for annotData in docData.annotations:
-            self.appendMatch(annotsItem, annotData)
+            self.appendNotation(annotsItem, annotData)
         
         # add sequences
         for seqData in docData.sequences:
@@ -1098,20 +1107,20 @@ class documentsTree(wx.TreeCtrl):
         
         # add matches
         for matchData in seqData.matches:
-            self.appendMatch(seqItem, matchData)
+            self.appendNotation(seqItem, matchData)
         
         return seqItem
     # ----
     
     
-    def appendMatch(self, item, matchData):
-        """Append match to tree."""
+    def appendNotation(self, item, notationData):
+        """Append notation to tree."""
         
         # get mz
-        mz = round(matchData.mz, config.main['mzDigits'])
+        mz = round(notationData.mz, config.main['mzDigits'])
         
         # get error
-        error = matchData.delta(config.main['errorUnits'])
+        error = notationData.delta(config.main['errorUnits'])
         if error != None and config.main['errorUnits'] == 'ppm':
             error = round(error, config.main['ppmDigits'])
         elif error != None:
@@ -1119,16 +1128,16 @@ class documentsTree(wx.TreeCtrl):
         
         # make label
         if error != None:
-            label = '%s (%s %s) %s' % (mz, error, config.main['errorUnits'], matchData.label)
+            label = '%s (%s %s) %s' % (mz, error, config.main['errorUnits'], notationData.label)
         else:
-            label = '%s %s' % (mz, matchData.label)
+            label = '%s %s' % (mz, notationData.label)
         
         # add match
-        matchItem = self.AppendItem(item, label)
-        self.SetItemImage(matchItem, 6, wx.TreeItemIcon_Normal)
-        self.SetPyData(matchItem, matchData)
+        notationItem = self.AppendItem(item, label)
+        self.SetItemImage(notationItem, 6, wx.TreeItemIcon_Normal)
+        self.SetPyData(notationItem, notationData)
         
-        return matchItem
+        return notationItem
     # ----
     
     
@@ -1174,14 +1183,14 @@ class documentsTree(wx.TreeCtrl):
         """Erase all bullets and make defaults."""
         
         self.bullets.RemoveAll()
-        self.bullets.Add(images.lib['bulletDocuments'])
+        self.bullets.Add(images.lib['bulletsDocument'])
         self.bullets.Add(self._makeColourBullet((150,150,150), False))
-        self.bullets.Add(images.lib['bulletAnnotationsOn'])
-        self.bullets.Add(images.lib['bulletAnnotationsOff'])
-        self.bullets.Add(images.lib['bulletSequenceOn'])
-        self.bullets.Add(images.lib['bulletSequenceOff'])
-        self.bullets.Add(images.lib['bulletMatchOn'])
-        self.bullets.Add(images.lib['bulletMatchOff'])
+        self.bullets.Add(images.lib['bulletsAnnotationsOn'])
+        self.bullets.Add(images.lib['bulletsAnnotationsOff'])
+        self.bullets.Add(images.lib['bulletsSequenceOn'])
+        self.bullets.Add(images.lib['bulletsSequenceOff'])
+        self.bullets.Add(images.lib['bulletsNotationOn'])
+        self.bullets.Add(images.lib['bulletsNotationOff'])
     # ----
     
     
