@@ -27,9 +27,10 @@ import os.path
 
 # load modules
 from ids import *
-import config
 import mwx
 import images
+import config
+import libs
 import mspy
 import doc
 
@@ -82,12 +83,14 @@ class panelMascot(wx.MiniFrame):
         self.mainSizer.Add(query, 1, wx.EXPAND, 0)
         self.mainSizer.Add(gauge, 0, wx.EXPAND, 0)
         
+        # hide panels
         self.mainSizer.Hide(1)
         self.mainSizer.Hide(2)
         self.mainSizer.Hide(3)
         self.mainSizer.Hide(4)
         self.mainSizer.Hide(5)
         
+        # fit layout
         self.mainSizer.Fit(self)
         self.SetSizer(self.mainSizer)
     # ----
@@ -117,10 +120,10 @@ class panelMascot(wx.MiniFrame):
         self.query_butt.Bind(wx.EVT_BUTTON, self.onToolSelected)
         
         server_label = wx.StaticText(panel, -1, "Server:")
-        choices=config.libs.mascot.keys()
+        choices=libs.mascot.keys()
         choices.insert(0, 'Select Server')
         self.server_combo = wx.ComboBox(panel, -1, choices=choices, size=(220, mwx.SMALL_COMBO_HEIGHT), style=wx.CB_READONLY)
-        if config.mascot['common']['server'] in config.libs.mascot:
+        if config.mascot['common']['server'] in libs.mascot:
             self.server_combo.SetValue(config.mascot['common']['server'])
         else:
             self.server_combo.Select(0)
@@ -803,13 +806,13 @@ class panelMascot(wx.MiniFrame):
             self.search_butt.Enable(False)
         
         # fit layout
-        self.SetMinSize((-1,-1))
-        self.mainSizer.Fit(self)
         self.Layout()
+        self.mainSizer.Fit(self)
+        
+        # force resize
         size = self.GetSize()
-        self.SetSize((size[0]+1,size[1]))
+        self.SetSize((size[0], size[1]+1))
         self.SetSize(size)
-        self.SetMinSize(size)
     # ----
     
     
@@ -885,7 +888,7 @@ class panelMascot(wx.MiniFrame):
         # make new query
         query = ''
         for peak in peaklist:
-            query += '%f\t%f\n' % (peak.mz, (peak.intensity-peak.baseline))
+            query += '%f\t%f\n' % (peak.mz, peak.intensity)
         
         # set data
         self.paramQuery_value.SetValue(query)
@@ -954,7 +957,7 @@ class panelMascot(wx.MiniFrame):
         
         # server
         name = config.mascot['common']['server']
-        server = config.libs.mascot[name]
+        server = libs.mascot[name]
         
         # get data from the server
         socket.setdefaulttimeout(5)
@@ -1227,7 +1230,7 @@ class panelMascot(wx.MiniFrame):
         
         # get server params
         name = self.server_combo.GetValue()
-        if name in config.libs.mascot:
+        if name in libs.mascot:
             config.mascot['common']['server'] = name
         else:
             wx.Bell()
@@ -1521,7 +1524,7 @@ class panelMascot(wx.MiniFrame):
         
         # make search path
         name = config.mascot['common']['server']
-        script = 'http://'+config.libs.mascot[name]['host']+config.libs.mascot[name]['path']+config.libs.mascot[name]['search']+'?1'
+        script = 'http://'+libs.mascot[name]['host']+libs.mascot[name]['path']+libs.mascot[name]['search']+'?1'
         
         # make html page
         buff = '<?xml version="1.0" encoding="utf-8"?>\n'
@@ -1550,7 +1553,7 @@ class panelMascot(wx.MiniFrame):
         
         buff += '    <div id="info">\n'
         buff += '      <h1>mMass - Mascot Search</h1>\n'
-        buff += '      <p class="sending">Sending data to server: %s<br />Please wait...</p>\n' % (config.libs.mascot[name]['host'])
+        buff += '      <p class="sending">Sending data to server: %s<br />Please wait...</p>\n' % (libs.mascot[name]['host'])
         buff += '      <p class="note">(Press Search button if data was not send automatically.)</p>\n'
         buff += '      <p class="button"><input type="submit" value="Search" /></p>\n'
         buff += '    </div>\n\n'
