@@ -21,10 +21,11 @@ import wx
 
 # load modules
 from ids import *
-import config
 import mwx
 import images
+import config
 import mspy
+import mspy.plot
 import doc
 
 
@@ -32,13 +33,14 @@ import doc
 # -------------------------------
 
 class panelMatch(wx.MiniFrame):
-    """Match tools."""
+    """Data match tool."""
     
     def __init__(self, parentTool, mainFrame, module):
         wx.MiniFrame.__init__(self, parentTool, -1, 'Match Data', size=(400, 300), style=wx.DEFAULT_FRAME_STYLE & ~ (wx.RESIZE_BOX | wx.MAXIMIZE_BOX))
         
         self.parentTool = parentTool
         self.mainFrame = mainFrame
+        
         self.processing = None
         
         self.currentModule = module
@@ -90,17 +92,14 @@ class panelMatch(wx.MiniFrame):
         self.mainSizer.Add(self.summaryList, 1, wx.EXPAND|wx.ALL, mwx.LISTCTRL_NO_SPACE)
         self.mainSizer.Add(gauge, 0, wx.EXPAND, 0)
         
-        # hide gauge
+        # hide panels
         self.mainSizer.Hide(2)
         self.mainSizer.Hide(3)
         self.mainSizer.Hide(4)
         
         # fit layout
-        self.SetMinSize((100,100))
-        self.Layout()
         self.mainSizer.Fit(self)
         self.SetSizer(self.mainSizer)
-        self.SetMinSize(self.GetSize())
     # ----
     
     
@@ -138,6 +137,8 @@ class panelMatch(wx.MiniFrame):
         self.ignoreCharge_check = wx.CheckBox(panel, -1, "Ignore charge")
         self.ignoreCharge_check.SetFont(wx.SMALL_FONT)
         self.ignoreCharge_check.SetValue(config.match['ignoreCharge'])
+        if self.currentModule == 'massfilter':
+            self.ignoreCharge_check.Disable()
         
         self.match_butt = wx.Button(panel, -1, "Match", size=(-1, mwx.SMALL_BUTTON_HEIGHT))
         self.match_butt.SetFont(wx.SMALL_FONT)
@@ -229,23 +230,23 @@ class panelMatch(wx.MiniFrame):
         """Make plot canvas and set defalt parameters."""
         
         # init canvas
-        self.errorCanvas = mspy.plot.canvas(self, size=(-1, 220), style=mwx.PLOTCANVAS_STYLE_PANEL)
+        self.errorCanvas = mspy.plot.canvas(self, size=(621, 220), style=mwx.PLOTCANVAS_STYLE_PANEL)
         self.errorCanvas.draw(mspy.plot.container([]))
         
         # set default params
         self.errorCanvas.setProperties(xLabel='m/z')
-        self.errorCanvas.setProperties(yLabel='error in %s' % config.match['units'])
+        self.errorCanvas.setProperties(yLabel='error (%s)' % config.match['units'])
         self.errorCanvas.setProperties(showZero=True)
         self.errorCanvas.setProperties(showLegend=False)
         self.errorCanvas.setProperties(showPosBar=True)
         self.errorCanvas.setProperties(posBarHeight=6)
         self.errorCanvas.setProperties(showGel=False)
-        self.errorCanvas.setProperties(showCurTracker=True)
         self.errorCanvas.setProperties(checkLimits=True)
         self.errorCanvas.setProperties(autoScaleY=False)
         self.errorCanvas.setProperties(xPosDigits=config.main['mzDigits'])
         self.errorCanvas.setProperties(yPosDigits=2)
         self.errorCanvas.setProperties(reverseDrawing=True)
+        self.errorCanvas.setMFunction('cross')
         
         axisFont = wx.Font(config.spectrum['axisFontSize'], wx.SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, 0)
         self.errorCanvas.setProperties(axisFont=axisFont)
@@ -280,13 +281,22 @@ class panelMatch(wx.MiniFrame):
         # make elements
         self.gauge = mwx.gauge(panel, -1)
         
+        stop_butt = wx.BitmapButton(panel, -1, images.lib['stopper'], style=wx.BORDER_NONE)
+        stop_butt.Bind(wx.EVT_BUTTON, self.onStop)
+        
         # pack elements
-        mainSizer = wx.BoxSizer(wx.VERTICAL)
-        mainSizer.Add(self.gauge, 0, wx.EXPAND|wx.ALL, mwx.GAUGE_SPACE)
+        sizer = wx.BoxSizer(wx.HORIZONTAL)
+        sizer.Add(self.gauge, 1, wx.ALIGN_CENTER_VERTICAL)
+        sizer.AddSpacer(10)
+        sizer.Add(stop_butt, 0, wx.ALIGN_CENTER_VERTICAL)
         
         # fit layout
-        mainSizer.Fit(panel)
+        mainSizer = wx.BoxSizer(wx.VERTICAL)
+        if wx.Platform == '__WXMAC__':
+            mainSizer.Add(wx.StaticLine(panel), 0, wx.EXPAND|wx.TOP, -1)
+        mainSizer.Add(sizer, 1, wx.EXPAND|wx.ALL, mwx.GAUGE_SPACE)
         panel.SetSizer(mainSizer)
+        mainSizer.Fit(panel)
         
         return panel
     # ----
@@ -338,12 +348,9 @@ class panelMatch(wx.MiniFrame):
         
         # fit layout
         self.SetMinSize((-1,-1))
-        self.mainSizer.Fit(self)
         self.Layout()
-        size = self.GetSize()
-        self.SetSize((size[0]+1,size[1]))
-        self.SetSize(size)
-        self.SetMinSize(size)
+        self.mainSizer.Fit(self)
+        self.SetMinSize(self.GetSize())
     # ----
     
     
@@ -359,6 +366,7 @@ class panelMatch(wx.MiniFrame):
             self.MakeModal(False)
             self.mainSizer.Hide(4)
             self.processing = None
+            mspy.start()
         
         # fit layout
         self.Layout()
@@ -368,6 +376,16 @@ class panelMatch(wx.MiniFrame):
     # ----
     
     
+    def onStop(self, evt):
+        """Cancel current processing."""
+        
+        if self.processing and self.processing.isAlive():
+            mspy.stop()
+        else:
+            wx.Bell()
+    # ----
+    
+    
     def onMatch(self, evt=None):
         """Match data to peaklist."""
         
@@ -481,18 +499,36 @@ class panelMatch(wx.MiniFrame):
         try:
             config.match['tolerance'] = float(self.tolerance_value.GetValue())
             
+            config.match['units'] = 'ppm'
             if self.unitsDa_radio.GetValue():
                 config.match['units'] = 'Da'
-            else:
-                config.match['units'] = 'ppm'
             
+            config.match['ignoreCharge'] = 0
             if self.ignoreCharge_check.GetValue():
                 config.match['ignoreCharge'] = 1
-            else:
-                config.match['ignoreCharge'] = 0
+            
+            config.match['filterAnnotations'] = 0
+            if self.filterAnnotations_check.GetValue():
+                config.match['filterAnnotations'] = 1
+            
+            config.match['filterMatches'] = 0
+            if self.filterMatches_check.GetValue():
+                config.match['filterMatches'] = 1
+            
+            config.match['filterUnselected'] = 0
+            if self.filterUnselected_check.GetValue():
+                config.match['filterUnselected'] = 1
+            
+            config.match['filterIsotopes'] = 0
+            if self.filterIsotopes_check.GetValue():
+                config.match['filterIsotopes'] = 1
+            
+            config.match['filterUnknown'] = 0
+            if self.filterUnknown_check.GetValue():
+                config.match['filterUnknown'] = 1
             
             return True
-            
+        
         except:
             wx.Bell()
             return False
@@ -504,15 +540,15 @@ class panelMatch(wx.MiniFrame):
         
         # get filters
         filters = ''
-        if self.filterAnnotations_check.GetValue():
+        if config.match['filterAnnotations']:
             filters += 'A'
-        if self.filterMatches_check.GetValue():
+        if config.match['filterMatches']:
             filters += 'M'
-        if self.filterUnselected_check.GetValue():
+        if config.match['filterUnselected']:
             filters += 'S'
-        if self.filterIsotopes_check.GetValue():
+        if config.match['filterIsotopes']:
             filters += 'I'
-        if self.filterUnknown_check.GetValue():
+        if config.match['filterUnknown']:
             filters += 'X'
         
         # get peaklist
@@ -523,67 +559,82 @@ class panelMatch(wx.MiniFrame):
     def runMatch(self):
         """Match data to peaklist."""
         
-        # set columns
-        if self.currentModule == 'massfilter':
-            massCol = 1
-            chargeCol = None
-            errorCol = 2
-            matchObject = doc.annotation
-        elif self.currentModule == 'digest':
-            massCol = 2
-            chargeCol = 3
-            errorCol = 5
-            matchObject = doc.match
-        elif self.currentModule == 'fragment':
-            massCol = 3
-            chargeCol = 4
-            errorCol = 6
-            matchObject = doc.match
-        elif self.currentModule == 'compounds':
-            massCol = 1
-            chargeCol = 2
-            errorCol = 5
-            matchObject = doc.annotation
-        
-        # clear previous match
-        for item in self.currentData:
-            item[errorCol] = None
-            item[-1] = []
-        
-        # match data
-        self.currentErrors = []
-        self.currentCalibrationPoints = []
-        
-        digits = '%0.' + `config.main['mzDigits']` + 'f'
-        for pIndex, peak in enumerate(self.currentPeaklist):
-            for x, item in enumerate(self.currentData):
-                
-                # check charge
-                if (chargeCol==None or peak.charge==None or config.match['ignoreCharge'] or (peak.charge and item[chargeCol]==abs(peak.charge))):
+        # run task
+        try:
+            
+            # set columns
+            if self.currentModule == 'massfilter':
+                massCol = 1
+                chargeCol = None
+                errorCol = 2
+                matchObject = doc.annotation
+            elif self.currentModule == 'digest':
+                massCol = 2
+                chargeCol = 3
+                errorCol = 5
+                matchObject = doc.match
+            elif self.currentModule == 'fragment':
+                massCol = 3
+                chargeCol = 4
+                errorCol = 6
+                matchObject = doc.match
+            elif self.currentModule == 'compounds':
+                massCol = 1
+                chargeCol = 2
+                errorCol = 5
+                matchObject = doc.annotation
+            
+            # clear previous match
+            for item in self.currentData:
+                item[errorCol] = None
+                item[-1] = []
+            
+            # match data
+            self.currentErrors = []
+            self.currentCalibrationPoints = []
+            
+            digits = '%0.' + `config.main['mzDigits']` + 'f'
+            for pIndex, peak in enumerate(self.currentPeaklist):
+                for x, item in enumerate(self.currentData):
                     
-                    # check mass tolerance
-                    error = mspy.delta(peak.mz, item[massCol], config.match['units'])
-                    if abs(error) <= config.match['tolerance']:
+                    mspy.CHECK_FORCE_QUIT()
+                    
+                    # check charge
+                    if (chargeCol==None or peak.charge==None or config.match['ignoreCharge'] or (peak.charge and item[chargeCol]==abs(peak.charge))):
                         
-                        # create new match object
-                        match = matchObject(label='', mz=peak.mz, intensity=peak.intensity, baseline=peak.baseline, theoretical=item[massCol])
-                        match.peakIndex = pIndex
-                        self.currentData[x][-1].append(match)
-                        
-                        # errors and calibration points
-                        label = 'Peak ' + digits % peak.mz
-                        self.currentErrors.append([peak.mz, error])
-                        self.currentCalibrationPoints.append([label, item[massCol], peak.mz])
+                        # check mass tolerance
+                        error = mspy.delta(peak.mz, item[massCol], config.match['units'])
+                        if abs(error) <= config.match['tolerance']:
+                            
+                            # create new match object
+                            match = matchObject(label='', mz=peak.mz, ai=peak.ai, base=peak.base, theoretical=item[massCol])
+                            match.peakIndex = pIndex
+                            self.currentData[x][-1].append(match)
+                            
+                            # errors and calibration points
+                            label = 'Peak ' + digits % peak.mz
+                            self.currentErrors.append([peak.mz, error])
+                            self.currentCalibrationPoints.append([label, item[massCol], peak.mz])
+            
+            # show best error only
+            for item in self.currentData:
+                for match in item[-1]:
+                    error = match.delta(config.match['units'])
+                    if item[errorCol] == None or abs(item[errorCol]) > abs(error):
+                        item[errorCol] = error
+            
+            # get match summary
+            self.makeMatchSummary()
         
-        # show best error only
-        for item in self.currentData:
-            for match in item[-1]:
-                error = match.delta(config.match['units'])
-                if item[errorCol] == None or abs(item[errorCol]) > abs(error):
-                    item[errorCol] = error
-        
-        # get match summary
-        self.makeMatchSummary()
+        # task canceled
+        except mspy.ForceQuit:
+            self.currentErrors = []
+            self.currentCalibrationPoints = []
+            self.currentSummary = []
+            for item in self.currentData:
+                item[errorCol] = None
+                item[-1] = []
+            return
     # ----
     
     
@@ -618,7 +669,7 @@ class panelMatch(wx.MiniFrame):
             container.append(peaklist)
         
         # set units
-        self.errorCanvas.setProperties(yLabel='error in %s' % (config.match['units']))
+        self.errorCanvas.setProperties(yLabel='error (%s)' % (config.match['units']))
         
         # draw container
         self.errorCanvas.draw(container)
@@ -667,11 +718,11 @@ class panelMatch(wx.MiniFrame):
         
         # convert peaklist
         peaklist = []
-        basePeak = self.currentPeaklist.basePeak
-        f = abs(maxY - minY) / (basePeak.intensity - basePeak.baseline)
+        basePeak = self.currentPeaklist.basepeak()
+        f = abs(maxY - minY) / basePeak.intensity
         for peak in self.currentPeaklist:
-            intensity = ((peak.intensity - peak.baseline) * f) + minY
-            peaklist.append(mspy.peak(mz=peak.mz, intensity=intensity, baseline=minY))
+            intensity = (peak.intensity * f) + minY
+            peaklist.append(mspy.peak(mz=peak.mz, ai=intensity, base=minY))
         
         # convert to mspy.peaklist
         return mspy.peaklist(peaklist)
