@@ -20,9 +20,9 @@ import wx
 
 # load modules
 from ids import *
-import config
 import mwx
 import images
+import config
 import mspy
 import doc
 
@@ -126,36 +126,91 @@ class panelPeaklist(wx.Panel):
         """Make peaklist list."""
         
         # init peaklist
-        self.peakList = mwx.sortListCtrl(self, -1, size=(191, -1), style=mwx.LISTCTRL_STYLE_MULTI)
+        self.peakList = mwx.sortListCtrl(self, -1, size=(201, -1), style=mwx.LISTCTRL_STYLE_MULTI)
         self.peakList.SetFont(wx.SMALL_FONT)
         self.peakList.setSecondarySortColumn(0)
         self.peakList.setAltColour(mwx.LISTCTRL_ALTCOLOUR)
         
         # set events
+        self.peakList.Bind(wx.EVT_LIST_COL_RIGHT_CLICK, self.onColumnRMU)
         self.peakList.Bind(wx.EVT_LIST_ITEM_SELECTED, self.onItemSelected)
         self.peakList.Bind(wx.EVT_LIST_ITEM_ACTIVATED, self.onItemActivated)
         self.peakList.Bind(wx.EVT_KEY_DOWN, self.onListKey)
         
         # make columns
-        self.peakList.InsertColumn(0, "m/z", wx.LIST_FORMAT_RIGHT)
-        self.peakList.InsertColumn(1, "int.", wx.LIST_FORMAT_RIGHT)
-        self.peakList.InsertColumn(2, "r. int.", wx.LIST_FORMAT_RIGHT)
-        self.peakList.InsertColumn(3, "z", wx.LIST_FORMAT_RIGHT)
-        self.peakList.InsertColumn(4, "s/n", wx.LIST_FORMAT_RIGHT)
-        self.peakList.InsertColumn(5, "base", wx.LIST_FORMAT_RIGHT)
-        self.peakList.InsertColumn(6, "fwhm", wx.LIST_FORMAT_RIGHT)
-        self.peakList.InsertColumn(7, "resol.", wx.LIST_FORMAT_RIGHT)
-        
-        # set column widths
-        for col, width in enumerate((90,80,55,30,50,60,60,60)):
-            self.peakList.SetColumnWidth(col, width)
+        self.makePeakListColumns()
         
         # set DnD
-        dropTarget = fileDropTarget(self.parent.onDocumentDropping)
+        dropTarget = fileDropTarget(self.parent.onDocumentDropped)
         self.peakList.SetDropTarget(dropTarget)
     # ----
     
     
+    def makePeakListColumns(self):
+        """Make peaklist columns according to config."""
+        
+        # delete current columns
+        self.peakList.deleteColumns()
+        
+        # add new columns
+        x = 0
+        for column in config.main['peaklistColumns']:
+            
+            if column == 'mz':
+                self.peakList.InsertColumn(x, "m/z", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 90)
+                x += 1
+            
+            elif column == 'ai':
+                self.peakList.InsertColumn(x, "a.i.", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 90)
+                x += 1
+            
+            elif column == 'base':
+                self.peakList.InsertColumn(x, "base", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 60)
+                x += 1
+            
+            elif column == 'int':
+                self.peakList.InsertColumn(x, "int.", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 90)
+                x += 1
+            
+            elif column == 'rel':
+                self.peakList.InsertColumn(x, "r. int.", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 55)
+                x += 1
+            
+            elif column == 'sn':
+                self.peakList.InsertColumn(x, "s/n", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 50)
+                x += 1
+            
+            elif column == 'z':
+                self.peakList.InsertColumn(x, "z", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 30)
+                x += 1
+            
+            elif column == 'mass':
+                self.peakList.InsertColumn(x, "mass", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 90)
+                x += 1
+            
+            elif column == 'fwhm':
+                self.peakList.InsertColumn(x, "fwhm", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 60)
+                x += 1
+            
+            elif column == 'resol':
+                self.peakList.InsertColumn(x, "resol.", wx.LIST_FORMAT_RIGHT)
+                self.peakList.SetColumnWidth(x, 60)
+                x += 1
+            
+            else:
+                continue
+    # ----
+    
+    
     def makeEditor(self):
         """Make panel for peak editing."""
         
@@ -163,30 +218,35 @@ class panelPeaklist(wx.Panel):
         panel = mwx.bgrPanel(self, -1, images.lib['bgrPeakEditor'])
         
         # make elements
-        peakMZ_label = wx.StaticText(panel, -1, "m/z:")
-        self.peakMZ_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), validator=mwx.validator('floatPos'))
-        peakMZ_label.SetFont(wx.SMALL_FONT)
-        self.peakMZ_value.SetFont(wx.SMALL_FONT)
+        peakMz_label = wx.StaticText(panel, -1, "m/z:")
+        self.peakMz_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), validator=mwx.validator('floatPos'))
+        peakMz_label.SetFont(wx.SMALL_FONT)
+        self.peakMz_value.SetFont(wx.SMALL_FONT)
         
-        peakIntensity_label = wx.StaticText(panel, -1, "a.i.:")
-        self.peakIntensity_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), validator=mwx.validator('floatPos'))
-        peakIntensity_label.SetFont(wx.SMALL_FONT)
-        self.peakIntensity_value.SetFont(wx.SMALL_FONT)
+        peakAi_label = wx.StaticText(panel, -1, "a.i.:")
+        self.peakAi_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), validator=mwx.validator('floatPos'))
+        peakAi_label.SetFont(wx.SMALL_FONT)
+        self.peakAi_value.SetFont(wx.SMALL_FONT)
         
-        peakCharge_label = wx.StaticText(panel, -1, "charge:")
-        self.peakCharge_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), validator=mwx.validator('int'))
-        peakCharge_label.SetFont(wx.SMALL_FONT)
-        self.peakCharge_value.SetFont(wx.SMALL_FONT)
+        peakBase_label = wx.StaticText(panel, -1, "base:")
+        self.peakBase_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), style=wx.TE_PROCESS_ENTER, validator=mwx.validator('floatPos'))
+        peakBase_label.SetFont(wx.SMALL_FONT)
+        self.peakBase_value.SetFont(wx.SMALL_FONT)
         
         peakSN_label = wx.StaticText(panel, -1, "s/n:")
         self.peakSN_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), validator=mwx.validator('floatPos'))
         peakSN_label.SetFont(wx.SMALL_FONT)
         self.peakSN_value.SetFont(wx.SMALL_FONT)
         
-        peakBaseline_label = wx.StaticText(panel, -1, "base:")
-        self.peakBaseline_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), style=wx.TE_PROCESS_ENTER, validator=mwx.validator('floatPos'))
-        peakBaseline_label.SetFont(wx.SMALL_FONT)
-        self.peakBaseline_value.SetFont(wx.SMALL_FONT)
+        peakCharge_label = wx.StaticText(panel, -1, "charge:")
+        self.peakCharge_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), validator=mwx.validator('int'))
+        peakCharge_label.SetFont(wx.SMALL_FONT)
+        self.peakCharge_value.SetFont(wx.SMALL_FONT)
+        
+        peakFwhm_label = wx.StaticText(panel, -1, "fwhm:")
+        self.peakFwhm_value = wx.TextCtrl(panel, -1, '', size=(80, mwx.SMALL_TEXTCTRL_HEIGHT), style=wx.TE_PROCESS_ENTER, validator=mwx.validator('floatPos'))
+        peakFwhm_label.SetFont(wx.SMALL_FONT)
+        self.peakFwhm_value.SetFont(wx.SMALL_FONT)
         
         self.peakMonoisotopic_check = wx.CheckBox(panel, -1, "monoisotopic")
         self.peakMonoisotopic_check.SetValue(True)
@@ -196,24 +256,26 @@ class panelPeaklist(wx.Panel):
         self.peakAdd_butt.SetFont(wx.SMALL_FONT)
         self.peakAdd_butt.Bind(wx.EVT_BUTTON, self.onAddPeak)
         
-        self.peakReplace_butt = wx.Button(panel, -1, "Replace", size=(-1, mwx.SMALL_BUTTON_HEIGHT))
+        self.peakReplace_butt = wx.Button(panel, -1, "Update", size=(-1, mwx.SMALL_BUTTON_HEIGHT))
         self.peakReplace_butt.SetFont(wx.SMALL_FONT)
         self.peakReplace_butt.Bind(wx.EVT_BUTTON, self.onReplacePeak)
         self.peakReplace_butt.Enable(False)
         
         # pack elements
         grid = wx.GridBagSizer(mwx.GRIDBAG_VSPACE, mwx.GRIDBAG_HSPACE)
-        grid.Add(peakMZ_label, (0,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
-        grid.Add(self.peakMZ_value, (0,1), flag=wx.EXPAND)
-        grid.Add(peakIntensity_label, (1,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
-        grid.Add(self.peakIntensity_value, (1,1), flag=wx.EXPAND)
-        grid.Add(peakCharge_label, (2,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
-        grid.Add(self.peakCharge_value, (2,1), flag=wx.EXPAND)
+        grid.Add(peakMz_label, (0,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.peakMz_value, (0,1), flag=wx.EXPAND)
+        grid.Add(peakAi_label, (1,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.peakAi_value, (1,1), flag=wx.EXPAND)
+        grid.Add(peakBase_label, (2,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.peakBase_value, (2,1), flag=wx.EXPAND)
         grid.Add(peakSN_label, (3,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
         grid.Add(self.peakSN_value, (3,1), flag=wx.EXPAND)
-        grid.Add(peakBaseline_label, (4,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
-        grid.Add(self.peakBaseline_value, (4,1), flag=wx.EXPAND)
-        grid.Add(self.peakMonoisotopic_check, (5,1), flag=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(peakCharge_label, (4,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.peakCharge_value, (4,1), flag=wx.EXPAND)
+        grid.Add(peakFwhm_label, (5,0), flag=wx.ALIGN_RIGHT|wx.ALIGN_CENTER_VERTICAL)
+        grid.Add(self.peakFwhm_value, (5,1), flag=wx.EXPAND)
+        grid.Add(self.peakMonoisotopic_check, (6,1), flag=wx.ALIGN_LEFT|wx.ALIGN_CENTER_VERTICAL)
         grid.AddGrowableCol(1)
         
         buttons = wx.BoxSizer(wx.HORIZONTAL)
@@ -257,6 +319,49 @@ class panelPeaklist(wx.Panel):
     # ----
     
     
+    def onColumnRMU(self, evt):
+        """Show column popup menu."""
+        
+        menu = wx.Menu()
+        menu.Append(ID_viewPeaklistColumnMz, "m/z", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnAi, "a.i", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnBase, "Baseline", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnInt, "Intensity", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnRel, "Rel. Intensity", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnSn, "s/n", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnZ, "Charge", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnMass, "Mass", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnFwhm, "FWHM", "", wx.ITEM_CHECK)
+        menu.Append(ID_viewPeaklistColumnResol, "Resolution", "", wx.ITEM_CHECK)
+        
+        menu.Check(ID_viewPeaklistColumnMz, bool('mz' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnAi, bool('ai' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnBase, bool('base' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnInt, bool('int' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnRel, bool('rel' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnSn, bool('sn' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnZ, bool('z' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnMass, bool('mass' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnFwhm, bool('fwhm' in config.main['peaklistColumns']))
+        menu.Check(ID_viewPeaklistColumnResol, bool('resol' in config.main['peaklistColumns']))
+        
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnMz)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnAi)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnBase)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnInt)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnRel)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnSn)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnZ)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnMass)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnFwhm)
+        self.Bind(wx.EVT_MENU, self.parent.onViewPeaklistColumns, id=ID_viewPeaklistColumnResol)
+        
+        self.PopupMenu(menu)
+        menu.Destroy()
+        self.SetFocus()
+    # ----
+    
+    
     def onListKey(self, evt):
         """Key pressed."""
         
@@ -335,7 +440,7 @@ class panelPeaklist(wx.Panel):
         
         # make annotation
         peak = self.currentDocument.spectrum.peaklist[self.selectedPeak]
-        annot = doc.annotation(label='', mz=peak.mz, intensity=peak.intensity, baseline=peak.baseline, charge=peak.charge)
+        annot = doc.annotation(label='', mz=peak.mz, ai=peak.ai, base=peak.base, charge=peak.charge)
         
         # get annotation label
         dlg = dlgNotation(self.parent, annot, button='Add')
@@ -405,14 +510,26 @@ class panelPeaklist(wx.Panel):
             
             indexes = []
             
-            # use absolute intensity
-            if thresholdType == 'Absolute int.':
+            # use m/z
+            if thresholdType == 'm/z':
                 for i, peak in enumerate(self.currentDocument.spectrum.peaklist):
-                    if peak.intensity - peak.baseline < threshold:
+                    if peak.mz < threshold:
+                        indexes.append(i)
+            
+            # use a.i.
+            elif thresholdType == 'a.i.':
+                for i, peak in enumerate(self.currentDocument.spectrum.peaklist):
+                    if peak.ai < threshold:
+                        indexes.append(i)
+            
+            # use intensity
+            if thresholdType == 'Intensity':
+                for i, peak in enumerate(self.currentDocument.spectrum.peaklist):
+                    if peak.intensity < threshold:
                         indexes.append(i)
             
             # use relative intensity
-            elif thresholdType == 'Relative int.':
+            elif thresholdType == 'Relative Intensity':
                 threshold /= 100
                 for i, peak in enumerate(self.currentDocument.spectrum.peaklist):
                     if peak.relIntensity < threshold:
@@ -424,12 +541,6 @@ class panelPeaklist(wx.Panel):
                     if peak.sn != None and peak.sn < threshold:
                         indexes.append(i)
             
-            # use m/z
-            elif thresholdType == 'm/z':
-                for i, peak in enumerate(self.currentDocument.spectrum.peaklist):
-                    if peak.mz < threshold:
-                        indexes.append(i)
-            
             # delete peaks
             if indexes:
                 self.currentDocument.backup(('spectrum'))
@@ -452,7 +563,7 @@ class panelPeaklist(wx.Panel):
         
         # empty peaklist
         self.currentDocument.backup(('spectrum'))
-        self.currentDocument.spectrum.peaklist.delete()
+        self.currentDocument.spectrum.peaklist.empty()
         self.parent.onDocumentChanged(('spectrum'))
     # ----
     
@@ -471,6 +582,9 @@ class panelPeaklist(wx.Panel):
             self.currentDocument.backup(('spectrum'))
             self.currentDocument.spectrum.peaklist.append(peak)
             self.parent.onDocumentChanged(('spectrum'))
+        
+        # set focus to mz
+        self.peakMz_value.SetFocus()
     # ----
     
     
@@ -487,7 +601,6 @@ class panelPeaklist(wx.Panel):
         if peak:
             self.currentDocument.backup(('spectrum'))
             self.currentDocument.spectrum.peaklist[self.selectedPeak] = peak
-            self.currentDocument.spectrum.peaklist.refresh()
             self.parent.onDocumentChanged(('spectrum'))
     # ----
     
@@ -503,6 +616,15 @@ class panelPeaklist(wx.Panel):
     # ----
     
     
+    def updatePeaklistColumns(self):
+        """Set new peaklist columns and update."""
+        
+        self.makePeakListColumns()
+        self.Layout()
+        self.updatePeakList()
+    # ----
+    
+    
     def updatePeakList(self):
         """Refresh peaklist."""
         
@@ -528,16 +650,33 @@ class panelPeaklist(wx.Panel):
         # set new data map
         self.peakListMap = []
         for peak in self.currentDocument.spectrum.peaklist:
-            self.peakListMap.append((
-                peak.mz,
-                peak.intensity-peak.baseline,
-                peak.relIntensity*100,
-                peak.charge,
-                peak.sn,
-                peak.baseline,
-                peak.fwhm,
-                peak.resolution()
-            ))
+            row = []
+            for column in config.main['peaklistColumns']:
+                if column == 'mz':
+                    row.append(peak.mz)
+                elif column == 'ai':
+                    row.append(peak.ai)
+                elif column == 'base':
+                    row.append(peak.base)
+                elif column == 'int':
+                    row.append(peak.intensity)
+                elif column == 'rel':
+                    row.append(peak.relIntensity*100)
+                elif column == 'sn':
+                    row.append(peak.sn)
+                elif column == 'z':
+                    row.append(peak.charge)
+                elif column == 'mass':
+                    row.append(peak.mass())
+                elif column == 'fwhm':
+                    row.append(peak.fwhm)
+                elif column == 'resol':
+                    row.append(peak.resolution)
+                else:
+                    continue
+            
+            self.peakListMap.append((row))
+            
         self.peakList.setDataMap(self.peakListMap)
         
         # add new data
@@ -549,7 +688,7 @@ class panelPeaklist(wx.Panel):
         self.peakList.sort()
         
         # scroll top
-        if len(self.currentDocument.spectrum.peaklist)>0:
+        if len(self.currentDocument.spectrum.peaklist) > 0:
             self.peakList.EnsureVisible(0)
     # ----
     
@@ -562,45 +701,59 @@ class panelPeaklist(wx.Panel):
         intFormat = '%0.' + `config.main['intDigits']` + 'f'
         fwhmFormat = '%0.' + `max(config.main['mzDigits'],3)` + 'f'
         
-        # format data
-        mz = mzFormat % (item[0])
-        intensity = intFormat % (item[1])
-        
-        relint = ""
-        if item[2]:
-            relint = '%0.2f' % (item[2])
-        
-        charge = ""
-        if item[3] != None:
-            charge = str(item[3])
-        
-        sn = ""
-        if item[4] != None:
-            sn = '%0.1f' % (item[4])
+        # insert data
+        x = 0
+        for column in config.main['peaklistColumns']:
             
-        base = intFormat % (item[5])
-        
-        fwhm = ""
-        if item[6]:
-            fwhm = fwhmFormat % (item[6])
-        
-        resolution = ""
-        if item[7]:
-            resolution = '%0.0f' % (item[7])
-        
-        # add data
-        if insert:
-            self.peakList.InsertStringItem(row, mz)
-        else:
-            self.peakList.SetStringItem(row, 0, mz)
-        
-        self.peakList.SetStringItem(row, 1, intensity)
-        self.peakList.SetStringItem(row, 2, relint)
-        self.peakList.SetStringItem(row, 3, charge)
-        self.peakList.SetStringItem(row, 4, sn)
-        self.peakList.SetStringItem(row, 5, base)
-        self.peakList.SetStringItem(row, 6, fwhm)
-        self.peakList.SetStringItem(row, 7, resolution)
+            # get data
+            data = ''
+            
+            if column == 'mz':
+                data = mzFormat % (item[x])
+            
+            elif column == 'ai':
+                data = intFormat % (item[x])
+            
+            elif column == 'base':
+                data = intFormat % (item[x])
+            
+            elif column == 'int':
+                data = intFormat % (item[x])
+            
+            elif column == 'rel':
+                if item[x]:
+                    data = '%0.2f' % (item[x])
+            
+            elif column == 'sn':
+                if item[x] != None:
+                    data = '%0.1f' % (item[x])
+            
+            elif column == 'z':
+                if item[x] != None:
+                    data = str(item[x])
+            
+            elif column == 'mass':
+                if item[x]:
+                    data = mzFormat % (item[x])
+            
+            elif column == 'fwhm':
+                if item[x]:
+                    data = fwhmFormat % (item[x])
+            
+            elif column == 'resol':
+                if item[x]:
+                    data = '%0.0f' % (item[x])
+            
+            else:
+                continue
+            
+            # add data
+            if x== 0 and insert:
+                self.peakList.InsertStringItem(row, data)
+            else:
+                self.peakList.SetStringItem(row, x, data)
+            
+            x += 1
     # ----
     
     
@@ -608,23 +761,26 @@ class panelPeaklist(wx.Panel):
         """Refresh peak editing panel."""
         
         # empty data
-        self.peakMZ_value.SetValue('')
-        self.peakIntensity_value.SetValue('')
-        self.peakCharge_value.SetValue('')
+        self.peakMz_value.SetValue('')
+        self.peakAi_value.SetValue('')
+        self.peakBase_value.SetValue('')
         self.peakSN_value.SetValue('')
-        self.peakBaseline_value.SetValue('')
+        self.peakCharge_value.SetValue('')
+        self.peakFwhm_value.SetValue('')
         self.peakMonoisotopic_check.SetValue(True)
         self.peakReplace_butt.Enable(False)
         
         # set peak data
         if peak:
-            self.peakMZ_value.SetValue(str(peak.mz))
-            self.peakIntensity_value.SetValue(str(peak.intensity))
+            self.peakMz_value.SetValue(str(round(peak.mz,6)))
+            self.peakAi_value.SetValue(str(peak.ai))
+            self.peakBase_value.SetValue(str(peak.base))
+            if peak.sn:
+                self.peakSN_value.SetValue(str(round(peak.sn,3)))
             if peak.charge:
                 self.peakCharge_value.SetValue(str(peak.charge))
-            if peak.sn:
-                self.peakSN_value.SetValue(str(peak.sn))
-            self.peakBaseline_value.SetValue(str(peak.baseline))
+            if peak.fwhm:
+                self.peakFwhm_value.SetValue(str(round(peak.fwhm,6)))
             if peak.isotope == 0:
                 self.peakMonoisotopic_check.SetValue(True)
             else:
@@ -638,21 +794,22 @@ class panelPeaklist(wx.Panel):
         
         try:
             # get data
-            mz = self.peakMZ_value.GetValue()
-            intensity = self.peakIntensity_value.GetValue()
-            baseline = self.peakBaseline_value.GetValue()
+            mz = self.peakMz_value.GetValue()
+            ai = self.peakAi_value.GetValue()
+            base = self.peakBase_value.GetValue()
             sn = self.peakSN_value.GetValue()
             charge = self.peakCharge_value.GetValue()
+            fwhm = self.peakFwhm_value.GetValue()
             monoisotope = self.peakMonoisotopic_check.GetValue()
             
             # format data
             mz = float(mz)
-            intensity = float(intensity)
+            ai = float(ai)
             
-            if baseline:
-                baseline = float(baseline)
+            if base:
+                base = float(base)
             else:
-                baseline = 0.
+                base = 0.
             
             if sn:
                 sn = float(sn)
@@ -664,13 +821,18 @@ class panelPeaklist(wx.Panel):
             else:
                 charge = None
             
+            if fwhm:
+                fwhm = float(fwhm)
+            else:
+                fwhm = None
+            
             if monoisotope:
                 isotope = 0
             else:
                 isotope = None
             
             # make peak
-            peak = mspy.peak(mz=mz, intensity=intensity, baseline=baseline, sn=sn, charge=charge, isotope=isotope)
+            peak = mspy.peak(mz=mz, ai=ai, base=base, sn=sn, charge=charge, isotope=isotope, fwhm=fwhm)
             return peak
         
         except:
@@ -703,16 +865,35 @@ class panelPeaklist(wx.Panel):
         # show copy dialog
         dlg = dlgCopy(self.parent)
         if dlg.ShowModal() == wx.ID_OK:
-            columns = dlg.getData()
+            dlg.getData()
             dlg.Destroy()
             
-            # get data
+            # export data
             buff = ''
             for row in selection:
+                peak = self.currentDocument.spectrum.peaklist[self.peakList.GetItemData(row)]
+                
                 line = ''
-                for col in columns:
-                    item = self.peakList.GetItem(row, col)
-                    line += item.GetText() + '\t'
+                if 'mz' in config.export['peaklistColumns']:
+                    line += str(peak.mz) + '\t'
+                if 'ai' in config.export['peaklistColumns']:
+                    line += str(peak.ai) + '\t'
+                if 'base' in config.export['peaklistColumns']:
+                    line += str(peak.base) + '\t'
+                if 'int' in config.export['peaklistColumns']:
+                    line += str(peak.intensity) + '\t'
+                if 'rel' in config.export['peaklistColumns']:
+                    line += str(peak.relIntensity*100) + '\t'
+                if 'sn' in config.export['peaklistColumns']:
+                    line += str(peak.sn) + '\t'
+                if 'z' in config.export['peaklistColumns']:
+                    line += str(peak.charge) + '\t'
+                if 'mass' in config.export['peaklistColumns']:
+                    line += str(peak.mass()) + '\t'
+                if 'fwhm' in config.export['peaklistColumns']:
+                    line += str(peak.fwhm) + '\t'
+                if 'resol' in config.export['peaklistColumns']:
+                    line += str(peak.resolution) + '\t'
                 buff += '%s\n' % (line.rstrip())
             
             # make text object for data
@@ -728,13 +909,14 @@ class panelPeaklist(wx.Panel):
             dlg.Destroy()
     # ----
     
+    
 
 
 class dlgThreshold(wx.Dialog):
     """Set threshold."""
     
     def __init__(self, parent):
-        wx.Dialog.__init__(self, parent, -1, "Threshold", style=wx.DEFAULT_DIALOG_STYLE|wx.STAY_ON_TOP)
+        wx.Dialog.__init__(self, parent, -1, "Delete by Threshold", style=wx.DEFAULT_DIALOG_STYLE|wx.STAY_ON_TOP)
         
         self.parent = parent
         self.threshold = None
@@ -757,17 +939,18 @@ class dlgThreshold(wx.Dialog):
         
         # make elements
         threshold_label = wx.StaticText(self, -1, "Minimal value:")
-        self.threshold_value = wx.TextCtrl(self, -1, "", size=(120, -1), validator=mwx.validator('floatPos'))
+        self.threshold_value = wx.TextCtrl(self, -1, "", size=(150, -1), validator=mwx.validator('floatPos'))
         self.threshold_value.Bind(wx.EVT_TEXT, self.onChange)
         
         thresholdType_label = wx.StaticText(self, -1, "Threshold type:")
-        choices=['Absolute int.', 'Relative int.', 's/n', 'm/z']
-        self.thresholdType_combo = wx.ComboBox(self, -1, choices=choices, size=(120, mwx.COMBO_HEIGHT), style=wx.CB_READONLY)
+        choices=['m/z', 'a.i.', 'Intensity', 'Relative Intensity', 's/n']
+        self.thresholdType_combo = wx.ComboBox(self, -1, choices=choices, size=(150, mwx.COMBO_HEIGHT), style=wx.CB_READONLY)
         self.thresholdType_combo.Select(0)
         
         cancel_butt = wx.Button(self, wx.ID_CANCEL, "Cancel")
-        self.delete_butt = wx.Button(self, wx.ID_OK, "Delete")
-        self.delete_butt.Bind(wx.EVT_BUTTON, self.onDelete)
+        delete_butt = wx.Button(self, wx.ID_OK, "Delete")
+        delete_butt.Bind(wx.EVT_BUTTON, self.onDelete)
+        delete_butt.SetDefault()
         
         # pack elements
         grid = wx.GridBagSizer(mwx.GRIDBAG_VSPACE, mwx.GRIDBAG_HSPACE)
@@ -780,7 +963,7 @@ class dlgThreshold(wx.Dialog):
         
         buttSizer = wx.BoxSizer(wx.HORIZONTAL)
         buttSizer.Add(cancel_butt, 0, wx.RIGHT, 15)
-        buttSizer.Add(self.delete_butt, 0)
+        buttSizer.Add(delete_butt, 0)
         
         mainSizer = wx.BoxSizer(wx.VERTICAL)
         mainSizer.Add(staticSizer, 0, wx.CENTER|wx.ALL, mwx.PANEL_SPACE_MAIN)
@@ -817,13 +1000,13 @@ class dlgThreshold(wx.Dialog):
         return self.threshold, self.thresholdType_combo.GetValue()
     # ----
     
-
+    
 
 class dlgCopy(wx.Dialog):
     """Set coumns to copy."""
     
     def __init__(self, parent):
-        wx.Dialog.__init__(self, parent, -1, "Select Columns", style=wx.DEFAULT_DIALOG_STYLE|wx.STAY_ON_TOP)
+        wx.Dialog.__init__(self, parent, -1, "Select Columns to Copy", style=wx.DEFAULT_DIALOG_STYLE|wx.STAY_ON_TOP)
         
         self.parent = parent
         
@@ -844,64 +1027,92 @@ class dlgCopy(wx.Dialog):
         staticSizer = wx.StaticBoxSizer(wx.StaticBox(self, -1, ""), wx.VERTICAL)
         
         # make elements
-        self.colMZ_check = wx.CheckBox(self, -1, "M/Z")
-        self.colIntensity_check = wx.CheckBox(self, -1, "Intensity")
-        self.colRelIntensity_check = wx.CheckBox(self, -1, "Relative intensity")
-        self.colCharge_check = wx.CheckBox(self, -1, "Charge")
-        self.colSN_check = wx.CheckBox(self, -1, "S/N ratio")
-        self.colBaseline_check = wx.CheckBox(self, -1, "Baseline")
-        self.colFWHM_check = wx.CheckBox(self, -1, "FWHM")
-        self.colResolution_check = wx.CheckBox(self, -1, "Resolution")
+        self.peaklistColumnMz_check = wx.CheckBox(self, -1, "m/z")
+        self.peaklistColumnMz_check.SetValue(config.export['peaklistColumns'].count('mz'))
         
-        self.colMZ_check.SetValue(True)
-        self.colIntensity_check.SetValue(True)
+        self.peaklistColumnAi_check = wx.CheckBox(self, -1, "a.i.")
+        self.peaklistColumnAi_check.SetValue(config.export['peaklistColumns'].count('ai'))
         
-        self.copy_butt = wx.Button(self, wx.ID_OK, "Copy")
+        self.peaklistColumnBase_check = wx.CheckBox(self, -1, "Baseline")
+        self.peaklistColumnBase_check.SetValue(config.export['peaklistColumns'].count('base'))
+        
+        self.peaklistColumnInt_check = wx.CheckBox(self, -1, "Intensity")
+        self.peaklistColumnInt_check.SetValue(config.export['peaklistColumns'].count('int'))
+        
+        self.peaklistColumnRel_check = wx.CheckBox(self, -1, "Relative intensity")
+        self.peaklistColumnRel_check.SetValue(config.export['peaklistColumns'].count('rel'))
+        
+        self.peaklistColumnSn_check = wx.CheckBox(self, -1, "s/n")
+        self.peaklistColumnSn_check.SetValue(config.export['peaklistColumns'].count('sn'))
+        
+        self.peaklistColumnZ_check = wx.CheckBox(self, -1, "Charge")
+        self.peaklistColumnZ_check.SetValue(config.export['peaklistColumns'].count('z'))
+        
+        self.peaklistColumnMass_check = wx.CheckBox(self, -1, "Mass")
+        self.peaklistColumnMass_check.SetValue(config.export['peaklistColumns'].count('mass'))
+        
+        self.peaklistColumnFwhm_check = wx.CheckBox(self, -1, "FWHM")
+        self.peaklistColumnFwhm_check.SetValue(config.export['peaklistColumns'].count('fwhm'))
+        
+        self.peaklistColumnResol_check = wx.CheckBox(self, -1, "Resolution")
+        self.peaklistColumnResol_check.SetValue(config.export['peaklistColumns'].count('resol'))
+        
+        cancel_butt = wx.Button(self, wx.ID_CANCEL, "Cancel")
+        copy_butt = wx.Button(self, wx.ID_OK, "Copy")
+        copy_butt.SetDefault()
         
         # pack elements
-        itemSizer = wx.BoxSizer(wx.VERTICAL)
-        itemSizer.Add(self.colMZ_check, 0, wx.ALIGN_LEFT, 0)
-        itemSizer.Add(self.colIntensity_check, 0, wx.ALIGN_LEFT|wx.TOP|wx.BOTTOM, 5)
-        itemSizer.Add(self.colRelIntensity_check, 0, wx.ALIGN_LEFT|wx.BOTTOM, 5)
-        itemSizer.Add(self.colCharge_check, 0, wx.ALIGN_LEFT|wx.BOTTOM, 5)
-        itemSizer.Add(self.colSN_check, 0, wx.ALIGN_LEFT|wx.BOTTOM, 5)
-        itemSizer.Add(self.colBaseline_check, 0, wx.ALIGN_LEFT|wx.BOTTOM, 5)
-        itemSizer.Add(self.colFWHM_check, 0, wx.ALIGN_LEFT|wx.BOTTOM, 5)
-        itemSizer.Add(self.colResolution_check, 0, wx.ALIGN_LEFT, 0)
+        grid = wx.GridBagSizer(mwx.GRIDBAG_VSPACE, mwx.GRIDBAG_HSPACE)
+        grid.Add(self.peaklistColumnMz_check, (0,0))
+        grid.Add(self.peaklistColumnAi_check, (1,0))
+        grid.Add(self.peaklistColumnBase_check, (2,0))
+        grid.Add(self.peaklistColumnInt_check, (3,0))
+        grid.Add(self.peaklistColumnRel_check, (4,0))
+        grid.Add(self.peaklistColumnSn_check, (0,2))
+        grid.Add(self.peaklistColumnZ_check, (1,2))
+        grid.Add(self.peaklistColumnMass_check, (2,2))
+        grid.Add(self.peaklistColumnFwhm_check, (3,2))
+        grid.Add(self.peaklistColumnResol_check, (4,2))
         
-        staticSizer.Add(itemSizer, 0, wx.ALL, 5)
+        staticSizer.Add(grid, 0, wx.ALL, 5)
+        
+        buttSizer = wx.BoxSizer(wx.HORIZONTAL)
+        buttSizer.Add(cancel_butt, 0, wx.RIGHT, 15)
+        buttSizer.Add(copy_butt, 0)
         
         mainSizer = wx.BoxSizer(wx.VERTICAL)
         mainSizer.Add(staticSizer, 0, wx.CENTER|wx.ALL, mwx.PANEL_SPACE_MAIN)
-        mainSizer.Add(self.copy_butt, 0, wx.CENTER|wx.LEFT|wx.RIGHT|wx.BOTTOM, mwx.PANEL_SPACE_MAIN)
+        mainSizer.Add(buttSizer, 0, wx.CENTER|wx.LEFT|wx.RIGHT|wx.BOTTOM, mwx.PANEL_SPACE_MAIN)
         
         return mainSizer
     # ----
     
     
     def getData(self):
-        """Delete peaks below threshold"""
+        """Set selected columns to export presets."""
         
         # get data
-        columns = []
-        if self.colMZ_check.IsChecked():
-            columns.append(0)
-        if self.colIntensity_check.IsChecked():
-            columns.append(1)
-        if self.colRelIntensity_check.IsChecked():
-            columns.append(2)
-        if self.colCharge_check.IsChecked():
-            columns.append(3)
-        if self.colSN_check.IsChecked():
-            columns.append(4)
-        if self.colBaseline_check.IsChecked():
-            columns.append(5)
-        if self.colFWHM_check.IsChecked():
-            columns.append(6)
-        if self.colResolution_check.IsChecked():
-            columns.append(7)
-        
-        return columns
+        config.export['peaklistColumns'] = []
+        if self.peaklistColumnMz_check.IsChecked():
+            config.export['peaklistColumns'].append('mz')
+        if self.peaklistColumnAi_check.IsChecked():
+            config.export['peaklistColumns'].append('ai')
+        if self.peaklistColumnBase_check.IsChecked():
+            config.export['peaklistColumns'].append('base')
+        if self.peaklistColumnInt_check.IsChecked():
+            config.export['peaklistColumns'].append('int')
+        if self.peaklistColumnRel_check.IsChecked():
+            config.export['peaklistColumns'].append('rel')
+        if self.peaklistColumnZ_check.IsChecked():
+            config.export['peaklistColumns'].append('z')
+        if self.peaklistColumnMass_check.IsChecked():
+            config.export['peaklistColumns'].append('mass')
+        if self.peaklistColumnSn_check.IsChecked():
+            config.export['peaklistColumns'].append('sn')
+        if self.peaklistColumnFwhm_check.IsChecked():
+            config.export['peaklistColumns'].append('fwhm')
+        if self.peaklistColumnResol_check.IsChecked():
+            config.export['peaklistColumns'].append('resol')
     # ----
     
 
