@@ -23,9 +23,9 @@ import os.path
 
 # load modules
 from ids import *
-import config
 import mwx
 import images
+import config
 import mspy
 import doc
 
@@ -73,9 +73,11 @@ class panelProfound(wx.MiniFrame):
         self.mainSizer.Add(pmf, 1, wx.EXPAND, 0)
         self.mainSizer.Add(query, 1, wx.EXPAND, 0)
         
+        # hide panels
         self.mainSizer.Hide(1)
         self.mainSizer.Hide(2)
         
+        # fit layout
         self.mainSizer.Fit(self)
         self.SetSizer(self.mainSizer)
     # ----
@@ -375,13 +377,13 @@ class panelProfound(wx.MiniFrame):
             self.query_butt.SetBitmapLabel(images.lib['profoundQueryOn'])
         
         # fit layout
-        self.SetMinSize((-1,-1))
-        self.mainSizer.Fit(self)
         self.Layout()
+        self.mainSizer.Fit(self)
+        
+        # force resize
         size = self.GetSize()
-        self.SetSize((size[0]+1,size[1]))
+        self.SetSize((size[0], size[1]+1))
         self.SetSize(size)
-        self.SetMinSize(size)
     # ----
     
     
@@ -425,7 +427,7 @@ class panelProfound(wx.MiniFrame):
         # make new query
         query = ''
         for peak in peaklist:
-            query += '%f\t%f\n' % (peak.mz, (peak.intensity-peak.baseline))
+            query += '%f\t%f\n' % (peak.mz, peak.intensity)
         
         # set data
         self.paramQuery_value.SetValue(query)
