@@ -23,9 +23,9 @@ import os.path
 
 # load modules
 from ids import *
-import config
 import mwx
 import images
+import config
 import mspy
 import doc
 
@@ -34,7 +34,7 @@ import doc
 # -------------------------------------------
 
 class panelProspector(wx.MiniFrame):
-    """MS-Fit search tool."""
+    """ProteinProspector search tools."""
     
     def __init__(self, parent, tool=config.prospector['common']['searchType']):
         wx.MiniFrame.__init__(self, parent, -1, 'Protein Prospector', size=(300, -1), style=wx.DEFAULT_FRAME_STYLE & ~ (wx.RESIZE_BORDER | wx.RESIZE_BOX | wx.MAXIMIZE_BOX))
@@ -75,10 +75,12 @@ class panelProspector(wx.MiniFrame):
         self.mainSizer.Add(msTag, 1, wx.EXPAND, 0)
         self.mainSizer.Add(query, 1, wx.EXPAND, 0)
         
+        # hide panels
         self.mainSizer.Hide(1)
         self.mainSizer.Hide(2)
         self.mainSizer.Hide(3)
         
+        # fit layout
         self.mainSizer.Fit(self)
         self.SetSizer(self.mainSizer)
     # ----
@@ -544,13 +546,13 @@ class panelProspector(wx.MiniFrame):
             self.search_butt.Enable(False)
         
         # fit layout
-        self.SetMinSize((-1,-1))
-        self.mainSizer.Fit(self)
         self.Layout()
+        self.mainSizer.Fit(self)
+        
+        # force resize
         size = self.GetSize()
-        self.SetSize((size[0]+1,size[1]))
+        self.SetSize((size[0], size[1]+1))
         self.SetSize(size)
-        self.SetMinSize(size)
     # ----
     
     
@@ -600,7 +602,7 @@ class panelProspector(wx.MiniFrame):
         # make new query
         query = ''
         for peak in peaklist:
-            query += '%f\t%f\n' % (peak.mz, (peak.intensity-peak.baseline))
+            query += '%f\t%f\n' % (peak.mz, peak.intensity)
         
         # set data
         self.paramQuery_value.SetValue(query)
