@@ -20,10 +20,11 @@ import xml.sax
 import base64
 import struct
 import os.path
+import numpy
 from copy import deepcopy
 
-# load configuration
-import config
+# load stopper
+from stopper import CHECK_FORCE_QUIT
 
 # register essential objects
 import objects
@@ -39,15 +40,7 @@ class parseMZDATA():
         self.path = path
         self._scans = None
         self._scanlist = None
-        self._info = {
-            'title': '',
-            'operator': '',
-            'contact': '',
-            'institution': '',
-            'date': '',
-            'instrument': '',
-            'notes': '',
-        }
+        self._info = None
         
         # check path
         if not os.path.exists(path):
@@ -105,7 +98,7 @@ class parseMZDATA():
         except stopParsing:
             self._info = handler.data
         except xml.sax.SAXException:
-            pass
+            self._info = False
         
         return self._info
     # ----
@@ -136,7 +129,7 @@ class parseMZDATA():
     # ----
     
     
-    def getScan(self, scanID=None, raw=False):
+    def getScan(self, scanID=None):
         """Get spectrum from document."""
         
         # use preloaded data if available
@@ -162,11 +155,8 @@ class parseMZDATA():
         if not data:
             return False
         
-        # return data
-        if raw:
-            return data
-        else:
-            return self._makeScan(data)
+        # return scan
+        return self._makeScan(data)
     # ----
     
     
@@ -183,6 +173,7 @@ class parseMZDATA():
             scan = objects.scan(points=points)
         
         # set metadata
+        scan.title = scanData['title']
         scan.scanNumber = scanData['scanNumber']
         scan.parentScanNumber = scanData['parentScanNumber']
         scan.msLevel = scanData['msLevel']
@@ -199,35 +190,35 @@ class parseMZDATA():
     # ----
     
     
-    def _parsePoints(self, scan):
+    def _parsePoints(self, scanData):
         """Parse spectrum data."""
         
         # check data
-        if not scan['mzData'] or not scan['intData']:
+        if not scanData['mzData'] or not scanData['intData']:
             return []
         
         # decode data
-        mzData = base64.b64decode(scan['mzData'])
-        intData = base64.b64decode(scan['intData'])
+        mzData = base64.b64decode(scanData['mzData'])
+        intData = base64.b64decode(scanData['intData'])
         
         # get endian
         mzEndian = '!'
         intEndian = '!'
-        if scan['mzEndian'] == 'little':
+        if scanData['mzEndian'] == 'little':
             mzEndian = '<'
-        elif scan['mzEndian'] == 'big':
+        elif scanData['mzEndian'] == 'big':
             mzEndian = '>'
-        if scan['intEndian'] == 'little':
+        if scanData['intEndian'] == 'little':
             intEndian = '<'
-        elif scan['intEndian'] == 'big':
+        elif scanData['intEndian'] == 'big':
             intEndian = '>'
         
         # get precision
         mzPrecision = 'f'
         intPrecision = 'f'
-        if scan['mzPrecision'] == 64:
+        if scanData['mzPrecision'] == 64:
             mzPrecision = 'd'
-        if scan['intPrecision'] == 64:
+        if scanData['intPrecision'] == 64:
             intPrecision = 'd'
         
         # convert from binary
@@ -237,7 +228,15 @@ class parseMZDATA():
         intData = struct.unpack(intEndian + intPrecision * count, intData[0:len(intData)])
         
         # format
-        data = map(list, zip(mzData, intData))
+        if scanData['spectrumType'] == 'discrete':
+            data = map(list, zip(mzData, intData))
+        else:
+            mzData = numpy.array(mzData)
+            mzData.shape = (-1,1)
+            intData = numpy.array(intData)
+            intData.shape = (-1,1)
+            data = numpy.concatenate((mzData,intData), axis=1)
+            data = data.copy()
         
         return data
     # ----
@@ -350,6 +349,7 @@ class scanlistHandler(xml.sax.handler.ContentHandler):
                 self.currentID = int(self.currentID)
             
             scan = {
+                'title': '',
                 'scanNumber': self.currentID,
                 'parentScanNumber': None,
                 'msLevel': None,
@@ -479,6 +479,7 @@ class scanHandler(xml.sax.handler.ContentHandler):
                 self._isMatch = True
                 
                 self.data = {
+                    'title': '',
                     'scanNumber': scanID,
                     'parentScanNumber': None,
                     'msLevel': None,
@@ -654,6 +655,7 @@ class runHandler(xml.sax.handler.ContentHandler):
                 self.currentID = int(self.currentID)
             
             scan = {
+                'title': '',
                 'scanNumber': self.currentID,
                 'parentScanNumber': None,
                 'msLevel': None,
