@@ -21,15 +21,18 @@ import numpy
 import calculations
 
 
+# MAIN PLOT OBJECTS
+# -----------------
+
 class container:
     """Container to hold plot objects."""
     
-    def __init__(self, objects=[]):
+    def __init__(self, objects):
         self.objects = objects
     # ----
     
     
-    def getBoundingBox(self, minX=None, maxX=None):
+    def getBoundingBox(self, minX=None, maxX=None, absolute=False):
         """Get bounding box coverring all visible objects."""
         
         # init values if no data in objects
@@ -39,7 +42,7 @@ class container:
         have = False
         for obj in self.objects:
             if obj.properties['visible']:
-                oRect = obj.getBoundingBox(minX, maxX)
+                oRect = obj.getBoundingBox(minX, maxX, absolute)
                 if have and oRect:
                     rect[0] = numpy.minimum(rect[0], oRect[0])
                     rect[1] = numpy.maximum(rect[1], oRect[1])
@@ -66,18 +69,16 @@ class container:
         for obj in self.objects:
             if obj.properties['visible']:
                 legend = obj.getLegend()
-                if legend[0] != '':
+                if legend and legend[0] != '':
                     names.append(obj.getLegend())
             
         return names
     # ----
     
     
-    def getPoint(self, obj, xPos, screenCoord=False):
+    def getPoint(self, obj, xPos, coord='screen'):
         """Get interpolated Y position for given X."""
-        
-        point = self.objects[obj].getPoint(xPos, screenCoord)
-        return point
+        return self.objects[obj].getPoint(xPos, coord)
     # ----
     
     
@@ -145,18 +146,24 @@ class container:
         """Draw labels for all visible objects."""
         
         # get labels from objects
+        annots = []
         labels = []
         for obj in self.objects:
-            if obj.properties['visible']:
+            if obj.properties['visible'] and isinstance(obj, annotations):
+                annots += obj.makeLabels(dc, printerScale)
+            elif obj.properties['visible']:
                 labels += obj.makeLabels(dc, printerScale)
         
         # check labels
-        if not labels:
+        if not annots and not labels:
             return
         
         # sort labels
+        annots.sort()
+        annots.reverse()
         labels.sort()
         labels.reverse()
+        labels = annots + labels
         
         # preset font by first label
         font = labels[0][3]['labelFont']
@@ -183,7 +190,7 @@ class container:
                 continue
             
             # check free space and draw label
-            if overlapLabels or _checkFreeSpace(textCoords, occupied):
+            if overlapLabels or self._checkFreeSpace(textCoords, occupied):
                 
                 # check pen
                 if properties['labelFont'] != font:
@@ -234,6 +241,11 @@ class container:
     # ----
     
     
+    def empty(self):
+        del self.objects[:]
+    # ----
+    
+    
     def __additem__(self, obj):
         self.objects.append(obj)
     # ----
@@ -258,6 +270,354 @@ class container:
         return len(self.objects)
     # ----
     
+    
+    def _checkFreeSpace(self, coords, occupied):
+        """Check free space for label."""
+        
+        curX1, curY1, curX2, curY2 = coords
+        
+        # check occupied space
+        for occX1, occY1, occX2, occY2 in occupied:
+            if (curX1 < curX2) and ((occX1 <= curX1 <= occX2) or (occX1 <= curX2 <= occX2) or (curX1 <= occX1 and curX2 >= occX2)):
+                if (occY2 <= curY1 <= occY1) or (occY2 <= curY2 <= occY1) or (curY1 >= occY1 and curY2 <= occY2):
+                    return False
+            elif (curX1 > curX2) and ((occX2 <= curX1 <= occX1) or (occX2 <= curX2 <= occX1) or (curX1 <= occX2 and curX2 >= occX1)):
+                if (occY1 <= curY1 <= occY2) or (occY1 <= curY2 <= occY2) or (curY1 >= occY2 and curY2 <= occY1):
+                    return False
+        
+        return True
+    # ----
+    
+
+
+class annotations:
+    """Base class for annotations drawing."""
+    
+    def __init__(self, points, **attr):
+        
+        # set default params
+        self.properties = {
+                            'visible': True,
+                            'flipped': False,
+                            'xOffset': 0,
+                            'yOffset': 0,
+                            'normalized': False,
+                            'showInGel': False,
+                            'exactFit': False,
+                            'showPoints': True,
+                            'showLabels': True,
+                            'showXPos': True,
+                            'pointColour': (0, 0, 255),
+                            'pointSize': 3,
+                            'labelAngle': 90,
+                            'labelBgr': True,
+                            'labelColour': (0, 0, 0),
+                            'labelBgrColour': (255, 255, 255),
+                            'labelFont': wx.Font(10, wx.SWISS, wx.FONTSTYLE_NORMAL, wx.FONTWEIGHT_NORMAL, 0),
+                            'labelMaxLength': 20,
+                            'xPosDigits': 2,
+                            }
+        
+        self.currentScale = (1, 1)
+        self.currentShift = (0, 0)
+        self.normalization = (1, 0)
+        
+        # get new attributes
+        for name, value in attr.items():
+            self.properties[name] = value
+        
+        # convert points to array
+        self.points = numpy.array([[p[0], p[1]] for p in points])
+        self.pointsCropped = self.points
+        self.pointsScaled = self.pointsCropped
+        if len(self.points):
+            self.pointsBox = (numpy.minimum.reduce(self.points), numpy.maximum.reduce(self.points))
+        
+        # get labels
+        self.labels = ['']*len(points)
+        for x, point in enumerate(points):
+            if len(point) > 2:
+                self.labels[x] = point[2]
+        self.labelsCropped = self.labels
+        
+        # calculate normalization
+        self._normalization()
+    # ----
+    
+    
+    def setProperties(self, **attr):
+        """Set object properties."""
+        
+        for name, value in attr.items():
+            self.properties[name] = value
+    # ----
+    
+    
+    def setNormalization(self, scale, shift):
+        """Force specified normalization to be used insted of calculated one."""
+        self.normalization = (scale, shift)
+    # ----
+    
+    
+    def getBoundingBox(self, minX=None, maxX=None, absolute=False):
+        """Get bounding box for whole data or X selection"""
+        
+        # use relevant data
+        if minX != None and maxX != None:
+            self.cropPoints(minX, maxX)
+            data = self.pointsCropped
+        else:
+            data = self.points
+        
+        # check data
+        if not len(data):
+            return False
+        
+        # get range
+        if minX != None and maxX != None:
+            minXY = numpy.minimum.reduce(data)
+            maxXY = numpy.maximum.reduce(data)
+        else:
+            minXY = [self.pointsBox[0][0], self.pointsBox[0][1]]
+            maxXY = [self.pointsBox[1][0], self.pointsBox[1][1]]
+        
+        # extend values slightly to fit data
+        if not absolute and not self.properties['exactFit']:
+            xExtend = (maxXY[0] - minXY[0]) * 0.05
+            yExtend = (maxXY[1] - minXY[1]) * 0.05
+            minXY[0] -= xExtend
+            maxXY[0] += xExtend
+            minXY[1] -= yExtend
+            maxXY[1] += yExtend
+            
+        # extend values to fit labels
+        elif not absolute:
+            if self.properties['showLabels'] and self.properties['labelAngle']==0:
+                maxXY[1] += (maxXY[1] - minXY[1]) * 0.2
+            elif self.properties['showLabels'] and self.properties['labelAngle']==90:
+                maxXY[1] += (maxXY[1] - minXY[1]) * 0.4
+            else:
+                maxXY[1] += (maxXY[1] - minXY[1]) * 0.05
+        
+        # apply normalization
+        if self.properties['normalized']:
+            minXY[1] = (minXY[1] - self.normalization[1]) / self.normalization[0]
+            maxXY[1] = (maxXY[1] - self.normalization[1]) / self.normalization[0]
+        
+        # apply offset
+        minXY[0] += self.properties['xOffset']
+        minXY[1] += self.properties['yOffset']
+        maxXY[0] += self.properties['xOffset']
+        maxXY[1] += self.properties['yOffset']
+        
+        # apply flipping
+        if self.properties['flipped']:
+            minY = -1 * maxXY[1]
+            maxY = -1 * minXY[1]
+            minXY[1] = minY
+            maxXY[1] = maxY
+        
+        return [minXY, maxXY]
+    # ----
+    
+    
+    def getLegend(self):
+        """Get legend."""
+        return None
+    # ----
+    
+    
+    def cropPoints(self, minX, maxX):
+        """Crop points to selected X range."""
+        
+        # check data
+        if not len(self.points):
+            self.pointsCropped = self.points[:]
+            self.labelsCropped = self.labels[:]
+            return
+        
+        # apply offset
+        minX -= self.properties['xOffset']
+        maxX -= self.properties['xOffset']
+        
+        # out of range
+        if self.points[-1][0] < minX or self.points[0][0] > maxX:
+            self.pointsCropped = []
+            self.labelsCropped = []
+            return
+        
+        # get index of points in selection
+        length = len(self.points)
+        
+        int1 = [0,length]
+        while int1[1] - int1[0] > 1:
+            i1 = int( sum( int1) / 2)
+            if self.points[i1][0] < minX:
+                int1[0] = i1
+            else:
+                int1[1] = i1
+        
+        int2 = [0,length]
+        while int2[1] - int2[0] > 1:
+            i2 = int( sum( int2) / 2)
+            if self.points[i2][0] > maxX:
+                int2[1] = i2
+            else:
+                int2[0] = i2
+        
+        i1 = int1[0]
+        i2 = int2[1]
+        if self.points[i1][0] < minX:
+            i1 = int1[1]
+        if self.points[i2-1][0] > maxX:
+            i2 = int2[0]
+        
+        # crop data
+        self.pointsCropped = self.points[i1:i2]
+        self.labelsCropped = self.labels[i1:i2]
+    # ----
+    
+    
+    def scaleAndShift(self, scale, shift):
+        """Scale and shift points to screen coordinations."""
+        
+        self.pointsScaled = self.pointsCropped
+        
+        xScale = scale[0]
+        yScale = scale[1]
+        xShift = shift[0]
+        yShift = shift[1]
+        
+        # apply flipping
+        if self.properties['flipped']:
+            yScale *= -1
+        
+        # apply normalization
+        if self.properties['normalized']:
+            yScale /= self.normalization[0]
+            yShift -= self.normalization[1] * yScale
+        
+        # apply offset
+        xShift += self.properties['xOffset'] * xScale
+        yShift += self.properties['yOffset'] * yScale
+        
+        # recalculate data
+        if len(self.pointsCropped):
+            self.pointsScaled = calculations.scaleAndShift(self.pointsCropped, xScale, yScale, xShift, yShift)
+        
+        self.currentScale = scale
+        self.currentShift = shift
+    # ----
+    
+    
+    def filterPoints(self, filterSize):
+        """Filter points for printing and exporting"""
+        pass
+    # ----
+    
+    
+    def draw(self, dc, printerScale):
+        """Draw object."""
+        
+        # check data
+        if not len(self.pointsScaled):
+            return
+        
+        # draw points
+        if self.properties['showPoints']:
+            pencolour = [max(x-70,0) for x in self.properties['pointColour']]
+            pen = wx.Pen(pencolour, 1*printerScale['drawings'], wx.SOLID)
+            brush = wx.Brush(self.properties['pointColour'], wx.SOLID)
+            dc.SetPen(pen)
+            dc.SetBrush(brush)
+            for point in self.pointsScaled:
+                dc.DrawCircle(point[0], point[1], self.properties['pointSize']*printerScale['drawings'])
+    # ----
+    
+    
+    def drawGel(self, dc, gelCoords, gelHeight, printerScale):
+        """Draw gel."""
+        pass
+    # ----
+    
+    
+    def makeLabels(self, dc, printerScale):
+        """Get object labels."""
+        
+        # check labels
+        if not self.properties['showLabels'] or not self.labelsCropped:
+            return []
+        
+        # set font
+        dc.SetFont(_scaleFont(self.properties['labelFont'], printerScale['fonts']))
+        
+        # prepare labels
+        labels = []
+        format = '%0.'+`self.properties['xPosDigits']`+'f - '
+        for x, label in enumerate(self.labelsCropped):
+            
+            # check max length
+            if len(label) > self.properties['labelMaxLength']:
+                label = label[:self.properties['labelMaxLength']] + '...'
+            
+            # add X position
+            if self.properties['showXPos']:
+                label = (format % self.pointsCropped[x][0]) + label
+            
+            # get position
+            xPos = self.pointsScaled[x][0]
+            yPos = self.pointsScaled[x][1]
+            
+            # get text position
+            textSize = dc.GetTextExtent(label)
+            if self.properties['labelAngle'] == 90:
+                if self.properties['flipped']:
+                    textXPos = xPos + textSize[1]*0.5
+                    textYPos = yPos + 5*printerScale['drawings']
+                    textCoords = (textXPos, textYPos, textXPos-textSize[1], textYPos+textSize[0])
+                else:
+                    textXPos = xPos - textSize[1]*0.5
+                    textYPos = yPos - 5*printerScale['drawings']
+                    textCoords = (textXPos, textYPos, textXPos+textSize[1], textYPos-textSize[0])
+            
+            elif self.properties['labelAngle'] == 0:
+                if self.properties['flipped']:
+                    textXPos = xPos - textSize[0]*0.5
+                    textYPos = yPos + 4*printerScale['drawings']
+                    textCoords = (textXPos, textYPos, textXPos+textSize[0], textYPos-textSize[1])
+                else:
+                    textXPos = xPos - textSize[0]*0.5
+                    textYPos = yPos - textSize[1] - 4*printerScale['drawings']
+                    textCoords = (textXPos, textYPos, textXPos+textSize[0], textYPos-textSize[1])
+            
+            # add label and sort by intensity
+            labels.append((self.pointsCropped[x][1], label, textCoords, self.properties))
+            
+        return labels
+    # ----
+    
+    
+    def _normalization(self):
+        """Calculate normalization constants."""
+        
+        self.normalization = (1,0)
+        
+        # check data
+        if not len(self.points):
+            return
+        
+        # get range
+        minXY, maxXY = self.pointsBox
+        maxXY = numpy.maximum.reduce(self.points)
+        
+        # calculate scale and shift
+        shift = minXY[1]
+        rng = maxXY[1]
+        scale = rng / (100 * (1 + shift / rng))
+        
+        self.normalization = (scale, shift)
+    # ----
+    
 
 
 class points:
@@ -299,9 +659,11 @@ class points:
         self.points = numpy.array(points)
         self.cropped = self.points
         self.scaled = self.cropped
+        if len(self.points):
+            self.pointsBox = (numpy.minimum.reduce(self.points), numpy.maximum.reduce(self.points))
         
         # calculate normalization
-        self._mormalization()
+        self._normalization()
     # ----
     
     
@@ -313,15 +675,62 @@ class points:
     # ----
     
     
-    def getBoundingBox(self, minX=None, maxX=None):
+    def setNormalization(self, scale, shift):
+        """Force specified normalization to be used insted of calculated one."""
+        self.normalization = (scale, shift)
+    # ----
+    
+    
+    def getBoundingBox(self, minX=None, maxX=None, absolute=False):
         """Get bounding box for whole data or X selection"""
         
         # use relevant data
         if minX != None and maxX != None:
             self.cropPoints(minX, maxX)
-            return self._calcBoundingBox(self.cropped)
+            data = self.cropped
         else:
-            return self._calcBoundingBox(self.points)
+            data = self.points
+        
+        # check data
+        if not len(data):
+            return False
+        
+        # get range
+        if minX != None and maxX != None:
+            minXY = numpy.minimum.reduce(data)
+            maxXY = numpy.maximum.reduce(data)
+        else:
+            minXY = [self.pointsBox[0][0], self.pointsBox[0][1]]
+            maxXY = [self.pointsBox[1][0], self.pointsBox[1][1]]
+        
+        # extend values slightly to fit data
+        if not absolute and not self.properties['exactFit']:
+            xExtend = (maxXY[0] - minXY[0]) * 0.05
+            yExtend = (maxXY[1] - minXY[1]) * 0.05
+            minXY[0] -= xExtend
+            maxXY[0] += xExtend
+            minXY[1] -= yExtend
+            maxXY[1] += yExtend
+        
+        # apply normalization
+        if self.properties['normalized']:
+            minXY[1] = (minXY[1] - self.normalization[1]) / self.normalization[0]
+            maxXY[1] = (maxXY[1] - self.normalization[1]) / self.normalization[0]
+        
+        # apply offset
+        minXY[0] += self.properties['xOffset']
+        minXY[1] += self.properties['yOffset']
+        maxXY[0] += self.properties['xOffset']
+        maxXY[1] += self.properties['yOffset']
+        
+        # apply flipping
+        if self.properties['flipped']:
+            minY = -1 * maxXY[1]
+            maxY = -1 * minXY[1]
+            minXY[1] = minY
+            maxXY[1] = maxY
+        
+        return [minXY, maxXY]
     # ----
     
     
@@ -483,59 +892,17 @@ class points:
     # ----
     
     
-    def _calcBoundingBox(self, data):
-        """Calculate bounding box for given data."""
-        
-        # check data
-        if not len(data):
-            return False
-        
-        # get range
-        minXY = numpy.minimum.reduce(data)
-        maxXY = numpy.maximum.reduce(data)
-        
-        # extend values slightly to fit data
-        if not self.properties['exactFit']:
-            xExtend = (maxXY[0] - minXY[0]) * 0.05
-            yExtend = (maxXY[1] - minXY[1]) * 0.05
-            minXY[0] -= xExtend
-            maxXY[0] += xExtend
-            minXY[1] -= yExtend
-            maxXY[1] += yExtend
-        
-        # apply normalization
-        if self.properties['normalized']:
-            minXY[1] = (minXY[1] - self.normalization[1]) / self.normalization[0]
-            maxXY[1] = (maxXY[1] - self.normalization[1]) / self.normalization[0]
-        
-        # apply offset
-        minXY[0] += self.properties['xOffset']
-        minXY[1] += self.properties['yOffset']
-        maxXY[0] += self.properties['xOffset']
-        maxXY[1] += self.properties['yOffset']
-        
-        # apply flipping
-        if self.properties['flipped']:
-            minY = -1 * maxXY[1]
-            maxY = -1 * minXY[1]
-            minXY[1] = minY
-            maxXY[1] = maxY
-        
-        return [minXY, maxXY]
-    # ----
-    
-    
-    def _mormalization(self):
+    def _normalization(self):
         """Calculate normalization constants."""
         
         self.normalization = (1,0)
         
         # check data
-        if not len(self.scaled):
+        if not len(self.points):
             return
         
         # get range
-        minXY = numpy.minimum.reduce(self.points)
+        minXY, maxXY = self.pointsBox
         maxXY = numpy.maximum.reduce(self.points)
         
         # calculate scale and shift
@@ -600,16 +967,20 @@ class spectrum:
         self.spectrumPoints = numpy.array(scan.points)
         self.spectrumCropped = self.spectrumPoints
         self.spectrumScaled = self.spectrumCropped
+        if len(self.spectrumPoints):
+            self.spectrumBox = (numpy.minimum.reduce(self.spectrumPoints), numpy.maximum.reduce(self.spectrumPoints))
         
         # convert peaklist points to array
         self.peaklist = scan.peaklist
-        self.peaklistPoints = numpy.array([[peak.mz, peak.intensity, peak.baseline] for peak in scan.peaklist])
+        self.peaklistPoints = numpy.array([[peak.mz, peak.ai, peak.base] for peak in scan.peaklist])
         self.peaklistCropped = self.peaklistPoints
         self.peaklistScaled = self.peaklistCropped
         self.peaklistCroppedPeaks = self.peaklist
+        if len(self.peaklistPoints):
+            self.peaklistBox = (numpy.minimum.reduce(self.peaklistPoints), numpy.maximum.reduce(self.peaklistPoints))
         
         # calculate normalization
-        self._mormalization()
+        self._normalization()
     # ----
     
     
@@ -621,15 +992,98 @@ class spectrum:
     # ----
     
     
-    def getBoundingBox(self, minX=None, maxX=None):
+    def setNormalization(self, scale, shift):
+        """Force specified normalization to be used insted of calculated one."""
+        self.normalization = (scale, shift)
+    # ----
+    
+    
+    def getBoundingBox(self, minX=None, maxX=None, absolute=False):
         """Get bounding box for whole data or X selection."""
         
+        spectrumBox = None
+        peaklistBox = None
+        
         # use relevant data
         if minX != None and maxX != None:
             self.cropPoints(minX, maxX)
-            return self._calcBoundingBox(self.spectrumCropped, self.peaklistCropped)
+            spectrumData = self.spectrumCropped
+            peaklistData = self.peaklistCropped
         else:
-            return self._calcBoundingBox(self.spectrumPoints, self.peaklistPoints)
+            spectrumData = self.spectrumPoints
+            peaklistData = self.peaklistPoints
+        
+        # calculate bounding box for spectrum
+        if len(spectrumData) and self.properties['showSpectrum']:
+            if minX != None and maxX != None:
+                minXY = numpy.minimum.reduce(spectrumData)
+                maxXY = numpy.maximum.reduce(spectrumData)
+            else:
+                minXY = [self.spectrumBox[0][0], self.spectrumBox[0][1]]
+                maxXY = [self.spectrumBox[1][0], self.spectrumBox[1][1]]
+            
+            if not absolute:
+                maxXY[1] += (maxXY[1] - minXY[1]) * 0.05
+            
+            spectrumBox = [minXY, maxXY]
+        
+        # calculate bounding box for peaklist
+        if len(peaklistData) and (self.properties['showSpectrum'] or self.properties['showLabels'] or self.properties['showTicks']):
+            if minX != None and maxX != None:
+                minXY = numpy.minimum.reduce(peaklistData)
+                maxXY = numpy.maximum.reduce(peaklistData)
+            else:
+                minXY = [self.peaklistBox[0][0], self.peaklistBox[0][1], self.peaklistBox[0][2]]
+                maxXY = [self.peaklistBox[1][0], self.peaklistBox[1][1], self.peaklistBox[1][2]]
+            
+            minXY = [minXY[0], min(minXY[1:])]
+            maxXY = [maxXY[0], max(maxXY[1:])]
+            
+            # extend values to fit labels
+            if not absolute:
+                xExtend = (maxXY[0] - minXY[0]) * 0.02
+                minXY[0] -= xExtend
+                maxXY[0] += xExtend
+                
+                if self.properties['showLabels'] and self.properties['labelAngle']==0:
+                    maxXY[1] += (maxXY[1] - minXY[1]) * 0.2
+                elif self.properties['showLabels'] and self.properties['labelAngle']==90:
+                    maxXY[1] += (maxXY[1] - minXY[1]) * 0.4
+                else:
+                    maxXY[1] += (maxXY[1] - minXY[1]) * 0.05
+            
+            peaklistBox = [minXY, maxXY]
+        
+        # use both
+        if spectrumBox and peaklistBox:
+            minXY, maxXY = [numpy.minimum(spectrumBox[0], peaklistBox[0]), numpy.maximum(spectrumBox[1], peaklistBox[1])]
+        elif spectrumBox:
+            minXY, maxXY = spectrumBox
+        elif peaklistBox:
+            minXY, maxXY = peaklistBox
+        else:
+            return False
+        
+        # apply normalization
+        if self.properties['normalized']:
+            minXY[1] = (minXY[1] - self.normalization[1]) / self.normalization[0]
+            maxXY[1] = (maxXY[1] - self.normalization[1]) / self.normalization[0]
+        
+        # apply offset
+        if not self.properties['normalized']:
+            minXY[0] += self.properties['xOffset']
+            minXY[1] += self.properties['yOffset']
+            maxXY[0] += self.properties['xOffset']
+            maxXY[1] += self.properties['yOffset']
+        
+        # apply flipping
+        if self.properties['flipped']:
+            minY = -1 * maxXY[1]
+            maxY = -1 * minXY[1]
+            minXY[1] = minY
+            maxXY[1] = maxY
+        
+        return [minXY, maxXY]
     # ----
     
     
@@ -659,64 +1113,26 @@ class spectrum:
     # ----
     
     
-    def getPoint(self, xPos, screenCoord=False):
+    def getPoint(self, xPos, coord='screen'):
         """Get interpolated Y position for given X."""
         
-        pointsLen = len(self.spectrumPoints)
+        # get relevant data
+        if coord == 'user':
+            points = self.spectrumCropped
+        else:
+            points = self.spectrumScaled
         
-        # check points
-        if not pointsLen and screenCoord:
-            return [xPos, 0, 0, 0]
-        elif not pointsLen:
-            return [xPos, 0]
+        # check data
+        if not len(points):
+            return None
         
-        # get relevant part to speed-up
-        startIndex = 0
-        for i in range(0, pointsLen, 500):
-            if self.spectrumPoints[i][0] > xPos:
-                break
-            else:
-                startIndex = i
+        # get xPos index
+        index = _getIndex(points, xPos)
+        if not index or index == len(points):
+            return None
         
-        # get index of nearest higher point
-        index = 0
-        for i in range(startIndex, pointsLen):
-            if self.spectrumPoints[i][0] > xPos:
-                index = i
-                break
-        
-        # interpolate between two points
-        x1 = self.spectrumPoints[index-1][0]
-        y1 = self.spectrumPoints[index-1][1]
-        x2 = self.spectrumPoints[index][0]
-        y2 = self.spectrumPoints[index][1]
-        yPos = y1 + ((xPos - x1) * (y2 - y1)/(x2 - x1))
-        
-        # recalculate screen coordinations
-        if screenCoord:
-            
-            xScale = self.currentScale[0]
-            yScale = self.currentScale[1]
-            xShift = self.currentShift[0]
-            yShift = self.currentShift[1]
-            
-            # apply flipping
-            if self.properties['flipped']:
-                yScale *= -1
-            
-            # apply normalization
-            if self.properties['normalized']:
-                yScale /= self.normalization[0]
-                yShift -= self.normalization[1] * yScale
-            
-            # apply offset
-            if not self.properties['normalized']:
-                xShift += self.properties['xOffset'] * xScale
-                yShift += self.properties['yOffset'] * yScale
-            
-            # recalculate point
-            xPos = xPos * xScale + xShift
-            yPos = yPos * yScale + yShift
+        # get yPos
+        yPos = _interpolateLine(points[index-1], points[index], x=xPos)
         
         return [xPos, yPos]
     # ----
@@ -767,7 +1183,7 @@ class spectrum:
         # scale and shift spectrum data
         if len(self.spectrumCropped):
             self.spectrumScaled = calculations.scaleAndShift(self.spectrumCropped, xScale, yScale, xShift, yShift)
-                
+        
         # scale and shift peaklist data
         if len(self.peaklistCropped):
             self.peaklistScaled = numpy.array((xScale, yScale, yScale)) * self.peaklistCropped + numpy.array((xShift, yShift, yShift))
@@ -809,6 +1225,9 @@ class spectrum:
         # draw peaklist gel
         elif len(self.peaklistScaled) and (self.properties['showSpectrum'] or self.properties['showLabels'] or self.properties['showTicks']):
             self._drawPeaklistGel(dc, gelCoords, gelHeight, printerScale)
+        
+        # draw gel legend
+        self._drawGelLegend(dc, gelCoords, gelHeight, printerScale)
     # ----
     
     
@@ -865,80 +1284,12 @@ class spectrum:
                     textCoords = (textXPos, textYPos, textXPos+textSize[0], textYPos-textSize[1])
             
             # add label and sort by intensity
-            labels.append((self.peaklistCroppedPeaks[x].intensity, label, textCoords, self.properties))
+            labels.append((self.peaklistCroppedPeaks[x].ai, label, textCoords, self.properties))
             
         return labels
     # ----
     
     
-    def _calcBoundingBox(self, spectrumData, peaklistData):
-        """Calculate bounding box for given data."""
-        
-        spectrumBox = None
-        peaklistBox = None
-        
-        # calculate bounding box for spectrum
-        if len(spectrumData) and self.properties['showSpectrum']:
-            minXY = numpy.minimum.reduce(spectrumData)
-            maxXY = numpy.maximum.reduce(spectrumData)
-            maxXY[1] += (maxXY[1] - minXY[1]) * 0.05
-            spectrumBox = [minXY, maxXY]
-        
-        # calculate bounding box for peaklist
-        if len(peaklistData) and (self.properties['showSpectrum'] or self.properties['showLabels'] or self.properties['showTicks']):
-            minXY = numpy.minimum.reduce(peaklistData)
-            minXY = [minXY[0], min(minXY[1:])]
-            maxXY = numpy.maximum.reduce(peaklistData)
-            maxXY = [maxXY[0], max(maxXY[1:])]
-            
-            # extend X values to fit labels
-            xExtend = (maxXY[0] - minXY[0]) * 0.02
-            minXY[0] -= xExtend
-            maxXY[0] += xExtend
-            
-            # extend Y values to fit labels
-            if self.properties['showLabels'] and self.properties['labelAngle']==0:
-                maxXY[1] += (maxXY[1] - minXY[1]) * 0.2
-            elif self.properties['showLabels'] and self.properties['labelAngle']==90:
-                maxXY[1] += (maxXY[1] - minXY[1]) * 0.4
-            else:
-                maxXY[1] += (maxXY[1] - minXY[1]) * 0.05
-            
-            peaklistBox = [minXY, maxXY]
-        
-        # use both
-        if spectrumBox and peaklistBox:
-            minXY, maxXY = [numpy.minimum(spectrumBox[0], peaklistBox[0]), numpy.maximum(spectrumBox[1], peaklistBox[1])]
-        elif spectrumBox:
-            minXY, maxXY = spectrumBox
-        elif peaklistBox:
-            minXY, maxXY = peaklistBox
-        else:
-            return False
-        
-        # apply normalization
-        if self.properties['normalized']:
-            minXY[1] = (minXY[1] - self.normalization[1]) / self.normalization[0]
-            maxXY[1] = (maxXY[1] - self.normalization[1]) / self.normalization[0]
-        
-        # apply offset
-        if not self.properties['normalized']:
-            minXY[0] += self.properties['xOffset']
-            minXY[1] += self.properties['yOffset']
-            maxXY[0] += self.properties['xOffset']
-            maxXY[1] += self.properties['yOffset']
-        
-        # apply flipping
-        if self.properties['flipped']:
-            minY = -1 * maxXY[1]
-            maxY = -1 * minXY[1]
-            minXY[1] = minY
-            maxXY[1] = maxY
-        
-        return [minXY, maxXY]
-    # ----
-    
-    
     def _cropSpectrumPoints(self, minX, maxX):
         """Crop spectrum points to current X range."""
         
@@ -1120,25 +1471,6 @@ class spectrum:
                 
             # get highest intensity
             maxY = min(intens, maxY)
-        
-        # set dc for legend
-        pencolour = [max(i-70,0) for i in self.properties['spectrumColour']]
-        pen = wx.Pen(pencolour, 1*printerScale['drawings'], wx.SOLID)
-        dc.SetPen(pen)
-        dc.SetTextForeground(self.properties['spectrumColour'])
-        dc.SetBrush(wx.Brush(self.properties['spectrumColour'], wx.SOLID))
-        
-        # draw legend circle
-        x = plotX2 - 9 * printerScale['drawings']
-        y = gelY1 + (gelHeight)/2
-        dc.DrawCircle(x, y, 3*printerScale['drawings'])
-        
-        # draw legend text
-        if self.properties['showGelLegend'] and self.properties['legend']:
-            textSize = dc.GetTextExtent(self.properties['legend'])
-            x = plotX2 - textSize[0] - 17*printerScale['drawings']
-            y = gelY1 + gelHeight/2 - textSize[1]/2
-            dc.DrawText(self.properties['legend'], x, y)
     # ----
     
     
@@ -1270,13 +1602,32 @@ class spectrum:
                 
             # get highest intensity
             maxY = min(intens, maxY)
+    # ----
+    
+    
+    def _drawGelLegend(self, dc, gelCoords, gelHeight, printerScale):
+        """docstring for _drawGelLegend"""
         
-        # set dc for legend
-        pencolour = [max(i-70,0) for i in self.properties['tickColour']]
+        # get plot coordinates
+        gelY1, plotX1, plotY1, plotX2, plotY2, zeroY = gelCoords
+        
+        # get colour
+        if len(self.spectrumPoints) and self.properties['showSpectrum']:
+            colour = self.properties['spectrumColour']
+        else:
+            colour = self.properties['tickColour']
+        
+        # set dc
+        pencolour = [max(i-70,0) for i in colour]
         pen = wx.Pen(pencolour, 1*printerScale['drawings'], wx.SOLID)
         dc.SetPen(pen)
-        dc.SetTextForeground(self.properties['tickColour'])
-        dc.SetBrush(wx.Brush(self.properties['tickColour'], wx.SOLID))
+        dc.SetTextForeground(colour)
+        dc.SetBrush(wx.Brush(colour, wx.SOLID))
+        
+        # draw legend circle
+        x = plotX2 - 9 * printerScale['drawings']
+        y = gelY1 + (gelHeight)/2
+        dc.DrawCircle(x, y, 3*printerScale['drawings'])
         
         # draw legend text
         if self.properties['showGelLegend'] and self.properties['legend']:
@@ -1284,15 +1635,10 @@ class spectrum:
             x = plotX2 - textSize[0] - 17*printerScale['drawings']
             y = gelY1 + gelHeight/2 - textSize[1]/2
             dc.DrawText(self.properties['legend'], x, y)
-        
-        # draw legend circle
-        x = plotX2 - 9 * printerScale['drawings']
-        y = gelY1 + gelHeight/2
-        dc.DrawCircle(x, y, 3*printerScale['drawings'])
     # ----
     
     
-    def _mormalization(self):
+    def _normalization(self):
         """Calculate normalization constants."""
         
         scale = 1
@@ -1300,12 +1646,10 @@ class spectrum:
         
         # get range
         if len(self.spectrumPoints):
-            spectrumMin = numpy.minimum.reduce(self.spectrumPoints)
-            spectrumMax = numpy.maximum.reduce(self.spectrumPoints)
+            spectrumMin, spectrumMax = self.spectrumBox
         
         if len(self.peaklistPoints):
-            peaklistMin = numpy.minimum.reduce(self.peaklistPoints)
-            peaklistMax = numpy.maximum.reduce(self.peaklistPoints)
+            peaklistMin, peaklistMax = self.peaklistBox
         
         # calculate scale and shift
         if len(self.spectrumPoints) and len(self.peaklistPoints):
@@ -1329,6 +1673,7 @@ class spectrum:
     
 
 
+
 # UTILITIES
 # ---------
 
@@ -1358,23 +1703,40 @@ def _scaleFont(font, scale):
 # ----
 
 
-def _checkFreeSpace(coords, occupied):
-    """Check free space for label."""
+def _interpolateLine(p1, p2, x=None, y=None):
+    """Get line interpolated X or Y value."""
     
-    curX1, curY1, curX2, curY2 = coords
+    # check points
+    if p1[0] == p2[0] and x!=None:
+        return max(p1[1], p2[1])
+    elif p1[0] == p2[0] and y!=None:
+        return p1[0]
     
-    # check occupied space
-    for occ in occupied:
-        occX1, occY1, occX2, occY2 = occ
-        if (curX1 < curX2) and ((occX1 <= curX1 <= occX2) or (occX1 <= curX2 <= occX2)):
-            if (occY2 <= curY1 <= occY1) or (occY2 <= curY2 <= occY1):
-                return False
-        elif (curX1 > curX2) and ((occX2 <= curX1 <= occX1) or (occX2 <= curX2 <= occX1)):
-            if (occY1 <= curY1 <= occY2) or (occY1 <= curY2 <= occY2):
-                return False
+    # get equation
+    m = (p2[1] - p1[1])/(p2[0] - p1[0])
+    b = p1[1] - m * p1[0]
     
-    return True
+    # get point
+    if x != None:
+        return m * x + b
+    elif y != None:
+        return (y - b) / m
 # ----
 
 
+def _getIndex(points, x):
+    """Get nearest index for selected point."""
+    
+    lo = 0
+    hi = len(points)
+    while lo < hi:
+        mid = (lo + hi) / 2
+        if x < points[mid][0]:
+            hi = mid
+        else:
+            lo = mid + 1
+        
+    return lo
+# ----
+
 
